@@ -29,7 +29,6 @@ EXCLUDE = (
     "cmd/gvmap/make_map.c",
     "cmd/gvmap/make_map.h",
     "cmd/gvmap/power.c",
-    "cmd/gvpr/gvprmain.c",
     "cmd/mingle/minglemain.cpp",
     "cmd/smyrna/arcball.c",
     "cmd/smyrna/arcball.h",
@@ -333,7 +332,6 @@ EXCLUDE = (
     "lib/neatogen/call_tri.h",
     "lib/neatogen/circuit.c",
     "lib/neatogen/closest.c",
-    "lib/neatogen/closest.h",
     "lib/neatogen/compute_hierarchy.c",
     "lib/neatogen/conjgrad.c",
     "lib/neatogen/conjgrad.h",
@@ -345,11 +343,9 @@ EXCLUDE = (
     "lib/neatogen/delaunay.h",
     "lib/neatogen/digcola.h",
     "lib/neatogen/dijkstra.c",
-    "lib/neatogen/dijkstra.h",
     "lib/neatogen/edges.c",
     "lib/neatogen/edges.h",
     "lib/neatogen/embed_graph.c",
-    "lib/neatogen/embed_graph.h",
     "lib/neatogen/fPQ.h",
     "lib/neatogen/geometry.c",
     "lib/neatogen/geometry.h",
@@ -376,7 +372,6 @@ EXCLUDE = (
     "lib/neatogen/overlap.c",
     "lib/neatogen/overlap.h",
     "lib/neatogen/pca.c",
-    "lib/neatogen/pca.h",
     "lib/neatogen/poly.c",
     "lib/neatogen/poly.h",
     "lib/neatogen/quad_prog_solve.c",
@@ -387,7 +382,6 @@ EXCLUDE = (
     "lib/neatogen/randomkit.h",
     "lib/neatogen/sgd.h",
     "lib/neatogen/site.c",
-    "lib/neatogen/site.h",
     "lib/neatogen/smart_ini_x.c",
     "lib/neatogen/sparsegraph.h",
     "lib/neatogen/stress.c",
