@@ -1,5 +1,10 @@
 #!/usr/bin/env bash
 
+if [ -z ${CI+x} ]; then
+  echo "this script is only intended to run in CI" >&2
+  exit 1
+fi
+
 set -e
 set -o pipefail
 set -u
@@ -12,33 +17,18 @@ else
     cat /etc/os-release
     . /etc/os-release
 fi
-GV_VERSION=$( cat GRAPHVIZ_VERSION )
+GV_VERSION=$(python3 gen_version.py)
 DIR=Packages/${ID}/${VERSION_ID}
 ARCH=$( uname -m )
 
-# Remove possible old version of Graphviz so that we can detect if
-# individual Graphviz tools are missing in the current release
-if [ "${ID}" = "centos" -o "${ID}" = "fedora" ]; then
-    # Both doxgen and asciidoc depend on graphviz and needs to be
-    # removed to allow Graphviz to be removed
-    if rpm -q doxygen; then
-        rpm --erase doxygen
-    fi
-    if rpm -q asciidoc; then
-        rpm --erase asciidoc
-    fi
-    if rpm -q graphviz; then
-        rpm --erase graphviz
-    fi
-fi
 if [ "${build_system}" = "cmake" ]; then
     if [ "${ID_LIKE:-}" = "debian" ]; then
         apt install ./${DIR}/graphviz-${GV_VERSION}-cmake.deb
     elif [ "${ID}" = "Darwin" ]; then
         unzip ${DIR}/Graphviz-${GV_VERSION}-Darwin.zip
-        cp -rp Graphviz-${GV_VERSION}-Darwin/* /usr/local
+        sudo cp -rp Graphviz-${GV_VERSION}-Darwin/* /usr/local
     else
-        rpm --install --force ${DIR}/graphviz-${GV_VERSION}-cmake.rpm
+        rpm --install --force -vv ${DIR}/graphviz-${GV_VERSION}-cmake.rpm
     fi
 else
     if [ "${ID_LIKE:-}" = "debian" ]; then
@@ -46,11 +36,14 @@ else
         apt install ./libgraphviz4_${GV_VERSION}-1_amd64.deb
         apt install ./libgraphviz-dev_${GV_VERSION}-1_amd64.deb
         apt install ./graphviz_${GV_VERSION}-1_amd64.deb
+        apt install ./libgv-python_${GV_VERSION}-1_amd64.deb
+        apt install ./libgv-ruby_${GV_VERSION}-1_amd64.deb
+        apt install ./libgv-tcl_${GV_VERSION}-1_amd64.deb
     elif [ "${ID}" = "Darwin" ]; then
         tar xf ${DIR}/graphviz-${GV_VERSION}-${ARCH}.tar.gz
-        cp -rp build/* /usr/local
+        sudo cp -rp build/* /usr/local
     else
         tar xvf ${DIR}/graphviz-${GV_VERSION}-rpms.tar.xz
-        rpm --install --force graphviz-*${GV_VERSION}*.rpm
+        rpm --install --force -vv graphviz-*${GV_VERSION}*.rpm
     fi
 fi
