@@ -1,5 +1,10 @@
 #!/usr/bin/env bash
 
+if [ -z ${CI+x} ]; then
+  echo "this script is only intended to run in CI" >&2
+  exit 1
+fi
+
 set -e
 set -o pipefail
 set -u
