@@ -1,5 +1,10 @@
 #!/usr/bin/env bash
 
+if [ -z ${CI+x} ]; then
+  echo "this script is only intended to run in CI" >&2
+  exit 1
+fi
+
 set -e
 set -o pipefail
 set -u
@@ -29,5 +34,4 @@ elif [ "${ID_LIKE:-}" = "debian" ]; then
   export TCLLIBPATH=/usr/lib/tcltk/graphviz/tcl
 fi
 
-export GV_VERSION=$( cat GRAPHVIZ_VERSION )
 python3 -m pytest -m "not slow" --junit-xml=report.xml ci/tests.py tests
