@@ -13,6 +13,7 @@ import pytest
 
 sys.path.append(os.path.join(os.path.dirname(__file__), "../tests"))
 from gvtest import (  # pylint: disable=wrong-import-position
+    ROOT,
     build_system,
     dot,
     freedesktop_os_release,
@@ -20,6 +21,7 @@ from gvtest import (  # pylint: disable=wrong-import-position
     is_macos,
     is_mingw,
     is_static_build,
+    run,
     which,
 )
 
@@ -76,11 +78,10 @@ def test_existence(binary: str):
 
     os_id = freedesktop_os_release().get("ID")
 
-    # FIXME: Remove skip when
-    # https://gitlab.com/graphviz/graphviz/-/issues/1834 is fixed
+    # https://gitlab.com/graphviz/graphviz/-/issues/1834
     if os_id == "rocky" and binary == "smyrna":
         check_that_tool_does_not_exist(binary, os_id)
-        pytest.skip("smyrna is not built for Rocky (#1834)")
+        pytest.skip("smyrna is not built for Rocky due to lacking libgts")
 
     if os_id == "rocky" and binary == "mingle":
         check_that_tool_does_not_exist(binary, os_id)
@@ -94,8 +95,8 @@ def test_existence(binary: str):
         check_that_tool_does_not_exist(binary, os_id)
         pytest.skip(f"{binary} is not built on Windows due to lacking Qt")
 
-    # FIXME: Smyrna dependencies are not avaiable in other jobs
-    if binary == "smyrna" and is_cmake() and platform.system() != "Linux":
+    # Smyrna dependencies are not avaiable in other jobs
+    if binary == "smyrna" and platform.system() != "Linux":
         check_that_tool_does_not_exist(binary, os_id)
         pytest.skip("smyrna is not built on non-Linux due to lacking dependencies")
 
@@ -106,10 +107,6 @@ def test_existence(binary: str):
         check_that_tool_does_not_exist(binary, os_id)
         pytest.skip(f"{binary} is not installed on Windows")
 
-    if binary == "smyrna" and not is_cmake() and is_macos():
-        check_that_tool_does_not_exist(binary, os_id)
-        pytest.skip("https://gitlab.com/graphviz/graphviz/-/issues/2422")
-
     if binary == "vimdot" and not is_cmake() and is_macos():
         check_that_tool_does_not_exist(binary, os_id)
         pytest.skip("https://gitlab.com/graphviz/graphviz/-/issues/2423")
@@ -155,16 +152,9 @@ def test_installation():
     check that Graphviz reports the expected version number
     """
 
-    expected_version = os.environ["GV_VERSION"]
+    expected_version = run([sys.executable, ROOT / "gen_version.py"]).strip()
 
-    actual_version_string = subprocess.check_output(
-        [
-            "dot",
-            "-V",
-        ],
-        universal_newlines=True,
-        stderr=subprocess.STDOUT,
-    )
+    actual_version_string = run(["dot", "-V"], stderr=subprocess.STDOUT)
     try:
         actual_version = actual_version_string.split()[4]
     except IndexError:
