@@ -8,14 +8,13 @@
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
-#include <cgraph/exit.h>
-
 #include "glutrender.h"
 #include "viewport.h"
 #include "arcball.h"
-#include "appmouse.h"
+#include "gui/appmouse.h"
 #include "glexpose.h"
-#include "glcompui.h"
+#include "gui/glcompui.h"
+#include <util/exit.h>
 
     /*call backs */
 
@@ -55,7 +54,7 @@ static void cb_reshape(int width, int height)
     /* setup various opengl things that we need */
     glMatrixMode(GL_PROJECTION);
     glLoadIdentity();
-    init_arcBall(view->arcball, (float)view->w, (float)view->h);
+    init_arcBall(view->arcball, view->w, view->h);
     if (view->w > view->h) {
 	aspect = (float) view->w / (float) view->h;
 	glOrtho(-aspect * GL_VIEWPORT_FACTOR, aspect * GL_VIEWPORT_FACTOR,
@@ -95,7 +94,9 @@ static void cb_mouseclick(int button, int state, int x, int y)
     begin_y = (float) y;
     if(state==GLUT_DOWN)
     {
-	view->widgets->common.functions.mousedown((glCompObj*)view->widgets, (float)x, (float)y, getGlCompMouseType(button));
+	view->widgets->base.common.functions.mousedown(&view->widgets->base,
+	                                               (float)x, (float)y,
+	                                               getGlCompMouseType(button));
 	if (button ==  GLUT_LEFT_BUTTON)
 	    appmouse_left_click_down(view,x,y);
         if (button ==  GLUT_RIGHT_BUTTON)
@@ -106,14 +107,13 @@ static void cb_mouseclick(int button, int state, int x, int y)
     else
     {
 	view->arcball->isDragging = 0;
-	view->widgets->common.functions.mouseup((glCompObj*)view->widgets, (float)x, (float)y, getGlCompMouseType(button));
+	view->widgets->base.common.functions.mouseup(&view->widgets->base,
+	                                             (float)x, (float)y,
+	                                             getGlCompMouseType(button));
 
-	if (button ==  GLUT_LEFT_BUTTON)
-	    appmouse_left_click_up(view,x,y);
-	if (button ==  GLUT_LEFT_BUTTON)
-	    appmouse_right_click_up(view,x,y);
-	if (button ==  GLUT_MIDDLE_BUTTON)
-	    appmouse_middle_click_up(view,x,y);
+	if (button == GLUT_LEFT_BUTTON || button == GLUT_RIGHT_BUTTON ||
+	    button == GLUT_MIDDLE_BUTTON)
+	    appmouse_up(view, x, y);
 	dx = 0.0;
 	dy = 0.0;
     }
@@ -129,7 +129,7 @@ static void cb_drag(int X, int Y)
     float y = (float) Y;
 
     if (view->widgets)
-	view->widgets->common.functions.mouseover((glCompObj*)view->widgets, x, y);
+	view->widgets->base.common.functions.mouseover(&view->widgets->base, x, y);
 
     dx = x - begin_x;
     dy = y - begin_y;
@@ -137,12 +137,8 @@ static void cb_drag(int X, int Y)
     view->mouse.dragY = dy;
     appmouse_move(view,x,y);
 
-    if((view->mouse.t==glMouseLeftButton) && (view->mouse.down)  )
-	appmouse_left_drag(view,x,y);
-    if((view->mouse.t==glMouseRightButton) && (view->mouse.down))
-	appmouse_right_drag(view,x,y);
-    if((view->mouse.t==glMouseMiddleButton) && (view->mouse.down))
-	appmouse_middle_drag(view,x,y);
+    if (view->mouse.down)
+	appmouse_drag(view, x, y);
     begin_x = x;
     begin_y = y;
     cb_display();
