@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -20,12 +20,14 @@
 #include "frmobjectui.h"
 #include <assert.h>
 #include "gvprpipe.h"
-#include <cgraph/agxbuf.h>
-#include <cgraph/alloc.h>
-#include <cgraph/strcasecmp.h>
-#include <cgraph/strview.h>
 #include <stdint.h>
 #include <string.h>
+#include <util/agxbuf.h>
+#include <util/alloc.h>
+#include <util/strcasecmp.h>
+#include <util/startswith.h>
+#include <util/strview.h>
+#include <util/unreachable.h>
 
 static int sel_node;
 static int sel_edge;
@@ -58,7 +60,7 @@ static attr_t *new_attr(void) {
 static attr_t *new_attr_with_ref(Agsym_t * sym)
 {
     attr_t *a = new_attr();
-    a->name = safestrdup(sym->name);
+    a->name = gv_strdup(sym->name);
     switch (sym->kind) {
     case AGRAPH:
 	a->objType[0] = 1;
@@ -72,6 +74,8 @@ static attr_t *new_attr_with_ref(Agsym_t * sym)
 	a->objType[2] = 1;
 	a->defValE = safestrdup(sym->defval);
 	break;
+    default:
+	UNREACHABLE();
     }
     return a;
 }
@@ -83,7 +87,7 @@ static attr_t *new_attr_ref(attr_t * refAttr)
     a->defValG = safestrdup(refAttr->defValG);
     a->defValN = safestrdup(refAttr->defValN);
     a->defValE = safestrdup(refAttr->defValE);
-    a->name = safestrdup(refAttr->name);
+    a->name = gv_strdup(refAttr->name);
     a->value = safestrdup(refAttr->value);
     return a;
 }
@@ -103,7 +107,6 @@ static void reset_attr_list_widgets(attr_list * l)
 static attr_list *attr_list_new(bool with_widgets) {
     int id;
     attr_list *l = gv_alloc(sizeof(attr_list));
-    l->with_widgets = with_widgets;
     /*create filter widgets */
 
     if (with_widgets) {
@@ -126,18 +129,21 @@ static attr_list *attr_list_new(bool with_widgets) {
     return l;
 }
 
-static int attr_compare(const void *a, const void *b)
-{
-    const attr_t *a1 = *(attr_t *const *) a;
-    const attr_t *a2 = *(attr_t *const *) b;
-    return strcasecmp(a1->name, a2->name);
+static int attr_compare_core(const void *key, const void *candidate) {
+  const char *k = key;
+  const attr_t *const *c = candidate;
+  return strcasecmp(k, (*c)->name);
+}
+
+static int attr_compare(const attr_t **a, const attr_t **b) {
+  return attr_compare_core((*a)->name, b);
 }
 
 static void attr_list_add(attr_list *l, attr_t *a) {
     if (!l || !a)
 	return;
     attrs_append(&l->attributes, a);
-    attrs_sort(&l->attributes, (int(*)(const attr_t**, const attr_t**))attr_compare);
+    attrs_sort(&l->attributes, attr_compare);
     /*update indices */
     for (size_t id = 0; id < attrs_size(&l->attributes); ++id)
 	attrs_get(&l->attributes, id)->index = id;
@@ -158,6 +164,8 @@ static attr_data_type get_attr_data_type(char c)
     case 'I':
 	return attr_int;
 	break;
+    default:
+	break;
     }
     return attr_alpha;
 }
@@ -183,7 +191,7 @@ static void set_attr_object_type(const char *str, int *t) {
   object_type_helper(a, t);
   while (true) {
     const char *start = a.data + a.size;
-    if (strncmp(start, " or ", strlen(" or ")) != 0) {
+    if (!startswith(start, " or ")) {
       break;
     }
     start += strlen(" or ");
@@ -197,59 +205,41 @@ static void set_attr_object_type(const char *str, int *t) {
   }
 }
 
-static attr_t *binarySearch(attr_list * l, char *searchKey)
-{
-  const attr_t key = {.name = searchKey};
-  const attr_t *keyp = &key;
-  attr_t **attrp = bsearch(&keyp, attrs_at(&l->attributes, 0),
+static attr_t *binarySearch(attr_list *l, const char *searchKey) {
+  attrs_sync(&l->attributes);
+  attr_t **attrp = bsearch(searchKey, attrs_front(&l->attributes),
                            attrs_size(&l->attributes), sizeof(attr_t*),
-                           attr_compare);
+                           attr_compare_core);
   if (attrp != NULL) {
     return *attrp;
   }
   return NULL;
 }
 
-static attr_t *pBinarySearch(attr_list *l, const char *searchKey) {
-    size_t low = 0;
-    size_t high = attrs_size(&l->attributes) - 1;
-
-    while (high != SIZE_MAX && low <= high) {
-	size_t middle = (low + high) / 2;
-	int res = strncasecmp(searchKey, attrs_get(&l->attributes, middle)->name, strlen(searchKey));
-	if (res == 0) {
-	    return attrs_get(&l->attributes, middle);
-	}
-	else if (res < 0) {
-	    high = middle - 1;
-	} else {
-	    low = middle + 1;
-	}
-    }
-    return NULL;
+static int cmp(const void *key, const void *candidate) {
+  const attr_t *const a = candidate;
+  return strncasecmp(key, a->name, strlen(key));
 }
 
 static void create_filtered_list(const char *prefix, attr_list *sl,
                                  attr_list *tl) {
-    int res;
-    attr_t *at;
     int objKind = get_object_type();
 
     if (strlen(prefix) == 0)
 	return;
     /*locate first occurrence */
-    at = pBinarySearch(sl, prefix);
+    attrs_sync(&sl->attributes);
+    attr_t *at = bsearch(prefix, attrs_front(&sl->attributes),
+                         attrs_size(&sl->attributes), sizeof(attr_t), cmp);
     if (!at)
 	return;
 
-    res = 0;
     /*go backward to get the first */
-    while (at->index > 0 && res == 0) {
+    for (int res = 0; at->index > 0 && res == 0; ) {
 	at = attrs_get(&sl->attributes, at->index - 1);
 	res = strncasecmp(prefix, at->name, strlen(prefix));
     }
-    res = 0;
-    while (at->index < attrs_size(&sl->attributes) && res == 0) {
+    for (int res = 0; at->index < attrs_size(&sl->attributes) && res == 0; ) {
 	at = attrs_get(&sl->attributes, at->index + 1);
 	res = strncasecmp(prefix, at->name, strlen(prefix));
 	if (res == 0 && at->objType[objKind] == 1)
@@ -344,22 +334,11 @@ static void filter_attributes(const char *prefix, topview *t) {
 
 // attribute text changed call back
 
-_BB void on_txtAttr_changed(GtkWidget * widget, gpointer user_data)
-{
+_BB void on_txtAttr_changed(GtkWidget *widget, void *user_data) {
   (void)user_data;
   filter_attributes(gtk_entry_get_text((GtkEntry*)widget), view->Topview);
 }
 
-static void set_refresh_filters(ViewInfo * v, char *name)
-{
-    if (strcasecmp(name, "pos") == 0)
-	v->refresh.pos = 1;
-    if (strcasecmp(name, "color") == 0)
-	v->refresh.color = 1;
-    if (strcasecmp(name, "selected") == 0)
-	v->refresh.selection = 1;
-}
-
 static void doApply(void)
 {
     char *attr_name;
@@ -389,9 +368,9 @@ static void doApply(void)
     attr = binarySearch(view->Topview->attributes, attr_name);
     assert(attr);
     attr->propagate = prog;
-    sym = agattr(g, objKind, attr_name, NULL);
+    sym = agattr_text(g, objKind, attr_name, NULL);
     if (!sym)			/*it shouldnt be null, just in case it is null */
-	sym = agattr(g, objKind, attr_name, def_val);
+	sym = agattr_text(g, objKind, attr_name, def_val);
     /*graph */
     if (objKind == AGRAPH)
 	agset(g, attr_name, value);
@@ -414,19 +393,16 @@ static void doApply(void)
 	fprintf(stderr,
 	    "on_attrApplyBtn_clicked: unknown object kind %d\n",
 	    objKind);
-    set_refresh_filters(view, attr_name);
 }
 
-_BB void on_attrApplyBtn_clicked(GtkWidget * widget, gpointer user_data)
-{
+_BB void on_attrApplyBtn_clicked(GtkWidget *widget, void *user_data) {
     (void)widget;
     (void)user_data;
 
     doApply();
 }
 
-_BB void on_attrRB0_clicked(GtkWidget * widget, gpointer user_data)
-{
+_BB void on_attrRB0_clicked(GtkWidget *widget, void *user_data) {
     (void)widget;
     (void)user_data;
 
@@ -438,15 +414,13 @@ _BB void on_attrRB0_clicked(GtkWidget * widget, gpointer user_data)
 /* This is the action attached to the publish button on the attributes
  * window. What should happen?
  */
-_BB void on_attrProg_toggled(GtkWidget * widget, gpointer user_data)
-{
+_BB void on_attrProg_toggled(GtkWidget *widget, void *user_data) {
   (void)widget;
   (void)user_data;
   /* FIX */
 }
 
-_BB void on_attrAddBtn_clicked(GtkWidget * widget, gpointer user_data)
-{
+_BB void on_attrAddBtn_clicked(GtkWidget *widget, void *user_data) {
     (void)widget;
     (void)user_data;
 
@@ -480,18 +454,18 @@ _BB void on_attrAddBtn_clicked(GtkWidget * widget, gpointer user_data)
     attr->propagate = 0;
 
     if (objKind == AGRAPH) {
-	agattr(g, AGRAPH, attr_name, defValue);
+	agattr_text(g, AGRAPH, attr_name, defValue);
 	attr->defValG = safestrdup(defValue);
 	attr->objType[0] = 1;
     }
 
     /*nodes */
     else if (objKind == AGNODE) {
-	agattr(g, AGNODE, attr_name, defValue);
+	agattr_text(g, AGNODE, attr_name, defValue);
 	attr->defValN = safestrdup(defValue);
 	attr->objType[1] = 1;
     } else if (objKind == AGEDGE) {
-	agattr(g, AGEDGE, attr_name, defValue);
+	agattr_text(g, AGEDGE, attr_name, defValue);
 	attr->defValE = safestrdup(defValue);
 	attr->objType[2] = 1;
     } else
@@ -525,16 +499,18 @@ attr_list *load_attr_list(Agraph_t * g)
 
 		switch (idx) {
 		case 0:
-		    attr->name = safestrdup(a);
+		    attr->name = gv_strdup(a);
 		    break;
 		case 1:
-		    attr->defValG = safestrdup(a);
-		    attr->defValN = safestrdup(a);
-		    attr->defValE = safestrdup(a);
+		    attr->defValG = gv_strdup(a);
+		    attr->defValN = gv_strdup(a);
+		    attr->defValE = gv_strdup(a);
 		    break;
 		case 2:
 		    set_attr_object_type(a, attr->objType);
 		    break;
+		default:
+		    break;
 		}
 	    }
 	    attr_list_add(l, attr);
@@ -579,15 +555,12 @@ static void set_header_text(void)
     int nodeCnt = 0;
     int edgeCnt = 0;
     char buf[512];
-    Agedge_t* ep;
-    Agnode_t* v;
-    Agraph_t* g;
 
-    g = view->g[view->activeGraph];
-    for (v = agfstnode(g); v; v = agnxtnode(g, v)) {
+    Agraph_t *g = view->g[view->activeGraph];
+    for (Agnode_t *v = agfstnode(g); v; v = agnxtnode(g, v)) {
 	if (ND_selected(v))
 	    nodeCnt++;
-	for (ep = agfstout(g, v); ep; ep = agnxtout(g, ep)) {
+	for (Agedge_t *ep = agfstout(g, v); ep; ep = agnxtout(g, ep)) {
 	    if (ND_selected(v))
 		edgeCnt++;
 	}
@@ -606,6 +579,15 @@ static void set_header_text(void)
 }
 
 void showAttrsWidget(void) {
+    if (view->activeGraph < 0) {
+	void *const dlg = gtk_message_dialog_new(NULL, GTK_DIALOG_MODAL,
+	                                         GTK_MESSAGE_QUESTION, GTK_BUTTONS_OK,
+	                                         "No active graph");
+	(void)gtk_dialog_run(dlg);
+	gtk_widget_destroy(dlg);
+	return;
+    }
+
     gtk_widget_hide(glade_xml_get_widget(xml, "dlgSettings"));
     gtk_widget_show(glade_xml_get_widget(xml, "dlgSettings"));
     gtk_notebook_set_current_page((GtkNotebook *)
@@ -644,8 +626,7 @@ static void gvpr_select(const char *attrname, const char *regex_str,
     set_header_text();
 }
 
-_BB void on_attrSearchBtn_clicked(GtkWidget * widget, gpointer user_data)
-{
+_BB void on_attrSearchBtn_clicked(GtkWidget *widget, void *user_data) {
     (void)widget;
     (void)user_data;
 
