@@ -8,7 +8,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -29,8 +29,13 @@
 
 #include <stdlib.h>
 #include <cgraph/cgraph.h>
-#include <cgraph/exit.h>
-#include <cgraph/stack.h>
+#include <cgraph/ingraphs.h>
+#include <util/agxbuf.h>
+#include <util/alloc.h>
+#include <util/exit.h>
+#include <util/gv_math.h>
+#include <util/list.h>
+#include <util/unreachable.h>
 
 typedef struct {
     Agrec_t h;
@@ -53,10 +58,6 @@ typedef struct {
 #define N(n)  (((Agnodeinfo_t*)(n->base.data))->val)
 #define NEXTBLK(g)  (((Agraphinfo_t*)(g->base.data))->next)
 
-#include <ingraphs/ingraphs.h>
-
-#define min(a,b) ((a) < (b) ? (a) :  (b))
-
 char **Files;
 int verbose;
 int silent;
@@ -66,29 +67,21 @@ char *suffix = 0;
 int external;			/* emit blocks as root graphs */
 int doTree;			/* emit block-cutpoint tree */
 
+DEFINE_LIST(edge_stack, Agedge_t *)
+
 typedef struct {
     int count;
     int nComp;
-    gv_stack_t stk;
+    edge_stack_t stk;
     Agraph_t *blks;
 } bcstate;
 
-static char *blockName(char *gname, int d)
-{
-    static char *buf;
-    static size_t bufsz;
-
-    size_t sz = strlen(gname) + 128;
-    if (sz > bufsz) {
-	free(buf);
-	buf = malloc(sz);
-    }
-
+static char *blockName(agxbuf *xb, char *gname, int d) {
     if (*gname == '%') /* anonymous graph */
-	sprintf(buf, "_%s_bcc_%d", gname, d);
+	agxbprint(xb, "_%s_bcc_%d", gname, d);
     else
-	sprintf(buf, "%s_bcc_%d", gname, d);
-    return buf;
+	agxbprint(xb, "%s_bcc_%d", gname, d);
+    return agxbuse(xb);
 }
 
 /* getName:
@@ -99,30 +92,24 @@ static char *blockName(char *gname, int d)
  */
 static char *getName(int ng, int nb)
 {
-    char *name;
-    static char *buf;
+    agxbuf name = {0};
 
     if (ng == 0 && nb == 0)
-	name = outfile;
+	agxbput(&name, outfile);
     else {
-	if (!buf) {
-	    size_t sz = strlen(outfile) + 100; // enough to handle '_<g>_<b>'
-	    buf = malloc(sz);
-	}
 	if (suffix) {
 	    if (nb < 0)
-		sprintf(buf, "%s_%d_T.%s", path, ng, suffix);
+		agxbprint(&name, "%s_%d_T.%s", path, ng, suffix);
 	    else
-		sprintf(buf, "%s_%d_%d.%s", path, ng, nb, suffix);
+		agxbprint(&name, "%s_%d_%d.%s", path, ng, nb, suffix);
 	} else {
 	    if (nb < 0)
-		sprintf(buf, "%s_%d_T", path, ng);
+		agxbprint(&name, "%s_%d_T", path, ng);
 	    else
-		sprintf(buf, "%s_%d_%d", path, ng, nb);
+		agxbprint(&name, "%s_%d_%d", path, ng, nb);
 	}
-	name = buf;
     }
-    return name;
+    return agxbdisown(&name);
 }
 
 static void gwrite(Agraph_t * g, int ng, int nb)
@@ -141,8 +128,10 @@ static void gwrite(Agraph_t * g, int ng, int nb)
 	if (!outf) {
 	    fprintf(stderr, "Could not open %s for writing\n", name);
 	    perror("bcomps");
+	    free(name);
 	    graphviz_exit(1);
 	}
+	free(name);
 	agwrite(g, outf);
 	fclose(outf);
     }
@@ -153,7 +142,9 @@ static Agraph_t *mkBlock(Agraph_t * g, bcstate * stp)
     Agraph_t *sg;
 
     stp->nComp++;
-    sg = agsubg(g, blockName(agnameof(g), stp->nComp), 1);
+    agxbuf xb = {0};
+    sg = agsubg(g, blockName(&xb, agnameof(g), stp->nComp), 1);
+    agxbfree(&xb);
     agbindrec(sg, "info", sizeof(Agraphinfo_t), true);
     NEXTBLK(sg) = stp->blks;
     stp->blks = sg;
@@ -176,36 +167,22 @@ dfs(Agraph_t * g, Agnode_t * u, bcstate * stp, Agnode_t * parent)
 	if (v == u)
 	    continue;
 	if (N(v) == 0) {
-	    stack_push_or_exit(&stp->stk, e);
+	    edge_stack_push_back(&stp->stk, e);
 	    dfs(g, v, stp, u);
-	    Low(u) = min(Low(u), Low(v));
+	    Low(u) = imin(Low(u), Low(v));
 	    if (Low(v) >= N(u)) {	/* u is an articulation point */
 		Cut(u) = 1;
 		sg = mkBlock(g, stp);
 		do {
-		    ep = stack_pop(&stp->stk);
+		    ep = edge_stack_pop_back(&stp->stk);
 		    agsubnode(sg, aghead(ep), 1);
 		    agsubnode(sg, agtail(ep), 1);
 		} while (ep != e);
 	    }
 	} else if (parent != v) {
-	    Low(u) = min(Low(u), N(v));
+	    Low(u) = imin(Low(u), N(v));
 	    if (N(v) < N(u))
-		stack_push_or_exit(&stp->stk, e);
-	}
-    }
-}
-
-static void nodeInduce(Agraph_t * g, Agraph_t * eg)
-{
-    Agnode_t *n;
-    Agedge_t *e;
-
-    for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
-	for (e = agfstout(eg, n); e; e = agnxtout(eg, e)) {
-	    if (agsubnode(g, aghead(e), 0)) {
-		agsubedge(g, e, 1);
-	    }
+		edge_stack_push_back(&stp->stk, e);
 	}
     }
 }
@@ -233,13 +210,13 @@ static int process(Agraph_t * g, int gcnt)
     Agraph_t *tree;
     int bcnt;
 
-    aginit(g, AGNODE, "info", sizeof(Agnodeinfo_t), TRUE);
-    aginit(g, AGEDGE, "info", sizeof(Agedgeinfo_t), TRUE);
-    aginit(g, AGRAPH, "info", sizeof(Agraphinfo_t), TRUE);
+    aginit(g, AGNODE, "info", sizeof(Agnodeinfo_t), true);
+    aginit(g, AGEDGE, "info", sizeof(Agedgeinfo_t), true);
+    aginit(g, AGRAPH, "info", sizeof(Agraphinfo_t), true);
 
     state.count = 0;
     state.nComp = 0;
-    state.stk = (gv_stack_t){0};
+    state.stk = (edge_stack_t){0};
     state.blks = 0;
 
     for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
@@ -247,7 +224,7 @@ static int process(Agraph_t * g, int gcnt)
 	    dfs(g, n, &state, 0);
     }
     for (blk = state.blks; blk; blk = NEXTBLK(blk)) {
-	nodeInduce(blk, g);
+	(void)graphviz_node_induce(blk, g);
     }
     if (external) {
 	bcnt = 0;
@@ -274,7 +251,7 @@ static int process(Agraph_t * g, int gcnt)
 	fprintf(stderr, "%s: %d blocks %d cutpoints\n", agnameof(g), bcnt,
 		cuts);
     }
-    stack_reset(&state.stk);
+    edge_stack_free(&state.stk);
     if (state.blks && NEXTBLK(state.blks))
 	return 1;		/* >= 2 blocks */
     else
@@ -300,15 +277,12 @@ static void usage(int v)
 static void split(char *name)
 {
     char *sfx = 0;
-    int size;
 
     sfx = strrchr(name, '.');
     if (sfx) {
-	size = sfx - name;
+	size_t size = (size_t)(sfx - name);
 	suffix = sfx + 1;
-	path = malloc(size + 1);
-	strncpy(path, name, size);
-	*(path + size) = '\0';
+	path = gv_strndup(name, size);
     } else {
 	path = name;
     }
@@ -350,6 +324,8 @@ static void init(int argc, char *argv[])
 		usage(1);
 	    }
 	    break;
+	default:
+	    UNREACHABLE();
 	}
     }
     argv += optind;
@@ -359,11 +335,6 @@ static void init(int argc, char *argv[])
 	Files = argv;
 }
 
-static Agraph_t *gread(FILE * fp)
-{
-  return agread(fp, NULL);
-}
-
 int main(int argc, char *argv[])
 {
     Agraph_t *g;
@@ -372,7 +343,7 @@ int main(int argc, char *argv[])
     int gcnt = 0;
 
     init(argc, argv);
-    newIngraph(&ig, Files, gread);
+    newIngraph(&ig, Files);
 
     while ((g = nextGraph(&ig)) != 0) {
 	r |= process(g, gcnt);
