@@ -16,18 +16,20 @@
 
 #include    "convert.h"
 #include    <assert.h>
-#include    <cgraph/agxbuf.h>
-#include    <cgraph/alloc.h>
-#include    <cgraph/exit.h>
-#include    <cgraph/gv_ctype.h>
-#include    <cgraph/stack.h>
-#include    <cgraph/unreachable.h>
 #include    <getopt.h>
 #include    <limits.h>
 #include    <stdbool.h>
 #include    <stdio.h>
+#include    <stdlib.h>
 #include    <string.h>
 #include    "openFile.h"
+#include    <util/agxbuf.h>
+#include    <util/alloc.h>
+#include    <util/debug.h>
+#include    <util/exit.h>
+#include    <util/gv_ctype.h>
+#include    <util/list.h>
+#include    <util/unreachable.h>
 #ifdef HAVE_EXPAT
 #include    <expat.h>
 
@@ -50,47 +52,45 @@ static char **Files;
 static int Verbose;
 static char* gname = "";
 
-static void pushString(gv_stack_t *stk, const char *s) {
+DEFINE_LIST_WITH_DTOR(strs, char *, free)
+
+static void pushString(strs_t *stk, const char *s) {
 
   // duplicate the string we will push
   char *copy = gv_strdup(s);
 
   // push this onto the stack
-  stack_push(stk, copy);
+  strs_push_back(stk, copy);
 }
 
-static void popString(gv_stack_t *stk) {
+static void popString(strs_t *stk) {
 
-  if (stack_is_empty(stk)) {
+  if (strs_is_empty(stk)) {
     fprintf(stderr, "PANIC: graphml2gv: empty element stack\n");
     graphviz_exit(EXIT_FAILURE);
   }
 
-  char *s = stack_pop(stk);
-  free(s);
+  char *const popped = strs_pop_back(stk);
+  free(popped);
 }
 
-static char *topString(gv_stack_t *stk) {
+static char *topString(strs_t *stk) {
 
-  if (stack_is_empty(stk)) {
+  if (strs_is_empty(stk)) {
     fprintf(stderr, "PANIC: graphml2gv: empty element stack\n");
     graphviz_exit(EXIT_FAILURE);
   }
 
-  return stack_top(stk);
+  return *strs_back(stk);
 }
 
-static void freeString(gv_stack_t *stk) {
-  while (!stack_is_empty(stk)) {
-    char *s = stack_pop(stk);
-    free(s);
-  }
-  stack_reset(stk);
+static void freeString(strs_t *stk) {
+  strs_free(stk);
 }
 
 typedef struct {
     char* gname;
-    gv_stack_t elements;
+    strs_t elements;
     int closedElementType;
     bool edgeinverted;
 } userdata_t;
@@ -99,11 +99,12 @@ static Agraph_t *root;		/* root graph */
 static Agraph_t *G;		/* Current graph */
 static Agedge_t *E; // current edge
 
-static gv_stack_t Gstack;
+DEFINE_LIST(graph_stack, Agraph_t *)
+static graph_stack_t Gstack;
 
 static userdata_t genUserdata(char *dfltname) {
   userdata_t user = {0};
-  user.elements = (gv_stack_t){0};
+  user.elements = (strs_t){0};
   user.closedElementType = TAG_NONE;
   user.edgeinverted = false;
   user.gname = dfltname;
@@ -125,12 +126,12 @@ static int isAnonGraph(const char *name) {
 static void push_subg(Agraph_t * g)
 {
   // save the root if this is the first graph
-  if (stack_is_empty(&Gstack)) {
+  if (graph_stack_is_empty(&Gstack)) {
     root = g;
   }
 
   // insert the new graph
-  stack_push(&Gstack, g);
+  graph_stack_push_back(&Gstack, g);
 
   // update the top graph
   G = g;
@@ -138,17 +139,17 @@ static void push_subg(Agraph_t * g)
 
 static Agraph_t *pop_subg(void)
 {
-  if (stack_is_empty(&Gstack)) {
+  if (graph_stack_is_empty(&Gstack)) {
     fprintf(stderr, "graphml2gv: Gstack underflow in graph parser\n");
     graphviz_exit(EXIT_FAILURE);
   }
 
   // pop the top graph
-  Agraph_t *g = stack_pop(&Gstack);
+  Agraph_t *g = graph_stack_pop_back(&Gstack);
 
   // update the top graph
-  if (!stack_is_empty(&Gstack)) {
-    G = stack_top(&Gstack);
+  if (!graph_stack_is_empty(&Gstack)) {
+    G = *graph_stack_back(&Gstack);
   }
 
   return g;
@@ -186,31 +187,22 @@ static char *defval = "";
 
 static void setEdgeAttr(Agedge_t *ep, char *name, const char *value,
                         userdata_t *ud) {
-    Agsym_t *ap;
-    char *attrname;
-
     if (strcmp(name, "headport") == 0) {
-	if (ud->edgeinverted)
-	    attrname = "tailport";
-	else
-	    attrname = "headport";
-	ap = agattr(root, AGEDGE, attrname, 0);
+	char *const attrname = ud->edgeinverted ? "tailport" : "headport";
+	Agsym_t *ap = agattr_text(root, AGEDGE, attrname, 0);
 	if (!ap)
-	    ap = agattr(root, AGEDGE, attrname, defval);
+	    ap = agattr_text(root, AGEDGE, attrname, defval);
 	agxset(ep, ap, value);
     } else if (strcmp(name, "tailport") == 0) {
-	if (ud->edgeinverted)
-	    attrname = "headport";
-	else
-	    attrname = "tailport";
-	ap = agattr(root, AGEDGE, attrname, 0);
+	char *const attrname = ud->edgeinverted ? "headport" : "tailport";
+	Agsym_t *ap = agattr_text(root, AGEDGE, attrname, 0);
 	if (!ap)
-	    ap = agattr(root, AGEDGE, attrname, defval);
+	    ap = agattr_text(root, AGEDGE, attrname, defval);
 	agxset(ep, ap, value);
     } else {
-	ap = agattr(root, AGEDGE, name, 0);
+	Agsym_t *ap = agattr_text(root, AGEDGE, name, 0);
 	if (!ap)
-	    ap = agattr(root, AGEDGE, name, defval);
+	    ap = agattr_text(root, AGEDGE, name, defval);
 	agxset(ep, ap, value);
     }
 }
@@ -247,16 +239,13 @@ startElementHandler(void *userData, const char *name, const char **atts)
 	    edgeMode = atts[pos];
 	}
 
-	if (stack_is_empty(&Gstack)) {
+	if (graph_stack_is_empty(&Gstack)) {
 	    if (strcmp(edgeMode, "directed") == 0) {
 		dir = Agdirected;
 	    } else if (strcmp(edgeMode, "undirected") == 0) {
 		dir = Agundirected;
 	    } else {
-		if (Verbose) {
-		    fprintf(stderr,
-			"Warning: graph has no edgedefault attribute - assume directed\n");
-		}
+		GV_INFO("Warning: graph has no edgedefault attribute - assume directed");
 		dir = Agdirected;
 	    }
 	    g = agopen((char *) id, dir, &AgDefaultDisc);
@@ -503,15 +492,14 @@ int main(int argc, char **argv)
 	    if (prev)
 		agclose(prev);
 	    prev = graph;
-	    if (Verbose) 
-		fprintf (stderr, "%s: %d nodes %d edges\n",
-		    agnameof(graph), agnnodes(graph), agnedges(graph));
+	    GV_INFO("%s: %d nodes %d edges", agnameof(graph), agnnodes(graph),
+	            agnedges(graph));
 	    agwrite(graph, outFile);
 	    fflush(outFile);
 	}
     }
 
-    stack_reset(&Gstack);
+    graph_stack_free(&Gstack);
 
     agxbfree(&buf);
     graphviz_exit(rv);
