@@ -8,7 +8,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -24,22 +24,23 @@
 
 #include <getopt.h>
 #include <algorithm>
-#include <assert.h>
+#include <cassert>
 #include <gvc/gvc.h>
-#include <cgraph/alloc.h>
-#include <cgraph/exit.h>
 #include <cgraph/ingraphs.h>
-#include <cgraph/startswith.h>
 #include <common/render.h>
+#include <common/utils.h>
 #include <neatogen/neatoprocs.h>
 #include <iostream>
 #include <limits>
 #include <map>
+#include <optional>
 #include <pack/pack.h>
 #include <set>
 #include <stddef.h>
 #include <string>
 #include <utility>
+#include <util/alloc.h>
+#include <util/exit.h>
 #include <vector>
 #include "openFile.h"
 
@@ -69,22 +70,13 @@ lt_symlist_t lt_preloaded_symbols[] = {
  */
 
 typedef struct {
-    Dtlink_t link;
     char *name;
     char *value;
-    size_t cnt;
 } attr_t;
 
-typedef struct {
-    Dtlink_t link;
-    char *name;
-    size_t cnt;
-} pair_t;
-
 static int verbose = 0;
 static char **myFiles = 0;
 static FILE *outfp;		/* output; stdout by default */
-static Agdesc_t kind;		/* type of graph */
 static std::vector<attr_t> G_args; // Storage for -G arguments
 static bool doPack;              /* Do packing if true */
 static char* gname = const_cast<char*>("root");
@@ -117,14 +109,13 @@ static void usage(int v)
  */
 static int setNameValue(char *arg)
 {
-    char *p;
     char *rhs = const_cast<char*>("true");
 
-    if ((p = strchr(arg, '='))) {
+    if (char *p = strchr(arg, '=')) {
 	*p++ = '\0';
 	rhs = p;
     }
-    G_args.push_back(attr_t{{0, {0}}, arg, rhs, 0});
+    G_args.push_back(attr_t{arg, rhs});
 
     return 0;
 }
@@ -148,19 +139,17 @@ static int setUInt(unsigned int *v, char *arg)
 }
 
 static Agsym_t *agraphattr(Agraph_t *g, char *name, const char *value) {
-    return agattr(g, AGRAPH, name, value);
+    return agattr_text(g, AGRAPH, name, value);
 }
 
 static Agsym_t *agnodeattr(Agraph_t *g, char *name, const char *value) {
-    return agattr(g, AGNODE, name, value);
+    return agattr_text(g, AGNODE, name, value);
 }
 
 static Agsym_t *agedgeattr(Agraph_t *g, char *name, const char *value) {
-    return agattr(g, AGEDGE, name, value);
+    return agattr_text(g, AGEDGE, name, value);
 }
 
-/* init:
- */
 static void init(int argc, char *argv[], pack_info* pinfo)
 {
     int c;
@@ -281,8 +270,7 @@ static void init_graph(Agraph_t *g, bool fill, GVC_t *gvc) {
     Ndim = GD_ndim(g) = 2;
     init_node_edge(g);
     if (fill) {
-	int ret = init_nop(g,0);
-	if (ret) {
+	if (int ret = init_nop(g, 0)) {
 	    if (ret < 0)
 		std::cerr << "Error loading layout info from graph " << agnameof(g) << '\n';
 	    else if (ret > 0)
@@ -304,32 +292,27 @@ static void init_graph(Agraph_t *g, bool fill, GVC_t *gvc) {
  * Copy all attributes from old object to new. Assume
  * attributes have been initialized.
  */
-static void cloneDfltAttrs(Agraph_t *old, Agraph_t *new_graph, int kind) {
-    Agsym_t *a;
-
-    for (a = agnxtattr(old, kind, 0); a; a =  agnxtattr(old, kind, a)) {
+static void cloneDfltAttrs(Agraph_t *old, Agraph_t *new_graph, int attr_kind) {
+    for (Agsym_t *a = agnxtattr(old, attr_kind, 0); a; a = agnxtattr(old, attr_kind, a)) {
 	if (aghtmlstr(a->defval)) {
-	    char *s = agstrdup_html(new_graph, a->defval);
-	    agattr(new_graph, kind, a->name, s);
-	    agstrfree(new_graph, s); // drop the extra reference count we bumped for s
+	    agattr_html(new_graph, attr_kind, a->name, a->defval);
 	} else {
-	    agattr(new_graph, kind, a->name, a->defval);
+	    agattr_text(new_graph, attr_kind, a->name, a->defval);
 	}
     }
 }
 static void cloneAttrs(void *old, void *new_graph) {
-    int kind = AGTYPE(old);
-    Agsym_t *a;
+    int attr_kind = AGTYPE(old);
     char* s;
     Agraph_t *g = agroot(old);
     Agraph_t *ng = agroot(new_graph);
 
-    for (a = agnxtattr(g, kind, 0); a; a =  agnxtattr(g, kind, a)) {
+    for (Agsym_t *a = agnxtattr(g, attr_kind, 0); a; a = agnxtattr(g, attr_kind, a)) {
 	s = agxget (old, a);
 	if (aghtmlstr(s)) {
 	    char *scopy = agstrdup_html(ng, s);
 	    agset(new_graph, a->name, scopy);
-	    agstrfree(ng, scopy); // drop the extra reference count we bumped for scopy
+	    agstrfree(ng, scopy, true); // drop the extra reference count we bumped for scopy
 	} else {
 	    agset(new_graph, a->name, s);
 	}
@@ -352,8 +335,6 @@ static void cloneEdge(Agedge_t *old, Agedge_t *new_edge) {
   ED_xlabel(new_edge) = ED_xlabel(old);
 }
 
-/* cloneNode:
- */
 static void cloneNode(Agnode_t *old, Agnode_t *new_node) {
   cloneAttrs(old, new_node);
   ND_coord(new_node).x = POINTS(ND_pos(old)[0]);
@@ -368,8 +349,6 @@ static void cloneNode(Agnode_t *old, Agnode_t *new_node) {
   ND_xlabel(new_node) = ND_xlabel(old);
 }
 
-/* cloneCluster:
- */
 static void cloneCluster(Agraph_t *old, Agraph_t *new_cluster) {
   // string attributes were cloned as subgraphs
   GD_label(new_cluster) = GD_label(old);
@@ -392,9 +371,9 @@ using attr_map_t = std::map<std::string, AttributeValue>;
  * objp. If the attribute has already been defined and
  * has a different default, set default to "".
  */
-static void fillDict(attr_map_t &newdict, Agraph_t *g, int kind) {
+static void fillDict(attr_map_t &newdict, Agraph_t *g, int attr_kind) {
 
-  for (Agsym_t *a = agnxtattr(g, kind, 0); a; a = agnxtattr(g, kind, a)) {
+  for (Agsym_t *a = agnxtattr(g, attr_kind, 0); a; a = agnxtattr(g, attr_kind, a)) {
     char *name = a->name;
     char *value = a->defval;
     auto it = newdict.find(name);
@@ -445,8 +424,6 @@ static void initAttrs(Agraph_t *root, std::vector<Agraph_t*> &gs) {
     fillGraph(root, e_attrs, agedgeattr, gs.size());
 }
 
-/* cloneGraphAttr:
- */
 static void cloneGraphAttr(Agraph_t * g, Agraph_t * ng)
 {
     cloneAttrs(g, ng);
@@ -475,7 +452,6 @@ static std::string xName(used_t &names, char *oldname) {
 
 #define MARK(e) (ED_alg(e) = e)
 #define MARKED(e) (ED_alg(e))
-#define ISCLUSTER(g) startswith(agnameof(g), "cluster")
 #define SETCLUST(g,h) (GD_alg(g) = h)
 #define GETCLUST(g) ((Agraph_t*)GD_alg(g))
 
@@ -500,14 +476,13 @@ cloneSubg(Agraph_t *g, Agraph_t *ng, Agsym_t *G_bb, used_t &gnames) {
 
     /* clone subgraphs */
     for (subg = agfstsubg (g); subg; subg = agnxtsubg (subg)) {
-	nsubg = agsubg(ng, const_cast<char*>(xName(gnames, agnameof(subg)).c_str()),
-	               1);
+	nsubg = agsubg(ng, xName(gnames, agnameof(subg)).data(), 1);
 	agbindrec (nsubg, "Agraphinfo_t", sizeof(Agraphinfo_t), true);
 	cloneSubg(subg, nsubg, G_bb, gnames);
 	/* if subgraphs are clusters, point to the new 
 	 * one so we can find it later.
 	 */
-	if (ISCLUSTER(subg))
+	if (is_a_cluster(subg))
 	    SETCLUST(subg, nsubg);
     }
 
@@ -560,29 +535,27 @@ static void cloneClusterTree(Agraph_t * g, Agraph_t * ng)
  * Create and return a new graph which is the logical union
  * of the graphs gs. 
  */
-static Agraph_t *cloneGraph(std::vector<Agraph_t*> &gs, GVC_t *gvc) {
+static Agraph_t *cloneGraph(std::vector<Agraph_t *> &gs, GVC_t *gvc,
+                            Agdesc_t kind) {
     Agraph_t *root;
     Agraph_t *subg;
     Agnode_t *n;
     Agnode_t *np;
-    Agsym_t *G_bb;
-    Agsym_t *rv;
     bool doWarn = true;
 
     if (verbose)
 	std::cerr << "Creating clone graph\n";
     root = agopen(gname, kind, &AgDefaultDisc);
     initAttrs(root, gs);
-    G_bb = agfindgraphattr(root, const_cast<char*>("bb"));
+    Agsym_t *const G_bb = agfindgraphattr(root, const_cast<char*>("bb"));
     if (doPack) assert(G_bb);
 
     /* add command-line attributes */
     for (attr_t &a : G_args) {
-	rv = agfindgraphattr(root, a.name);
-	if (rv)
+	if (Agsym_t *rv = agfindgraphattr(root, a.name))
 	    agxset(root, rv, a.value);
 	else
-	    agattr(root, AGRAPH, a.name, a.value);
+	    agattr_text(root, AGRAPH, a.name, a.value);
     }
 
     /* do common initialization. This will handle root's label. */
@@ -606,16 +579,14 @@ static Agraph_t *cloneGraph(std::vector<Agraph_t*> &gs, GVC_t *gvc) {
 		          << "Some nodes will be renamed.\n";
 		doWarn = false;
 	    }
-	    np = agnode(root, const_cast<char*>(xName(nnames, agnameof(n)).c_str()),
-	                1);
+	    np = agnode(root, xName(nnames, agnameof(n)).data(), 1);
 	    agbindrec (np, "Agnodeinfo_t", sizeof(Agnodeinfo_t), true);
 	    ND_alg(n) = np;
 	    cloneNode(n, np);
 	}
 
 	/* wrap the clone of g in a subgraph of root */
-	subg = agsubg(root, const_cast<char*>(xName(gnames, agnameof(g)).c_str()),
-	              1);
+	subg = agsubg(root, xName(gnames, agnameof(g)).data(), 1);
 	agbindrec (subg, "Agraphinfo_t", sizeof(Agraphinfo_t), true);
 	cloneSubg(g, subg, G_bb, gnames);
     }
@@ -647,30 +618,30 @@ static Agraph_t *cloneGraph(std::vector<Agraph_t*> &gs, GVC_t *gvc) {
  * either directed or undirected. If all graphs are strict, the
  * combined graph will be strict; other, the combined graph will
  * be non-strict.
+ *
+ * @param kind [out] The type to use for the combined graph
  */
-static std::vector<Agraph_t*> readGraphs(GVC_t *gvc) {
-    Agraph_t *g;
+static std::vector<Agraph_t *> readGraphs(GVC_t *gvc,
+                                          std::optional<Agdesc_t> &kind) {
     std::vector<Agraph_t*> gs;
     ingraph_state ig;
-    int kindUnset = 1;
 
     /* set various state values */
     PSinputscale = POINTS_PER_INCH;
     Nop = 2;
 
     newIngraph(&ig, myFiles);
-    while ((g = nextGraph(&ig)) != 0) {
+    while (Agraph_t *g = nextGraph(&ig)) {
 	if (verbose)
 	    std::cerr << "Reading graph " << agnameof(g) << '\n';
 	if (agnnodes(g) == 0) {
 	    std::cerr << "Graph " << agnameof(g) << " is empty - ignoring\n";
 	    continue;
 	}
-	if (kindUnset) {
-	    kindUnset = 0;
+	if (!kind.has_value()) {
 	    kind = g->desc;
 	}
-	else if (kind.directed != g->desc.directed) {
+	else if (kind->directed != g->desc.directed) {
 	    std::cerr << "Error: all graphs must be directed or undirected\n";
 	    graphviz_exit(1);
 	} else if (!agisstrict(g))
@@ -742,21 +713,22 @@ int main(int argc, char *argv[])
     lt_preloaded_symbols[0].address = &gvplugin_neato_layout_LTX_library;
 #endif
     gvc = gvContextPlugins(lt_preloaded_symbols, DEMAND_LOADING);
-    std::vector<Agraph_t*> gs = readGraphs(gvc);
+    std::optional<Agdesc_t> kind; // type of graph
+    std::vector<Agraph_t*> gs = readGraphs(gvc, kind);
     if (gs.empty())
 	graphviz_exit(0);
 
     /* pack graphs */
     if (doPack) {
-	assert(gs.size() <= INT_MAX);
-	if (packGraphs((int)gs.size(), gs.data(), 0, &pinfo)) {
+	if (packGraphs(gs.size(), gs.data(), 0, &pinfo)) {
 	    std::cerr << "gvpack: packing of graphs failed.\n";
 	    graphviz_exit(1);
 	}
     }
 
     /* create union graph and copy attributes */
-    g = cloneGraph(gs, gvc);
+    assert(kind.has_value());
+    g = cloneGraph(gs, gvc, *kind);
 
     /* compute new top-level bb and set */
     if (doPack) {
