@@ -8,23 +8,25 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
 #include    <assert.h>
 #include    "convert.h"
-#include    <cgraph/agxbuf.h>
-#include    <cgraph/alloc.h>
-#include    <cgraph/exit.h>
-#include    <cgraph/stack.h>
+#include    <stdbool.h>
 #include    <stdio.h>
+#include    <util/agxbuf.h>
+#include    <util/alloc.h>
+#include    <util/exit.h>
+#include    <util/gv_ctype.h>
+#include    <util/list.h>
+#include    <util/startswith.h>
+#include    <util/unreachable.h>
 #ifdef HAVE_EXPAT
 #include    <expat.h>
-#include    <ctype.h>
 #include    <limits.h>
-#include    <stdbool.h>
 #include    <stdlib.h>
 
 #ifndef XML_STATUS_ERROR
@@ -55,11 +57,11 @@ typedef struct {
     agxbuf xml_attr_name;
     agxbuf xml_attr_value;
     agxbuf composite_buffer;
-    int listen;
+    bool listen;
     attr_t closedElementType;
     attr_t globalAttrType;
-    int compositeReadState;
-    int edgeinverted;
+    bool compositeReadState;
+    bool edgeinverted;
     Dt_t *nameMap;
 } userdata_t;
 
@@ -69,7 +71,8 @@ static Agraph_t *G;		/* Current graph */
 static Agnode_t *N;		/* Set if Current_class == TAG_NODE */
 static Agedge_t *E;		/* Set if Current_class == TAG_EDGE */
 
-static gv_stack_t Gstack;
+DEFINE_LIST(graph_stack, Agraph_t *)
+static graph_stack_t Gstack;
 
 typedef struct {
     Dtlink_t link;
@@ -77,10 +80,9 @@ typedef struct {
     char *unique_name;
 } namev_t;
 
-static namev_t *make_nitem(Dt_t * d, namev_t * objp, Dtdisc_t * disc)
-{
-    (void)d;
+static void *make_nitem(void *p, Dtdisc_t *disc){
     (void)disc;
+    namev_t *objp = p;
 
     namev_t *np = malloc(sizeof(namev_t));
     if (np == NULL)
@@ -90,11 +92,8 @@ static namev_t *make_nitem(Dt_t * d, namev_t * objp, Dtdisc_t * disc)
     return np;
 }
 
-static void free_nitem(Dt_t * d, namev_t * np, Dtdisc_t * disc)
-{
-    (void)d;
-    (void)disc;
-
+static void free_nitem(void *name) {
+    namev_t *np = name;
     free(np->unique_name);
     free(np);
 }
@@ -103,17 +102,17 @@ static Dtdisc_t nameDisc = {
     .key = offsetof(namev_t, name),
     .size = -1,
     .link = offsetof(namev_t, link),
-    .makef = (Dtmake_f)make_nitem,
-    .freef = (Dtfree_f)free_nitem,
+    .makef = make_nitem,
+    .freef = free_nitem,
 };
 
 static userdata_t genUserdata(void) {
   userdata_t user = {0};
-  user.listen = FALSE;
+  user.listen = false;
   user.closedElementType = TAG_NONE;
   user.globalAttrType = TAG_NONE;
-  user.compositeReadState = FALSE;
-  user.edgeinverted = FALSE;
+  user.compositeReadState = false;
+  user.edgeinverted = false;
   user.nameMap = dtopen(&nameDisc, Dtoset);
   return user;
 }
@@ -148,7 +147,7 @@ static int isAnonGraph(const char *name)
 {
     if (*name++ != '%')
 	return 0;
-    while (isdigit((int)*name))
+    while (gv_isdigit(*name))
 	name++;			/* skip over digits */
     return (*name == '\0');
 }
@@ -156,10 +155,10 @@ static int isAnonGraph(const char *name)
 static void push_subg(Agraph_t * g)
 {
   // insert the new graph
-  stack_push(&Gstack, g);
+  graph_stack_push_back(&Gstack, g);
 
   // save the root if this is the first graph
-  if (stack_size(&Gstack) == 1) {
+  if (graph_stack_size(&Gstack) == 1) {
     root = g;
   }
 
@@ -170,17 +169,17 @@ static void push_subg(Agraph_t * g)
 static Agraph_t *pop_subg(void)
 {
   // is the stack empty?
-  if (stack_is_empty(&Gstack)) {
+  if (graph_stack_is_empty(&Gstack)) {
     fprintf(stderr, "gxl2gv: Gstack underflow in graph parser\n");
     graphviz_exit(EXIT_FAILURE);
   }
 
   // pop the top graph
-  Agraph_t *g = stack_pop(&Gstack);
+  Agraph_t *g = graph_stack_pop_back(&Gstack);
 
   // update the top graph
-  if (!stack_is_empty(&Gstack))
-    G = stack_top(&Gstack);
+  if (!graph_stack_is_empty(&Gstack))
+    G = *graph_stack_back(&Gstack);
 
   return g;
 }
@@ -216,12 +215,9 @@ static int get_xml_attr(char *attrname, const char **atts)
 
 static void setName(Dt_t * names, Agobj_t * n, char *value)
 {
-    Agsym_t *ap;
-    char *oldName;
-
-    ap = agattr(root, AGTYPE(n), GXL_ID, "");
+    Agsym_t *const ap = agattr_text(root, AGTYPE(n), GXL_ID, "");
     agxset(n, ap, agnameof(n));
-    oldName = agxget(n, ap);	/* set/get gives us new copy */
+    char *const oldName = agxget(n, ap); // set/get gives us new copy
     addToMap(names, oldName, value);
     agrename(n, value);
 }
@@ -232,18 +228,14 @@ static void
 setNodeAttr(Agnode_t * np, char *name, char *value, userdata_t * ud,
   bool is_html)
 {
-    Agsym_t *ap;
-
     if (strcmp(name, "name") == 0) {
 	setName(ud->nameMap, (Agobj_t *) np, value);
     } else {
-	ap = agattr(root, AGNODE, name, 0);
+	Agsym_t *ap = agattr_text(root, AGNODE, name, 0);
 	if (!ap)
-	    ap = agattr(root, AGNODE, name, defval);
+	    ap = agattr_text(root, AGNODE, name, defval);
 	if (is_html) {
-	    char *val = agstrdup_html(root, value);
-	    agxset(np, ap, val);
-	    agstrfree(root, val); // drop the extra reference count we bumped for val
+	    agxset_html(np, ap, value);
 	} else {
 	    agxset(np, ap, value);
 	}
@@ -262,15 +254,15 @@ setNodeAttr(Agnode_t * np, char *name, char *value, userdata_t * ud,
 static void
 setGlobalNodeAttr(Agraph_t * g, char *name, char *value)
 {
-    if (strncmp(name, NODELBL, NLBLLEN))
+    if (!startswith(name, NODELBL))
 	fprintf(stderr,
 		"Warning: global node attribute %s in graph %s does not begin with the prefix %s\n",
 		name, agnameof(g), NODELBL);
     else
 	name += NLBLLEN;
-    if ((g != root) && !agattr(root, AGNODE, name, 0))
-	agattr(root, AGNODE, name, defval);
-    agattr(G, AGNODE, name, value);
+    if ((g != root) && !agattr_text(root, AGNODE, name, 0))
+	agattr_text(root, AGNODE, name, defval);
+    agattr_text(G, AGNODE, name, value);
 }
 
 static void
@@ -285,27 +277,25 @@ setEdgeAttr(Agedge_t * ep, char *name, char *value, userdata_t * ud,
 	    attrname = "tailport";
 	else
 	    attrname = "headport";
-	ap = agattr(root, AGEDGE, attrname, 0);
+	ap = agattr_text(root, AGEDGE, attrname, 0);
 	if (!ap)
-	    ap = agattr(root, AGEDGE, attrname, defval);
+	    ap = agattr_text(root, AGEDGE, attrname, defval);
     } else if (strcmp(name, "tailport") == 0) {
 	if (ud->edgeinverted)
 	    attrname = "headport";
 	else
 	    attrname = "tailport";
-	ap = agattr(root, AGEDGE, attrname, 0);
+	ap = agattr_text(root, AGEDGE, attrname, 0);
 	if (!ap)
-	    ap = agattr(root, AGEDGE, attrname, defval);
+	    ap = agattr_text(root, AGEDGE, attrname, defval);
     } else {
-	ap = agattr(root, AGEDGE, name, 0);
+	ap = agattr_text(root, AGEDGE, name, 0);
 	if (!ap)
-	    ap = agattr(root, AGEDGE, name, defval);
+	    ap = agattr_text(root, AGEDGE, name, defval);
     }
 
     if (is_html) {
-	char *val = agstrdup_html(root, value);
-	agxset(ep, ap, val);
-	agstrfree(root, val); // drop the extra reference count we bumped for val
+	agxset_html(ep, ap, value);
     } else {
 	agxset(ep, ap, value);
     }
@@ -318,15 +308,15 @@ setEdgeAttr(Agedge_t * ep, char *name, char *value, userdata_t * ud,
 static void
 setGlobalEdgeAttr(Agraph_t * g, char *name, char *value)
 {
-    if (strncmp(name, EDGELBL, ELBLLEN))
+    if (!startswith(name, EDGELBL))
 	fprintf(stderr,
 		"Warning: global edge attribute %s in graph %s does not begin with the prefix %s\n",
 		name, agnameof(g), EDGELBL);
     else
 	name += ELBLLEN;
-    if ((g != root) && !agattr(root, AGEDGE, name, 0))
-	agattr(root, AGEDGE, name, defval);
-    agattr(g, AGEDGE, name, value);
+    if ((g != root) && !agattr_text(root, AGEDGE, name, 0))
+	agattr_text(root, AGEDGE, name, defval);
+    agattr_text(g, AGEDGE, name, value);
 }
 
 static void
@@ -335,17 +325,17 @@ setGraphAttr(Agraph_t * g, char *name, char *value, userdata_t * ud)
     Agsym_t *ap;
 
     if ((g == root) && !strcmp(name, "strict") && !strcmp(value, "true")) {
-	g->desc.strict = 1;
+	g->desc.strict = true;
     } else if (strcmp(name, "name") == 0)
 	setName(ud->nameMap, (Agobj_t *) g, value);
     else {
-	ap = agattr(root, AGRAPH, name, 0);
+	ap = agattr_text(root, AGRAPH, name, 0);
 	if (ap)
 	    agxset(g, ap, value);
 	else if (g == root)
-	    agattr(root, AGRAPH, name, value);
+	    agattr_text(root, AGRAPH, name, value);
 	else {
-	    ap = agattr(root, AGRAPH, name, defval);
+	    ap = agattr_text(root, AGRAPH, name, defval);
 	    agxset(g, ap, value);
 	}
     }
@@ -399,7 +389,7 @@ startElementHandler(void *userData, const char *name, const char **atts)
 	    edgeMode = atts[pos];
 	}
 
-	if (stack_is_empty(&Gstack)) {
+	if (graph_stack_is_empty(&Gstack)) {
 	    if (strcmp(edgeMode, "directed") == 0) {
 		g = agopen((char *) id, Agdirected, &AgDefaultDisc);
 	    } else if (strcmp(edgeMode, "undirected") == 0) {
@@ -471,9 +461,9 @@ startElementHandler(void *userData, const char *name, const char **atts)
 	tname = agnameof(t);
 
 	if (strcmp(tname, tail) == 0) {
-	    ud->edgeinverted = FALSE;
+	    ud->edgeinverted = false;
 	} else if (strcmp(tname, head) == 0) {
-	    ud->edgeinverted = TRUE;
+	    ud->edgeinverted = true;
 	}
 
 	pos = get_xml_attr("fromorder", atts);
@@ -513,7 +503,7 @@ startElementHandler(void *userData, const char *name, const char **atts)
 	       || strcmp(name, "bool") == 0
 	       || strcmp(name, "int") == 0 || strcmp(name, "float") == 0) {
 
-	ud->listen = TRUE;
+	ud->listen = true;
 	if (ud->compositeReadState) {
 	    agxbprint(&ud->composite_buffer, "<%s>", name);
 	}
@@ -535,7 +525,7 @@ startElementHandler(void *userData, const char *name, const char **atts)
 	       || strcmp(name, "bag") == 0
 	       || strcmp(name, "tup") == 0 || strcmp(name, "enum") == 0) {
 
-	ud->compositeReadState = TRUE;
+	ud->compositeReadState = true;
 	agxbprint(&ud->composite_buffer, "<%s>", name);
     } else {
 	/* must be some extension */
@@ -560,7 +550,7 @@ static void endElementHandler(void *userData, const char *name)
 	Current_class = TAG_GRAPH;
 	E = 0;
 	ud->closedElementType = TAG_EDGE;
-	ud->edgeinverted = FALSE;
+	ud->edgeinverted = false;
     } else if (strcmp(name, "attr") == 0) {
 	agxbuf new_name = {0};
 	char *value;
@@ -570,7 +560,7 @@ static void endElementHandler(void *userData, const char *name)
 	    agxbprint(&new_name, "%s%s", GXL_COMP, agxbuse(&ud->xml_attr_name));
 	    value = agxbuse(&ud->composite_buffer);
 	    agxbclear(&ud->xml_attr_value);
-	    ud->compositeReadState = FALSE;
+	    ud->compositeReadState = false;
 	} else {
 	    agxbput(&new_name, agxbuse(&ud->xml_attr_name));
 	    value = agxbuse(&ud->xml_attr_value);
@@ -592,13 +582,15 @@ static void endElementHandler(void *userData, const char *name)
 	case TAG_HTML_LIKE_STRING:
 	    setAttr(agxbuse(&new_name), value, ud, true);
 	    break;
+	default:
+	    UNREACHABLE();
 	}
 	agxbfree(&new_name);
 	ud->globalAttrType = TAG_NONE;
     } else if (strcmp(name, "string") == 0
 	       || strcmp(name, "bool") == 0
 	       || strcmp(name, "int") == 0 || strcmp(name, "float") == 0) {
-	ud->listen = FALSE;
+	ud->listen = false;
 	if (ud->compositeReadState) {
 	    agxbprint(&ud->composite_buffer, "</%s>", name);
 	}
@@ -646,7 +638,7 @@ Agraph_t *gxl_to_gv(FILE * gxlFile)
 	if (len == 0)
 	    break;
 	done = len < sizeof(buf);
-	assert(len <= (size_t)INT_MAX && "too large data for Expat API");
+	assert(len <= INT_MAX && "too large data for Expat API");
 	if (XML_Parse(parser, buf, (int)len, done) == XML_STATUS_ERROR) {
 	    fprintf(stderr,
 		    "%s at line %lu\n",
@@ -657,7 +649,7 @@ Agraph_t *gxl_to_gv(FILE * gxlFile)
     } while (!done);
     XML_ParserFree(parser);
     freeUserdata(udata);
-    stack_reset(&Gstack);
+    graph_stack_free(&Gstack);
 
     return root;
 }
