@@ -8,7 +8,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -30,13 +30,13 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <cgraph/cgraph.h>
-#include <cgraph/exit.h>
-#include <cgraph/stack.h>
-#include <cgraph/unreachable.h>
-#include <ingraphs/ingraphs.h>
+#include <cgraph/ingraphs.h>
 
 #include <getopt.h>
 #include "openFile.h"
+#include <util/exit.h>
+#include <util/list.h>
+#include <util/unreachable.h>
 
 #define INF UINT_MAX
 
@@ -103,14 +103,16 @@ static void nodeInduce(Agraph_t * g, Agraph_t* map)
 		Agraph_t* tscc = getscc(agtail(e));
 		Agraph_t* hscc = getscc(aghead(e));
 		if (tscc && hscc)
-		    agedge(map, getrep(tscc),
-			   getrep(hscc), NULL, TRUE);
+		    agedge(map, getrep(tscc), getrep(hscc), NULL, 1);
 	    }
 	}
     }
 }
 
-static unsigned visit(Agnode_t *n, Agraph_t *map, gv_stack_t *sp, sccstate *st) {
+DEFINE_LIST(node_stack, Agnode_t *)
+
+static unsigned visit(Agnode_t *n, Agraph_t *map, node_stack_t *sp,
+                      sccstate *st) {
     unsigned int m, min;
     Agnode_t *t;
     Agraph_t *subg;
@@ -118,7 +120,7 @@ static unsigned visit(Agnode_t *n, Agraph_t *map, gv_stack_t *sp, sccstate *st)
 
     min = ++st->ID;
     setval(n, min);
-    stack_push(sp, n);
+    node_stack_push_back(sp, n);
 
     for (e = agfstout(n->root, n); e; e = agnxtout(n->root, e)) {
 	t = aghead(e);
@@ -131,18 +133,18 @@ static unsigned visit(Agnode_t *n, Agraph_t *map, gv_stack_t *sp, sccstate *st)
     }
 
     if (getval(n) == min) {
-	if (!wantDegenerateComp && stack_top(sp) == n) {
+	if (!wantDegenerateComp && *node_stack_back(sp) == n) {
 	    setval(n, INF);
-	    stack_pop(sp);
+	    (void)node_stack_pop_back(sp);
 	} else {
 	    char name[32];
 	    Agraph_t *G = agraphof(n);;
 	    snprintf(name, sizeof(name), "cluster_%u", st->Comp++);
-	    subg = agsubg(G, name, TRUE);
+	    subg = agsubg(G, name, 1);
 	    agbindrec(subg, "scc_graph", sizeof(Agraphinfo_t), true);
-	    setrep(subg, agnode(map, name, TRUE));
+	    setrep(subg, agnode(map, name, 1));
 	    do {
-		t = stack_pop(sp);
+		t = node_stack_pop_back(sp);
 		agsubnode(subg, t, 1);
 		setval(t, INF);
 		setscc(t, subg);
@@ -218,7 +220,7 @@ static void process(Agraph_t * G)
     int nc = 0;
     float nontree_frac = 0;
     int Maxdegree = 0;
-    gv_stack_t stack = {0};
+    node_stack_t stack = {0};
     sccstate state;
 
     aginit(G, AGRAPH, "scc_graph", sizeof(Agraphinfo_t), true);
@@ -229,11 +231,11 @@ static void process(Agraph_t * G)
     if (Verbose)
 	nc = countComponents(G, &Maxdegree, &nontree_frac);
 
-    map = agopen("scc_map", Agdirected, (Agdisc_t *) 0);
+    map = agopen("scc_map", Agdirected, NULL);
     for (n = agfstnode(G); n; n = agnxtnode(G, n))
 	if (getval(n) == 0)
 	    visit(n, map, &stack, &state);
-    stack_reset(&stack);
+    node_stack_free(&stack);
     if (!StatsOnly)
 	agwrite(map, outfp);
     agclose(map);
