@@ -31,7 +31,6 @@
 
 #include <cgraph.h>
 
-#include	 	<cgraph/list.h>
 #include	 	<ctype.h>
 #include		<sys/types.h>
 #include		<stdarg.h>
@@ -40,6 +39,8 @@
 #include		<string.h>
 #include <assert.h>
 #include <stdint.h>
+#include <stdio.h>
+#include <util/list.h>
 
 #define	SUCCESS				0
 #define FAILURE				-1
@@ -47,7 +48,6 @@
 
 #define AGDISC(g,d)			((g)->clos->disc.d)
 #define AGCLOS(g,d)			((g)->clos->state.d)
-#define AGNEW(g,t)			((t*)(agalloc(g,sizeof(t))))
 
 	/* functional definitions */
 typedef Agobj_t *(*agobjsearchfn_t) (Agraph_t * g, Agobj_t * obj);
@@ -55,8 +55,7 @@ CGHDR_API int agapply(Agraph_t * g, Agobj_t * obj, agobjfn_t fn, void *arg,
 	    int preorder);
 
 	/* global variables */
-extern Agraph_t *Ag_G_global;
-extern char *AgDataRecName;
+extern const char AgDataRecName[];
 
 	/* set ordering disciplines */
 extern Dtdisc_t Ag_subnode_seq_disc;
@@ -71,9 +70,6 @@ extern Dtdisc_t Ag_subgraph_seq_disc;
 Agraph_t *agopen1(Agraph_t * g);
 int agstrclose(Agraph_t * g);
 
-	/* ref string management */
-void agmarkhtmlstr(char *s);
-
 /// Mask of `Agtag_s.seq` width
 enum { SEQ_MASK = (1 << (sizeof(unsigned) * 8 - 4)) - 1 };
 
@@ -82,11 +78,10 @@ Agnode_t *agfindnode_by_id(Agraph_t * g, IDTYPE id);
 uint64_t agnextseq(Agraph_t * g, int objtype);
 
 /* dict helper functions */
-Dict_t *agdtopen(Agraph_t * g, Dtdisc_t * disc, Dtmethod_t * method);
+Dict_t *agdtopen(Dtdisc_t *disc, Dtmethod_t *method);
 void agdtdisc(Agraph_t * g, Dict_t * dict, Dtdisc_t * disc);
 int agdtdelete(Agraph_t * g, Dict_t * dict, void *obj);
 int agdtclose(Agraph_t * g, Dict_t * dict);
-void agdictobjfree(void *p);
 
 /** @addtogroup cgraph_attr
  *  @{
@@ -112,16 +107,18 @@ void agedgeattr_delete(Agedge_t * e);
 typedef void *aagscan_t;
 typedef struct aagextra_s aagextra_t;
 
-int aagparse(void);
-void aglexinit(Agdisc_t * disc, void *ifile);
-int aaglex(void);
-void aglexeof(void);
-void aglexbad(void);
+int aaglex_init_extra(aagextra_t* user_defined, aagscan_t* scanner);
+int aaglex_destroy(aagscan_t);
+aagextra_t *aagget_extra(aagscan_t yyscanner);
+void aagset_in(FILE *, aagscan_t);
+
+int aagparse(aagscan_t scanner);
+void aglexeof(aagscan_t yyscanner);
+void aglexbad(aagscan_t yyscanner);
 
 	/* ID management */
 int agmapnametoid(Agraph_t *g, int objtype, char *str, IDTYPE *result,
                   bool createflag);
-int agallocid(Agraph_t * g, int objtype, IDTYPE request);
 void agfreeid(Agraph_t * g, int objtype, IDTYPE id);
 char *agprintid(Agobj_t * obj);
 bool aginternalmaplookup(Agraph_t *g, int objtype, char *str, IDTYPE *result);
@@ -134,8 +131,13 @@ void agregister(Agraph_t * g, int objtype, void *obj);
 
 	/* internal set operations */
 void agedgesetop(Agraph_t * g, Agedge_t * e, int insertion);
-void agdelnodeimage(Agraph_t * g, Agnode_t * node, void *ignored);
-void agdeledgeimage(Agraph_t * g, Agedge_t * edge, void *ignored);
+void agdelnodeimage(Agraph_t *g, Agobj_t *node, void *ignored);
+void agdeledgeimage(Agraph_t *g, Agobj_t *edge, void *ignored);
+/// rename an object
+///
+/// @param obj Target to rename
+/// @param newname Name to assign
+/// @return 0 on success
 CGHDR_API int agrename(Agobj_t * obj, char *newname);
 void agrecclose(Agobj_t * obj);
 
@@ -150,39 +152,7 @@ void agupdcb(Agraph_t * g, void *obj, Agsym_t * sym, Agcbstack_t * disc);
 void agdelcb(Agraph_t * g, void *obj, Agcbstack_t * disc);
 /// @}
 
-DEFINE_LIST(Agraphs, Agraph_t *)
-
-/// two alternative representations of a graph collection
-///
-/// The subgraphs of a graph have traditionally been stored in a `Dt_t`. However
-/// it is much faster to iterate through a C array of the subgraphs instead of a
-/// cdt tree of them. To preserve backwards compatibility, we maintain both:
-///
-///   what the cgraph.h user sees:            what we see internally:
-///
-///      Agraph_t                                Agraph_t
-///     ┌─────────┐                             ┌─────────┐
-///     │ …       │                             │ …       │
-///     ├─────────┤    ┌───────────┐            ├─────────┤    ┌───────────┐
-///     │  g_seq ─┼───►│   Dt_t    │            │  g_seq ─┼───►│   Dt_t    │
-///     ├─────────┤    └───────────┘            ├─────────┤    ├───────────┤
-///     │ …       │                             │ …       │    │ Agraphs_t │
-///                                                            └───────────┘
-///
-/// To facilitate this, `g_seq2` can be used to access the second
-/// representation.
-typedef struct {
-  Dt_t legacy;      ///< the traditional tree of subgraphs
-  Agraphs_t linear; ///< the newer contiguous collection
-} g_seq_t;
-
-/// access the list of subgraphs of a graph
-///
-/// See `g_seq_t` for an extended explanation of this.
-///
-/// @param g The parent graph to inspect
-/// @return A generic list of the subgraphs
-static inline Agraphs_t *g_seq2(Agraph_t *g) {
-  g_seq_t *seq = (g_seq_t *)g->g_seq;
-  return &seq->linear;
-}
+/// @defgroup cgmem memory
+/// @{
+void aginternalmapclearlocalnames(Agraph_t *g);
+/// @}
