@@ -8,13 +8,14 @@
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
-#include	<cgraph/list.h>
-#include	<cgraph/agxbuf.h>
-#include	<cgraph/alloc.h>
 #include	<circogen/blockpath.h>
+#include	<circogen/circular.h>
 #include	<circogen/edgelist.h>
 #include	<stddef.h>
 #include	<stdbool.h>
+#include	<util/agxbuf.h>
+#include	<util/alloc.h>
+#include	<util/list.h>
 
 /* The code below lays out a single block on a circle.
  */
@@ -26,9 +27,10 @@
  * Create two copies of the argument graph
  * One is a subgraph, the other is an actual copy since we will be
  * adding edges to it.
+ *
+ * @param state Context containing a counter to use for graph copy naming
  */
-static Agraph_t *clone_graph(Agraph_t * ing, Agraph_t ** xg)
-{
+static Agraph_t *clone_graph(Agraph_t *ing, Agraph_t **xg, circ_state *state) {
     Agraph_t *clone;
     Agraph_t *xclone;
     Agnode_t *n;
@@ -37,12 +39,11 @@ static Agraph_t *clone_graph(Agraph_t * ing, Agraph_t ** xg)
     Agedge_t *e;
     Agedge_t *xe;
     agxbuf gname = {0};
-    static int id = 0;
 
-    agxbprint(&gname, "_clone_%d", id++);
+    agxbprint(&gname, "_clone_%d", state->graphCopyCount++);
     clone = agsubg(ing, agxbuse(&gname), 1);
     agbindrec(clone, "Agraphinfo_t", sizeof(Agraphinfo_t), true);	//node custom data
-    agxbprint(&gname, "_clone_%d", id++);
+    agxbprint(&gname, "_clone_%d", state->graphCopyCount++);
     xclone = agopen(agxbuse(&gname), ing->desc, NULL);
     agxbfree(&gname);
     for (n = agfstnode(ing); n; n = agnxtnode(ing, n)) {
@@ -198,8 +199,9 @@ static void find_pair_edges(Agraph_t * g, Agnode_t * n, Agraph_t * outg)
 }
 
 /// Create layout skeleton of ing. Why is returned graph connected?
-static Agraph_t *remove_pair_edges(Agraph_t * ing)
-{
+///
+/// @param state Context containing a counter to use for graph copy naming
+static Agraph_t *remove_pair_edges(Agraph_t *ing, circ_state *state) {
     int counter = 0;
     int nodeCount;
     Agraph_t *outg;
@@ -207,12 +209,12 @@ static Agraph_t *remove_pair_edges(Agraph_t * ing)
     Agnode_t *currnode, *adjNode;
     Agedge_t *e;
 
-    outg = clone_graph(ing, &g);
+    outg = clone_graph(ing, &g, state);
     nodeCount = agnnodes(g);
     deglist_t dl = getList(g);
 
     while (counter < nodeCount - 3) {
-	currnode = deglist_is_empty(&dl) ? NULL : deglist_pop(&dl);
+	currnode = deglist_is_empty(&dl) ? NULL : deglist_pop_back(&dl);
 
 	/* Remove all adjacent nodes since they have to be reinserted */
 	for (e = agfstedge(g, currnode); e; e = agnxtedge(g, e, currnode)) {
@@ -358,14 +360,14 @@ static void dfs(Agraph_t * g, Agnode_t * n, Agraph_t * tree)
 }
 
 /// Construct spanning forest of g as subgraph
-static Agraph_t *spanning_tree(Agraph_t * g)
-{
+///
+/// @param state Context containing a counter to use for spanning tree naming
+static Agraph_t *spanning_tree(Agraph_t *g, circ_state *state) {
     Agnode_t *n;
     Agraph_t *tree;
     agxbuf gname = {0};
-    static int id = 0;
 
-    agxbprint(&gname, "_span_%d", id++);
+    agxbprint(&gname, "_span_%d", state->spanningTreeCount++);
     tree = agsubg(g, agxbuse(&gname), 1);
     agxbfree(&gname);
     agbindrec(tree, "Agraphinfo_t", sizeof(Agraphinfo_t), true);	//node custom data
@@ -543,15 +545,12 @@ static void place_node(Agraph_t * g, Agnode_t * n, nodelist_t * list)
 
     /* Look for 2 neighbors consecutive on list */
     if (nodelist_size(&neighbors) >= 2) {
-	for (size_t two, one = 0; one < nodelist_size(list); ++one) {
-	    if (one == nodelist_size(list) - 1)
-		two = 0;
-	    else
-		two = one + 1;
+	for (size_t one = 0; one < nodelist_size(list); ++one) {
+	    const size_t two = (one + 1) % nodelist_size(list);
 
 	    if (NEIGHBOR(nodelist_get(list, one)) &&
 	        NEIGHBOR(nodelist_get(list, two))) {
-		appendNodelist(list, one, n);
+		appendNodelist(list, one + 1, n);
 		placed = true;
 		break;
 	    }
@@ -562,7 +561,7 @@ static void place_node(Agraph_t * g, Agnode_t * n, nodelist_t * list)
     if (!placed && !nodelist_is_empty(&neighbors)) {
 	for (size_t one = 0; one < nodelist_size(list); ++one) {
 	    if (NEIGHBOR(nodelist_get(list, one))) {
-		appendNodelist(list, one, n);
+		appendNodelist(list, one + 1, n);
 		placed = true;
 		break;
 	    }
@@ -588,7 +587,9 @@ static void place_residual_nodes(Agraph_t * g, nodelist_t * list)
     }
 }
 
-nodelist_t layout_block(Agraph_t *g, block_t *sn, double min_dist) {
+/// @param state Context containing a counter to use for graph copy naming
+nodelist_t layout_block(Agraph_t *g, block_t *sn, double min_dist,
+                        circ_state *state) {
     Agraph_t *copyG, *tree, *subg;
     int k;
     double theta, radius, largest_node;
@@ -597,9 +598,9 @@ nodelist_t layout_block(Agraph_t *g, block_t *sn, double min_dist) {
     subg = sn->sub_graph;
     block_graph(g, sn);		/* add induced edges */
 
-    copyG = remove_pair_edges(subg);
+    copyG = remove_pair_edges(subg, state);
 
-    tree = spanning_tree(copyG);
+    tree = spanning_tree(copyG, state);
     nodelist_t longest_path = find_longest_path(tree);
     place_residual_nodes(subg, &longest_path);
     /* at this point, longest_path is a list of all nodes in the block */
