@@ -3,18 +3,17 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
+#pragma once
+
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-#ifndef BLOCKTREE_H
-#define BLOCKTREE_H
-
 #include <render.h>
 #include <circogen/circular.h>
 
@@ -24,8 +23,6 @@ extern "C" {
     extern void print_blocktree(block_t * sn, int depth);
 #endif
 
-#endif
-
 #ifdef __cplusplus
 }
 #endif
