@@ -186,9 +186,7 @@ getInfo (posinfo_t* pi, posstate * stp, double min_dist)
     for (child = stp->cp; child; child = child->next) {
 	if (BLK_PARENT(child) == pi->n) {
 	    childCount++;
-	    if (maxRadius < child->radius) {
-		maxRadius = child->radius;
-	    }
+	    maxRadius = fmax(maxRadius, child->radius);
 	    diameter += 2 * child->radius + min_dist;
 	}
     }
@@ -236,8 +234,7 @@ static void positionChildren(posinfo_t *info, posstate *stp, size_t length,
     else
 	childAngle = info->theta - info->diameter / (2 * childRadius);
 
-    if ((childRadius + info->maxRadius) > snRadius)
-	snRadius = childRadius + info->maxRadius;
+    snRadius = fmax(snRadius, childRadius + info->maxRadius);
 
     mindistAngle = min_dist / childRadius;
 
