@@ -11,12 +11,14 @@
  *************************************************************************/
 
 #include <sys/stat.h>
-#include <cgraph/alloc.h>
 #include <common/render.h>
 #include <gvc/gvio.h>
-#include <cgraph/strcasecmp.h>
+#include <stdatomic.h>
 #include <stdbool.h>
 #include <stdio.h>
+#include <util/alloc.h>
+#include <util/gv_fopen.h>
+#include <util/strcasecmp.h>
 
 static int N_EPSF_files;
 static Dict_t *EPSF_contents;
@@ -46,7 +48,7 @@ static usershape_t *user_init(const char *str)
     if (us)
 	return us;
 
-    if (!(fp = fopen(str, "r"))) {
+    if (!(fp = gv_fopen(str, "r"))) {
 	agwarningf("couldn't open epsf file %s\n", str);
 	return NULL;
     }
@@ -72,7 +74,7 @@ static usershape_t *user_init(const char *str)
 	us->macro_id = N_EPSF_files++;
 	fstat(fileno(fp), &statbuf);
 	char *contents = us->data = gv_calloc((size_t)statbuf.st_size + 1, sizeof(char));
-	fseek(fp, 0, SEEK_SET);
+	rewind(fp);
 	size_t rc = fread(contents, (size_t)statbuf.st_size, 1, fp);
 	if (rc == 1) {
             contents[statbuf.st_size] = '\0';
@@ -157,7 +159,7 @@ void cat_libfile(GVJ_t * job, const char **arglib, const char **stdlib)
 	    if (!safepath) {
 		agwarningf("can't find library file %s\n", p);
 	    }
-            else if ((fp = fopen(safepath, "r"))) {
+            else if ((fp = gv_fopen(safepath, "r"))) {
                 while (true) {
                     char bp[BUFSIZ] = {0};
                     size_t r = fread(bp, 1, sizeof(bp), fp);
@@ -254,7 +256,7 @@ char *ps_string(char *ins, int chset)
 {
     char *base;
     static agxbuf  xb;
-    static int warned;
+    static atomic_flag warned;
 
     switch (chset) {
     case CHAR_UTF8 :
@@ -272,9 +274,8 @@ char *ps_string(char *ins, int chset)
 	    base = utf8ToLatin1 (ins);
 	    break;
 	case NONLATIN :
-	    if (!warned) {
+	    if (!atomic_flag_test_and_set(&warned)) {
 		agwarningf("UTF-8 input uses non-Latin1 characters which cannot be handled by this PostScript driver\n");
-		warned = 1;
 	    }
 	    base = ins;
 	    break;
