@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -15,21 +15,24 @@
  * expression library evaluator
  */
 
-#include <cgraph/agxbuf.h>
-#include <cgraph/strview.h>
-#include <cgraph/exit.h>
-#include <cgraph/gv_ctype.h>
-#include <cgraph/unreachable.h>
 #include <expr/exlib.h>
 #include <expr/exop.h>
 #include <inttypes.h>
 #include <stdbool.h>
+#include <stddef.h>
 #include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <assert.h>
 #include <time.h>
+#include <util/agxbuf.h>
+#include <util/alloc.h>
+#include <util/exit.h>
+#include <util/gv_ctype.h>
+#include <util/gv_ftell.h>
+#include <util/strview.h>
+#include <util/unreachable.h>
 #ifdef _WIN32
 #define srand48 srand
 #define drand48 rand
@@ -39,39 +42,29 @@
 
 static Extype_t	eval(Expr_t*, Exnode_t*, void*);
 
-#define TOTNAME		4
-#define MAXNAME		23
 #define FRAME		64
 
-static const char*
-lexname(int op, int subop)
-{
-	char*	b;
-
-	static int	n;
-	static char	buf[TOTNAME][MAXNAME];
+static char *lexname(long op, int subop) {
+	agxbuf b = {0};
 
 	if (op > MINTOKEN && op < MAXTOKEN)
-		return exop((size_t)op - MINTOKEN);
-	if (++n >= TOTNAME)
-		n = 0;
-	b = buf[n];
+		return gv_strdup(exop((size_t)op - MINTOKEN));
 	if (op == '=')
 	{
 		if (subop > MINTOKEN && subop < MAXTOKEN)
-			snprintf(b, MAXNAME, "%s=", exop((size_t)subop - MINTOKEN));
+			agxbprint(&b, "%s=", exop((size_t)subop - MINTOKEN));
 		else if (subop > ' ' && subop <= '~')
-			snprintf(b, MAXNAME, "%c=", subop);
+			agxbprint(&b, "%c=", subop);
 		else
-			snprintf(b, MAXNAME, "(%d)=", subop);
+			agxbprint(&b, "(%d)=", subop);
 	}
 	else if (subop < 0)
-		snprintf(b, MAXNAME, "(EXTERNAL:%d)", op);
+		agxbprint(&b, "(EXTERNAL:%ld)", op);
 	else if (op > ' ' && op <= '~')
-		snprintf(b, MAXNAME, "%c", op);
+		agxbprint(&b, "%c", (char)op);
 	else
-		snprintf(b, MAXNAME, "(%d)", op);
-	return b;
+		agxbprint(&b, "(%ld)", op);
+	return agxbdisown(&b);
 }
 
 /* evaldyn:
@@ -87,13 +80,19 @@ static int evaldyn(Expr_t *ex, Exnode_t *exnode, void *env, int delete) {
 	char *keyname;
 
 	v = eval(ex, exnode->data.variable.index, env);
+	if (exnode->data.variable.symbol->index > 0) {
+		if (ex->disc->inf != NULL) {
+			return ex->disc->inf(v, exnode->data.variable.symbol, ex->disc);
+		}
+		return 0; // treat the RHS as an empty array
+	}
 	if (exnode->data.variable.symbol->index_type == INTEGER) {
-		if (!(b = dtmatch((Dt_t *)exnode->data.variable.symbol->local.pointer, &v))) {
+		if (!(b = dtmatch(exnode->data.variable.symbol->local, &v))) {
 			return 0;
 		}
 	} 
 	else {
-		int type = exnode->data.variable.index->type;
+		const long type = exnode->data.variable.index->type;
 		if (type != STRING) {
 			if (!BUILTIN(type)) {
 				key = ex->disc->keyf(v, type);
@@ -103,14 +102,12 @@ static int evaldyn(Expr_t *ex, Exnode_t *exnode, void *env, int delete) {
 			keyname = buf;
 		} else
 			keyname = v.string;
-		if (!(b = dtmatch((Dt_t *)exnode->data.variable.
-			symbol->local.pointer, keyname))) {
+		if (!(b = dtmatch(exnode->data.variable.symbol->local, keyname))) {
 			return 0;
 		}
 	}
 	if (delete) {
-		dtdelete((Dt_t *)exnode->data.variable.symbol->local.pointer, b);
-		free (b);
+		dtdelete(exnode->data.variable.symbol->local, b);
 	}
 	return 1;
 }
@@ -132,15 +129,16 @@ static Extype_t getdyn(Expr_t *ex, Exnode_t *exnode, void *env,
 
 		v = eval(ex, exnode->data.variable.index, env);
 		if (exnode->data.variable.symbol->index_type == INTEGER) {
-			if (!(b = dtmatch((Dt_t *) exnode->data.variable.symbol->local.pointer, &v)))
+			if (!(b = dtmatch(exnode->data.variable.symbol->local, &v)))
 			{
-				if (!(b = calloc(1, sizeof(Exassoc_t))))
+				if (!(b = vmalloc(ex->vm, sizeof(Exassoc_t))))
 					exnospace();
+				*b = (Exassoc_t){0};
 				b->key = v;
-				dtinsert((Dt_t *)exnode->data.variable.symbol->local.pointer, b);
+				dtinsert(exnode->data.variable.symbol->local, b);
 			}
 		} else {
-			int type = exnode->data.variable.index->type;
+			const long type = exnode->data.variable.index->type;
 			if (type != STRING) {
 				if (!BUILTIN(type)) {
 					key = ex->disc->keyf(v, type);
@@ -150,13 +148,14 @@ static Extype_t getdyn(Expr_t *ex, Exnode_t *exnode, void *env,
 				keyname = buf;
 			} else
 				keyname = v.string;
-			if (!(b = dtmatch((Dt_t *)exnode->data.variable.symbol->local.pointer, keyname)))
+			if (!(b = dtmatch(exnode->data.variable.symbol->local, keyname)))
 			{
-				if (!(b = calloc(1, sizeof(Exassoc_t) + strlen(keyname))))
+				if (!(b = vmalloc(ex->vm, sizeof(Exassoc_t) + strlen(keyname))))
 					exnospace();
+				*b = (Exassoc_t){0};
 				strcpy(b->name, keyname);
 				b->key = v;
-				dtinsert((Dt_t *)exnode->data.variable.symbol->local.pointer, b);
+				dtinsert(exnode->data.variable.symbol->local, b);
 			}
 		}
 		*assoc = b;
@@ -190,11 +189,10 @@ typedef struct
 static int
 prformat(void* vp, Sffmt_t* dp)
 {
-	Fmt_t*		fmt = (Fmt_t*)dp;
+	Fmt_t *const fmt = (Fmt_t *)((char *)dp - offsetof(Fmt_t, fmt));
 	Exnode_t*	node;
 	char*		s;
-	int			from;
-	int			to = 0;
+	long to = 0;
 	time_t			tm;
     struct tm *stm;
 
@@ -212,7 +210,7 @@ prformat(void* vp, Sffmt_t* dp)
 	else
 	{
 		node = fmt->actuals->data.operand.left;
-		from = node->type;
+		const long from = node->type;
 		switch (dp->fmt)
 		{
 		case 'f':
@@ -247,11 +245,12 @@ prformat(void* vp, Sffmt_t* dp)
 			{
 				if (fmt->value.string)
 				{
-					size_t n = strlen(fmt->value.string);
-					if ((s = fmtbuf(n + 1)))
-						memcpy(s, fmt->value.string, n + 1);
+					char *const copy = vmstrdup(fmt->expr->vm, fmt->value.string);
+					if (copy == NULL) {
+						exerror("printf: out of memory");
+					}
 					vmfree(fmt->expr->vm, fmt->value.string);
-					fmt->value.string = s;
+					fmt->value.string = copy;
 				}
 				if (!fmt->value.string)
 					fmt->value.string = "";
@@ -269,8 +268,8 @@ prformat(void* vp, Sffmt_t* dp)
 		fmt->fmt.size = sizeof(double);
 		break;
 	default:
-		*(Sflong_t*)vp = fmt->value.integer;
-		dp->size = sizeof(Sflong_t);
+		*(long long *)vp = fmt->value.integer;
+		dp->size = sizeof(long long);
 		break;
 	}
 	strview_t txt = {0};
@@ -281,12 +280,18 @@ prformat(void* vp, Sffmt_t* dp)
 	switch (dp->fmt)
 	{
 	case 'q':
-	case 'Q':
+	case 'Q': {
 		s = *(char**)vp;
-		*(char**)vp = fmtquote(s, "$'", "'", strlen(s));
+		char *quoted = fmtquote(s, "$'", "'");
+		*(char**)vp = vmstrdup(fmt->expr->vm, quoted);
+		free(quoted);
+		if (*(char**)vp == NULL) {
+			exerror("printf: out of memory");
+		}
 		dp->fmt = 's';
 		dp->size = -1;
 		break;
+	}
 	case 'S':
 		dp->flags &= ~SFFMT_LONG;
 		s = *(char**)vp;
@@ -310,15 +315,11 @@ prformat(void* vp, Sffmt_t* dp)
 			}
 			else if (strview_str_eq(txt, "lower"))
 			{
-				for (; *s; s++)
-					if (gv_isupper(*s))
-						*s = (char)tolower(*s);
+				gv_tolower_str(s);
 			}
 			else if (strview_str_eq(txt, "upper"))
 			{
-				for (; *s; s++)
-					if (gv_islower(*s))
-						*s = (char)toupper(*s);
+				gv_toupper_str(s);
 			}
 			else if (strview_str_eq(txt, "variable"))
 			{
@@ -332,23 +333,24 @@ prformat(void* vp, Sffmt_t* dp)
 		break;
 	case 't':
 	case 'T':
-		if ((tm = *(Sflong_t*)vp) == -1)
+		if ((tm = *(long long *)vp) == -1)
 			tm = time(NULL);
         if (txt.data == NULL) {
             exerror("printf: no time format provided");
         } else {
-            s = fmtbuf(TIME_LEN);
+            s = vmalloc(fmt->expr->vm, TIME_LEN);
             stm = localtime(&tm);
             char *format = malloc(sizeof(char) * (txt.size + 1));
-            if (format == NULL) {
+            if (s == NULL || format == NULL) {
+                vmfree(fmt->expr->vm, s);
                 exerror("printf: out of memory");
             } else {
                 strncpy(format, txt.data, txt.size);
                 format[txt.size] = '\0';
                 strftime(s, TIME_LEN, format, stm);
-                free(format);
                 *(char **)vp = s;
             }
+            free(format);
         }
 		dp->fmt = 's';
 		dp->size = -1;
@@ -385,11 +387,11 @@ static int print(Expr_t *ex, Exnode_t *exnode, void *env, FILE *sp) {
 	if (!sp)
 	{
 		v = eval(ex, exnode->data.print.descriptor, env);
-		if (v.integer < 0 || (long long unsigned)v.integer >= elementsof(ex->file) ||
+		if (v.integer < 0 || v.integer >= (long long)elementsof(ex->file) ||
 		    (!(sp = ex->file[v.integer]) &&
 		    !(sp = ex->file[v.integer] = tmpfile())))
 		{
-			exerror("printf: %" PRIdMAX ": invalid descriptor", (intmax_t)v.integer);
+			exerror("printf: %lld: invalid descriptor", v.integer);
 			return -1;
 		}
 	}
@@ -429,7 +431,7 @@ static int print(Expr_t *ex, Exnode_t *exnode, void *env, FILE *sp) {
 static int
 scformat(void* vp, Sffmt_t* dp)
 {
-	Fmt_t*		fmt = (Fmt_t*)dp;
+	Fmt_t *const fmt = (Fmt_t *)((char *)dp - offsetof(Fmt_t, fmt));
 	Exnode_t*	node;
 
 	if (!fmt->actuals)
@@ -441,15 +443,17 @@ scformat(void* vp, Sffmt_t* dp)
 	switch (dp->fmt)
 	{
 	case 'f':
-	case 'g':
+	case 'g': {
 		if (node->type != FLOATING)
 		{
 			exerror("scanf: %s: floating variable address argument expected", node->data.variable.symbol->name);
 			return -1;
 		}
 		fmt->fmt.size = sizeof(double);
-		*(void**)vp = &node->data.variable.symbol->value->data.constant.value;
+		const void *const value = &node->data.variable.symbol->value->data.constant.value;
+		memcpy(vp, &value, sizeof(value));
 		break;
+	}
 	case 's':
 	case '[': {
 		if (node->type != STRING)
@@ -464,28 +468,32 @@ scformat(void* vp, Sffmt_t* dp)
 		vmfree(fmt->expr->vm, s);
 		s = vmalloc(fmt->expr->vm, sizeof(char) * (size_t)fmt->fmt.size);
 		memset(s, 0, sizeof(char) * (size_t)fmt->fmt.size);
-		*(void**)vp = s;
+		memcpy(vp, &s, sizeof(s));
 		node->data.variable.symbol->value->data.constant.value.string = s;
 		break;
 	}
-	case 'c':
+	case 'c': {
 		if (node->type != CHARACTER) {
 			exerror("scanf: %s: char variable address argument expected", node->data.variable.symbol->name);
 			return -1;
 		}
-		fmt->fmt.size = sizeof(Sflong_t);
-		*((void **) vp) = &node->data.variable.symbol->value->data.constant.value;
+		fmt->fmt.size = sizeof(long long);
+		const void *const value = &node->data.variable.symbol->value->data.constant.value;
+		memcpy(vp, &value, sizeof(value));
 		break;
-	default:
+	}
+	default: {
 		if (node->type != INTEGER && node->type != UNSIGNED)
 		{
 			exerror("scanf: %s: integer variable address argument expected", node->data.variable.symbol->name);
 			return -1;
 		}
-		dp->size = sizeof(Sflong_t);
-		*(void**)vp = &node->data.variable.symbol->value->data.constant.value;
+		dp->size = sizeof(long long);
+		const void *const value = &node->data.variable.symbol->value->data.constant.value;
+		memcpy(vp, &value, sizeof(value));
 		break;
 	}
+	}
 	fmt->actuals = fmt->actuals->data.operand.right;
 	dp->flags |= SFFMT_VALUE;
 	return 0;
@@ -511,9 +519,9 @@ static int scan(Expr_t *ex, Exnode_t *exnode, void *env, FILE *sp) {
 		}
 		else
 			v.integer = 0;
-		if (v.integer < 0 || (size_t)v.integer >= elementsof(ex->file) || (!(sp = ex->file[v.integer]) && !(sp = ex->file[v.integer] = tmpfile())))
+		if (v.integer < 0 || v.integer >= (long long)elementsof(ex->file) || (!(sp = ex->file[v.integer]) && !(sp = ex->file[v.integer] = tmpfile())))
 		{
-			exerror("scanf: %" PRIdMAX ": invalid descriptor", (intmax_t)v.integer);
+			exerror("scanf: %lld: invalid descriptor", v.integer);
 			return 0;
 		}
 	}
@@ -533,10 +541,10 @@ static int scan(Expr_t *ex, Exnode_t *exnode, void *env, FILE *sp) {
 		}
 		fputs(v.string, sp);
 		rewind(sp);
-		n = sfscanf(sp, &fmt);
+		n = sfvscanf(sp, &fmt.fmt);
 		fclose(sp);
 	} else {
-		n = sfscanf(sp, &fmt);
+		n = sfvscanf(sp, &fmt.fmt);
 	}
 	if (fmt.actuals && !*fmt.fmt.form)
 		exerror("scanf: %s: too many arguments", fmt.actuals->data.operand.left->data.variable.symbol->name);
@@ -761,17 +769,17 @@ static char *str_mpy(Expr_t *ex, const char *l, const char *r) {
  * Add replacement string.
  * \digit is replaced with a subgroup match, if any.
  */
-static void replace(agxbuf *s, char *base, char *repl, int ng, int *sub) {
+static void replace(agxbuf *s, char *base, char *repl, int ng, size_t *sub) {
   char c;
-  int idx, offset;
+  int idx;
 
   while ((c = *repl++)) {
     if (c == '\\') {
       if ((c = *repl) && gv_isdigit(c)) {
         idx = c - '0';
         if (idx < ng) {
-          offset = sub[2 * idx];
-          agxbput_n(s, base + offset, (size_t)(sub[2 * idx + 1] - offset));
+          const size_t offset = sub[2 * idx];
+          agxbput_n(s, base + offset, sub[2 * idx + 1] - offset);
         }
         repl++;
       } else {
@@ -783,16 +791,13 @@ static void replace(agxbuf *s, char *base, char *repl, int ng, int *sub) {
   }
 }
 
-#define MCNT(s) (sizeof(s)/(2*sizeof(int)))
-
-static void
-addItem (Dt_t* arr, Extype_t v, char* tok)
-{
+static void addItem(Expr_t *ex, Dt_t *arr, Extype_t v, char *tok) {
 	Exassoc_t* b;
 
 	if (!(b = dtmatch(arr, &v))) {
-		if (!(b = calloc(1, sizeof(Exassoc_t))))
+		if (!(b = vmalloc(ex->vm, sizeof(Exassoc_t))))
 	    	exerror("out of space [assoc]");
+		*b = (Exassoc_t){0};
 		b->key = v;
 		dtinsert(arr, b);
 	}
@@ -809,7 +814,7 @@ static Extype_t exsplit(Expr_t *ex, Exnode_t *exnode, void *env) {
 	char *seps;
 	char *tok;
 	size_t sz;
-	Dt_t* arr = (Dt_t*)exnode->data.split.array->local.pointer;
+	Dt_t* arr = exnode->data.split.array->local;
 
 	str = eval(ex, exnode->data.split.string, env).string;
 	if (exnode->data.split.seps)
@@ -822,17 +827,17 @@ static Extype_t exsplit(Expr_t *ex, Exnode_t *exnode, void *env) {
 		sz = strspn (str, seps);
 	    if (sz) {
 			if (v.integer == 0) {  /* initial separator => empty field */
-	    		addItem (arr, v, "");
+	    		addItem(ex, arr, v, "");
 	    		v.integer++;
 			}
 			for (size_t i = 1; i < sz; i++) {
-	    		addItem (arr, v, "");
+	    		addItem(ex, arr, v, "");
 	    		v.integer++;
 			}
 		}
 		str += sz;
 		if (*str == '\0') { /* terminal separator => empty field */
-			addItem (arr, v, "");
+			addItem(ex, arr, v, "");
 			v.integer++;
 	    	break;
 		}
@@ -844,7 +849,7 @@ static Extype_t exsplit(Expr_t *ex, Exnode_t *exnode, void *env) {
 			memcpy(tok, str, sz);
 			tok[sz] = '\0';
 		}
-		addItem (arr, v, tok);
+		addItem(ex, arr, v, tok);
 		v.integer++;
 		str += sz;
 	}
@@ -862,7 +867,7 @@ static Extype_t extokens(Expr_t *ex, Exnode_t *exnode, void *env) {
 	char *seps;
 	char *tok;
 	size_t sz;
-	Dt_t* arr = (Dt_t*)exnode->data.split.array->local.pointer;
+	Dt_t* arr = exnode->data.split.array->local;
 
 	str = eval(ex, exnode->data.split.string, env).string;
 	if (exnode->data.split.seps)
@@ -886,7 +891,7 @@ static Extype_t extokens(Expr_t *ex, Exnode_t *exnode, void *env) {
 			memcpy(tok, str, sz);
 			tok[sz] = '\0';
 		}
-		addItem (arr, v, tok);
+		addItem(ex, arr, v, tok);
 		v.integer++;
 		str += sz;
 	}
@@ -904,8 +909,8 @@ static Extype_t exsub(Expr_t *ex, Exnode_t *exnode, void *env, bool global) {
 	char *p;
 	char *s;
 	Extype_t v;
-	int sub[20];
-	int flags = STR_MAXIMAL;
+	size_t sub[20];
+	int flags = 0;
 	int ng;
 
 	str = eval(ex, exnode->data.string.base, env).string;
@@ -940,7 +945,7 @@ static Extype_t exsub(Expr_t *ex, Exnode_t *exnode, void *env, bool global) {
 		return v;
 	}
 
-	ng = strgrpmatch(str, pat, sub, MCNT(sub), flags);
+	ng = strgrpmatch(str, pat, sub, sizeof(sub) / (sizeof(sub[0]) * 2), flags);
 	if (ng == 0) {
 		v.string = vmstrdup(ex->ve, str);
 		return v;
@@ -953,7 +958,7 @@ static Extype_t exsub(Expr_t *ex, Exnode_t *exnode, void *env, bool global) {
 
 	agxbuf buffer = {0};
 
-	agxbput_n(&buffer, str, (size_t)sub[0]);
+	agxbput_n(&buffer, str, sub[0]);
 
 	if (repl) {
 		replace(&buffer, str, repl, ng, sub);
@@ -961,8 +966,8 @@ static Extype_t exsub(Expr_t *ex, Exnode_t *exnode, void *env, bool global) {
 
 	s = str + sub[1];
 	if (global) {
-		while ((ng = strgrpmatch(s, pat, sub, MCNT(sub), flags))) {
-			agxbput_n(&buffer, s, (size_t)sub[0]);
+		while ((ng = strgrpmatch(s, pat, sub, sizeof(sub) / (sizeof(sub[0]) * 2), flags))) {
+			agxbput_n(&buffer, s, sub[0]);
 			if (repl) {
 				replace(&buffer, s, repl, ng, sub);
 			}
@@ -991,13 +996,12 @@ static Extype_t exsubstr(Expr_t *ex, Exnode_t *exnode, void *env) {
 	len = strlen(s.string);
 	i = eval(ex, exnode->data.string.pat, env);
 	if (i.integer < 0 || len < i.integer)
-		exerror("illegal start index in substr(%s,%" PRIdMAX ")", s.string,
-		        (intmax_t)i.integer);
+		exerror("illegal start index in substr(%s,%lld)", s.string, i.integer);
 	if (exnode->data.string.repl) {
 		l = eval(ex, exnode->data.string.repl, env);
 		if (l.integer < 0 || len - i.integer < l.integer)
-	    exerror("illegal length in substr(%s,%" PRIdMAX ",%" PRIdMAX ")",
-	            s.string, (intmax_t)i.integer, (intmax_t)l.integer);
+	    exerror("illegal length in substr(%s,%lld,%lld)",
+	            s.string, i.integer, l.integer);
 	} else
 		l.integer = len - i.integer;
 
@@ -1013,7 +1017,7 @@ static Extype_t exsubstr(Expr_t *ex, Exnode_t *exnode, void *env) {
 /* xConvert:
  * Convert from external type.
  */
-static void xConvert(Expr_t *ex, Exnode_t *exnode, int type, Extype_t v,
+static void xConvert(Expr_t *ex, Exnode_t *exnode, long type, Extype_t v,
 	 Exnode_t * tmp)
 {
 	*tmp = *exnode->data.operand.left;
@@ -1115,8 +1119,6 @@ static Extype_t eval(Expr_t *ex, Exnode_t *exnode, void *env) {
 			if (x->type == STRING)
 			{
 				v.string = vmstrdup(ex->vm, v.string);
-				if ((e = assoc ? assoc->value.string : x->data.variable.symbol->value->data.constant.value.string))
-					vmfree(ex->vm, e);
 			}
 			if (assoc)
 				assoc->value = v;
@@ -1130,8 +1132,7 @@ static Extype_t eval(Expr_t *ex, Exnode_t *exnode, void *env) {
 			else
 				i.integer = EX_SCALAR;
 			if (x->data.variable.dyna) {
-				Extype_t locv;
-				locv = getdyn(ex, x->data.variable.dyna, env, &assoc);
+				Extype_t locv = getdyn(ex, x->data.variable.dyna, env, &assoc);
 				x->data.variable.dyna->data.variable.dyna->data.constant.value = locv;
 			}
 			if (ex->disc->setf(ex, x, x->data.variable.symbol,
@@ -1167,7 +1168,7 @@ static Extype_t eval(Expr_t *ex, Exnode_t *exnode, void *env) {
 	case EXIT:
 		v = eval(ex, x, env);
 		if (ex->disc->exitf)
-			ex->disc->exitf(ex, env, (int)v.integer);
+			ex->disc->exitf(env, (int)v.integer);
 		else
 			graphviz_exit((int)v.integer);
 		UNREACHABLE();
@@ -1225,9 +1226,7 @@ static Extype_t eval(Expr_t *ex, Exnode_t *exnode, void *env) {
 							break;
 						continue;
 					case STRING:
-						if ((ex->disc->version >= 19981111L && ex->disc->matchf)
-						      ? ex->disc->matchf((*t)->string, v.string)
-						      : strmatch((*t)->string, v.string))
+						if (strmatch((*t)->string, v.string))
 							break;
 						continue;
 					case FLOATING:
@@ -1265,7 +1264,7 @@ static Extype_t eval(Expr_t *ex, Exnode_t *exnode, void *env) {
 		if (exnode->data.generate.array->op == DYNAMIC)
 		{
 			n = exnode->data.generate.index->type == STRING;
-			for (assoc = dtfirst((Dt_t*)exnode->data.generate.array->data.variable.symbol->local.pointer); assoc; assoc = dtnext((Dt_t*)exnode->data.generate.array->data.variable.symbol->local.pointer, assoc))
+			for (assoc = dtfirst(exnode->data.generate.array->data.variable.symbol->local); assoc; assoc = dtnext(exnode->data.generate.array->data.variable.symbol->local, assoc))
 			{
 				v.integer++;
 				if (n)
@@ -1297,15 +1296,13 @@ static Extype_t eval(Expr_t *ex, Exnode_t *exnode, void *env) {
 			}
 		}
 		return v;
-    case ITERATER:
+    case ITERATOR:
 		v.integer = 0;
 		if (exnode->data.generate.array->op == DYNAMIC) {
 			n = exnode->data.generate.index->type == STRING;
-			for (assoc = dtlast((Dt_t *) exnode->data.generate.array->
-						   data.variable.symbol->local.
-						   pointer); assoc;
-		 		assoc = dtprev((Dt_t *) exnode->data.generate.array->
-						  data.variable.symbol->local.pointer,
+			for (assoc = dtlast(exnode->data.generate.array->
+						   data.variable.symbol->local); assoc;
+		 		assoc = dtprev(exnode->data.generate.array->data.variable.symbol->local,
 						  assoc)) {
 				v.integer++;
 				if (n)
@@ -1333,7 +1330,16 @@ static Extype_t eval(Expr_t *ex, Exnode_t *exnode, void *env) {
 		}
 		return v;
 	case '#':
-		v.integer = dtsize ((Dt_t*)exnode->data.variable.symbol->local.pointer);
+		if (exnode->data.variable.symbol->index > 0) {
+			if (ex->disc->lengthf != NULL) {
+				v = ex->disc->lengthf(exnode->data.variable.symbol, ex->disc);
+			} else {
+				exerror("%s: cannot get length", x->data.variable.symbol->name);
+				v.integer = 0;
+			}
+		} else {
+			v.integer = dtsize(exnode->data.variable.symbol->local);
+		}
 		return v;
 	case IN_OP:
 		v.integer = evaldyn (ex, exnode, env, 0);
@@ -1343,7 +1349,7 @@ static Extype_t eval(Expr_t *ex, Exnode_t *exnode, void *env) {
 			v.integer = evaldyn (ex, exnode, env, 1);
 		}
 		else {
-			dtclear ((Dt_t*)exnode->data.variable.symbol->local.pointer);
+			dtclear(exnode->data.variable.symbol->local);
 			v.integer = 0;
 		}
 		return v;
@@ -1351,7 +1357,7 @@ static Extype_t eval(Expr_t *ex, Exnode_t *exnode, void *env) {
 		x = exnode->data.call.args;
 		for (n = 0, a = exnode->data.call.procedure->value->data.procedure.args; a && x; a = a->data.operand.right)
 		{
-			if (n < elementsof(args))
+			if (n < (int)elementsof(args))
 			{
 				save[n] = a->data.operand.left->data.variable.symbol->value->data.constant.value;
 				args[n++] = eval(ex, x->data.operand.left, env);
@@ -1360,19 +1366,22 @@ static Extype_t eval(Expr_t *ex, Exnode_t *exnode, void *env) {
 				a->data.operand.left->data.variable.symbol->value->data.constant.value = eval(ex, x->data.operand.left, env);
 			x = x->data.operand.right;
 		}
-		for (n = 0, a = exnode->data.call.procedure->value->data.procedure.args; a && n < elementsof(save); a = a->data.operand.right)
+		for (n = 0, a = exnode->data.call.procedure->value->data.procedure.args;
+		     a && n < (int)elementsof(save); a = a->data.operand.right)
 			a->data.operand.left->data.variable.symbol->value->data.constant.value = args[n++];
 		if (x)
 			exerror("too many actual args");
 		else if (a)
 			exerror("not enough actual args");
 		v = exeval(ex, exnode->data.call.procedure->value->data.procedure.body, env);
-		for (n = 0, a = exnode->data.call.procedure->value->data.procedure.args; a && n < elementsof(save); a = a->data.operand.right)
+		for (n = 0, a = exnode->data.call.procedure->value->data.procedure.args;
+		     a && n < (int)elementsof(save); a = a->data.operand.right)
 			a->data.operand.left->data.variable.symbol->value->data.constant.value = save[n++];
 		return v;
 	case ARRAY:
 		n = 0;
-		for (x = exnode->data.operand.right; x && n < elementsof(args); x = x->data.operand.right)
+		for (x = exnode->data.operand.right; x && n < (int)elementsof(args);
+		     x = x->data.operand.right)
 			args[n++] = eval(ex, x->data.operand.left, env);
 		return ex->disc->getf(ex, exnode->data.operand.left,
 		                      exnode->data.operand.left->data.variable.symbol,
@@ -1381,7 +1390,8 @@ static Extype_t eval(Expr_t *ex, Exnode_t *exnode, void *env) {
 	case FUNCTION:
 		n = 0;
 		args[n++].string = env;
-		for (x = exnode->data.operand.right; x && n < elementsof(args); x = x->data.operand.right)
+		for (x = exnode->data.operand.right; x && n < (int)elementsof(args);
+		     x = x->data.operand.right)
 			args[n++] = eval(ex, x->data.operand.left, env);
 		return ex->disc->getf(ex, exnode->data.operand.left,
 		                      exnode->data.operand.left->data.variable.symbol,
@@ -1393,8 +1403,7 @@ static Extype_t eval(Expr_t *ex, Exnode_t *exnode, void *env) {
 		else
 			i.integer = EX_SCALAR;
 		if (exnode->data.variable.dyna) {
-			Extype_t locv;
-			locv = getdyn(ex, exnode->data.variable.dyna, env, &assoc);
+			Extype_t locv = getdyn(ex, exnode->data.variable.dyna, env, &assoc);
 			exnode->data.variable.dyna->data.variable.dyna->data.constant.value = locv;
 		}
 		return ex->disc->getf(ex, exnode, exnode->data.variable.symbol,
@@ -1425,13 +1434,16 @@ static Extype_t eval(Expr_t *ex, Exnode_t *exnode, void *env) {
 			graphviz_exit(EXIT_FAILURE);
 		}
 		print(ex, exnode, env, buffer);
-		size_t size = (size_t)ftell(buffer);
+		const size_t size = gv_ftell(buffer);
 		rewind(buffer);
 		v.string = vmalloc(ex->ve, size + 1);
 		if (v.string == NULL) {
 			v.string = exnospace();
 		} else {
-			fread(v.string, size, 1, buffer);
+			if (fread(v.string, size, 1, buffer) < 1) {
+				fprintf(stderr, "failed to read back temporary file\n");
+				graphviz_exit(EXIT_FAILURE);
+			}
 			v.string[size] = '\0';
 		}
 		fclose(buffer);
@@ -1483,25 +1495,25 @@ static Extype_t eval(Expr_t *ex, Exnode_t *exnode, void *env) {
 					if ((r.integer = r.floating) == 0)
 						exerror("floating 0 modulus");
 					else
-						v.floating = (Sflong_t)v.floating % r.integer;
+						v.floating = (long long)v.floating % r.integer;
 					break;
 				case '&':
-					v.floating = (Sflong_t)v.floating & (Sflong_t)r.floating;
+					v.floating = (long long)v.floating & (long long)r.floating;
 					break;
 				case '|':
-					v.floating = (Sflong_t)v.floating | (Sflong_t)r.floating;
+					v.floating = (long long)v.floating | (long long)r.floating;
 					break;
 				case '^':
-					v.floating = (Sflong_t)v.floating ^ (Sflong_t)r.floating;
+					v.floating = (long long)v.floating ^ (long long)r.floating;
 					break;
 				case LSH:
-					v.floating = (Sflong_t)v.floating << (Sflong_t)r.floating;
+					v.floating = (long long)v.floating << (long long)r.floating;
 					break;
 				case RSH:
 #ifdef _WIN32
-					v.floating = (Sflong_t)((Sfulong_t)v.floating >> (Sflong_t)r.floating);
+					v.floating = (long long)((unsigned long long)v.floating >> (long long)r.floating);
 #else
-					v.floating = (Sfulong_t)v.floating >> (Sflong_t)r.floating;
+					v.floating = (unsigned long long)v.floating >> (long long)r.floating;
 #endif
 					break;
 				default:
@@ -1546,7 +1558,7 @@ static Extype_t eval(Expr_t *ex, Exnode_t *exnode, void *env) {
 					v.integer <<= r.integer;
 					break;
 				case RSH:
-					v.integer = (Sfulong_t)v.integer >> r.integer;
+					v.integer = (unsigned long long)v.integer >> r.integer;
 					break;
 				default:
 					goto huh;
@@ -1647,10 +1659,10 @@ static Extype_t eval(Expr_t *ex, Exnode_t *exnode, void *env) {
 			tmp.type = exnode->type;
 			return tmp.data.constant.value;
 		case '!':
-			v.floating = !(Sflong_t)v.floating;
+			v.floating = !(long long)v.floating;
 			return v;
 		case '~':
-			v.floating = ~(Sflong_t)v.floating;
+			v.floating = ~(long long)v.floating;
 			return v;
 		case '-':
 			if (x)
@@ -1662,13 +1674,13 @@ static Extype_t eval(Expr_t *ex, Exnode_t *exnode, void *env) {
 			v.floating += r.floating;
 			return v;
 		case '&':
-			v.floating = (Sflong_t)v.floating & (Sflong_t)r.floating;
+			v.floating = (long long)v.floating & (long long)r.floating;
 			return v;
 		case '|':
-			v.floating = (Sflong_t)v.floating | (Sflong_t)r.floating;
+			v.floating = (long long)v.floating | (long long)r.floating;
 			return v;
 		case '^':
-			v.floating = (Sflong_t)v.floating ^ (Sflong_t)r.floating;
+			v.floating = (long long)v.floating ^ (long long)r.floating;
 			return v;
 		case '*':
 			v.floating *= r.floating;
@@ -1683,7 +1695,7 @@ static Extype_t eval(Expr_t *ex, Exnode_t *exnode, void *env) {
 			if ((r.integer = r.floating) == 0)
 				exerror("floating 0 modulus");
 			else
-				v.floating = (Sflong_t)v.floating % r.integer;
+				v.floating = (long long)v.floating % r.integer;
 			return v;
 		case '<':
 			v.integer = v.floating < r.floating;
@@ -1704,10 +1716,10 @@ static Extype_t eval(Expr_t *ex, Exnode_t *exnode, void *env) {
 			v.integer = v.floating > r.floating;
 			return v;
 		case LSH:
-			v.integer = (Sflong_t) ((Sfulong_t)v.floating << (Sflong_t)r.floating);
+			v.integer = (long long)((unsigned long long)v.floating << (long long)r.floating);
 			return v;
 		case RSH:
-			v.integer = (Sflong_t) ((Sfulong_t)v.floating >> (Sflong_t)r.floating);
+			v.integer = (long long)((unsigned long long)v.floating >> (long long)r.floating);
 			return v;
 		}
 		break;
@@ -1746,16 +1758,16 @@ static Extype_t eval(Expr_t *ex, Exnode_t *exnode, void *env) {
 		switch (exnode->op)
 		{
 		case '<':
-			v.integer = (Sfulong_t)v.integer < (Sfulong_t)r.integer;
+			v.integer = (unsigned long long)v.integer < (unsigned long long)r.integer;
 			return v;
 		case LE:
-			v.integer = (Sfulong_t)v.integer <= (Sfulong_t)r.integer;
+			v.integer = (unsigned long long)v.integer <= (unsigned long long)r.integer;
 			return v;
 		case GE:
-			v.integer = (Sfulong_t)v.integer >= (Sfulong_t)r.integer;
+			v.integer = (unsigned long long)v.integer >= (unsigned long long)r.integer;
 			return v;
 		case '>':
-			v.integer = (Sfulong_t)v.integer > (Sfulong_t)r.integer;
+			v.integer = (unsigned long long)v.integer > (unsigned long long)r.integer;
 			return v;
 		}
 		/*FALLTHROUGH*/
@@ -1768,7 +1780,7 @@ static Extype_t eval(Expr_t *ex, Exnode_t *exnode, void *env) {
 			v.floating = v.integer;
 #else
 			if (exnode->type == UNSIGNED)
-				v.floating = (Sfulong_t)v.integer;
+				v.floating = (unsigned long long)v.integer;
 			else
 				v.floating = v.integer;
 #endif
@@ -1782,7 +1794,7 @@ static Extype_t eval(Expr_t *ex, Exnode_t *exnode, void *env) {
 				if (exnode->data.operand.left->type == UNSIGNED)
 					str = exprintf(ex->ve, "%llu", (unsigned long long)v.integer);
 				else
-					str = exprintf(ex->ve, "%lld", (long long)v.integer);
+					str = exprintf(ex->ve, "%lld", v.integer);
 				tmp.data.constant.value.string = str;
 			}
 			else if (ex->disc->convertf(&tmp, STRING, 0)) {
@@ -1790,7 +1802,7 @@ static Extype_t eval(Expr_t *ex, Exnode_t *exnode, void *env) {
 				if (exnode->data.operand.left->type == UNSIGNED)
 					str = exprintf(ex->ve, "%llu", (unsigned long long)v.integer);
 				else
-					str = exprintf(ex->ve, "%lld", (long long)v.integer);
+					str = exprintf(ex->ve, "%lld", v.integer);
 				tmp.data.constant.value.string = str;
 			}
 			tmp.type = STRING;
@@ -1848,10 +1860,10 @@ static Extype_t eval(Expr_t *ex, Exnode_t *exnode, void *env) {
 			v.integer = v.integer != r.integer;
 			return v;
 		case LSH:
-			v.integer = (Sflong_t)v.integer << (Sflong_t)r.integer;
+			v.integer = v.integer << r.integer;
 			return v;
 		case RSH:
-			v.integer = (Sfulong_t)v.integer >> (Sflong_t)r.integer;
+			v.integer = (unsigned long long)v.integer >> r.integer;
 			return v;
 		case '<':
 			v.integer = v.integer < r.integer;
@@ -1909,9 +1921,7 @@ static Extype_t eval(Expr_t *ex, Exnode_t *exnode, void *env) {
 		case EQ:
 		case NE:
 			v.integer = ((v.string && r.string)
-			              ? ((ex->disc->version >= 19981111L && ex->disc->matchf)
-			                ? ex->disc->matchf(v.string, r.string)
-			                : strmatch(v.string, r.string))
+			              ? strmatch(v.string, r.string)
 			              : (v.string == r.string)) == (exnode->op == EQ);
 			return v;
 		case '+':
@@ -1951,11 +1961,19 @@ static Extype_t eval(Expr_t *ex, Exnode_t *exnode, void *env) {
 		}
 		goto huh;
 	}
- huh:
-	if (exnode->binary)
-		exerror("operator %s %s %s not implemented", lexname(exnode->data.operand.left->type, -1), lexname(exnode->op, exnode->subop), exnode->data.operand.right ? lexname(exnode->data.operand.right->type, -1) : "UNARY");
-	else
-		exerror("operator %s %s not implemented", lexname(exnode->op, exnode->subop), lexname(exnode->data.operand.left->type, -1));
+ huh:;
+	char *left = lexname(exnode->data.operand.left->type, -1);
+	char *op = lexname(exnode->op, exnode->subop);
+	if (exnode->binary) {
+		char *right = exnode->data.operand.right
+		            ? lexname(exnode->data.operand.right->type, -1) : NULL;
+		exerror("operator %s %s %s not implemented", left, op,
+		        right ? right : "UNARY");
+		free(right);
+	} else
+		exerror("operator %s %s not implemented", op, left);
+	free(op);
+	free(left);
 	return exzero(exnode->type);
 }
 
