@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -17,34 +17,27 @@ extern "C" {
  * grammar support routines
  * stuffed in a header so exparse.y can work
  * with both yacc and bison
- * sometimes free stuff can cost a lot
  */
 
 #if !defined(_EXGRAM_H) && ( defined(MINTOKEN) || defined(YYTOKENTYPE) )
 #define _EXGRAM_H
 
-#if !defined(_EXPARSE_H)
-#define _EXPARSE_H
-#endif
-
-#include <cgraph/agxbuf.h>
 #include <expr/exlib.h>
+#include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <string.h>
+#include <util/agxbuf.h>
+#include <util/gv_ctype.h>
 
 #define ex_lex()		extoken_fn(expr.program)
 
 #define ALLOCATE(p,x)	exalloc(p,sizeof(x))
-#define QUALIFY(r,s)	((r)&&(expr.program->disc->flags&EX_QUALIFY)?qualify(r,s):(s))
 
 static int		a2t[] = { 0, FLOATING, INTEGER, STRING };
-static Switch_t		swstate;
 
 Exstate_t		expr;
 
-static int T(int t) 
-{
+static int T(long t) {
 	if (expr.program->disc->types)
 	    return expr.program->disc->types[t & TMASK];
 	else
@@ -55,18 +48,15 @@ static int T(int t)
  * allocate and initialize a new expression node in the current program
  */
 
-Exnode_t*
-exnewnode(Expr_t* p, int op, int binary, int type, Exnode_t* left, Exnode_t* right)
-{
+Exnode_t *exnewnode(Expr_t *p, long op, bool binary, long type, Exnode_t *left,
+                    Exnode_t *right) {
 	Exnode_t*	x;
 
 	x = ALLOCATE(p, Exnode_t);
-	memzero(x, sizeof(*x));
+	*x = (Exnode_t){0};
 	x->op = op;
 	x->type = type;
 	x->binary = binary;
-	x->local.number = 0;
-	x->local.pointer = 0;
 	x->data.operand.left = left;
 	x->data.operand.right = right;
 	return x;
@@ -99,29 +89,29 @@ exfreenode(Expr_t* p, Exnode_t* x)
 	case DYNAMIC:
 		if (x->data.variable.index)
 			exfreenode(p, x->data.variable.index);
-		if (x->data.variable.symbol->local.pointer)
+		if (x->data.variable.symbol->local)
 		{
-			dtclose((Dt_t*)x->data.variable.symbol->local.pointer);
-			x->data.variable.symbol->local.pointer = 0;
+			dtclose(x->data.variable.symbol->local);
+			x->data.variable.symbol->local = NULL;
 		}
 		break;
 	case '#':
-		if (x->data.variable.symbol->local.pointer) {
-			dtclose((Dt_t *) x->data.variable.symbol->local.pointer);
-			x->data.variable.symbol->local.pointer = 0;
+		if (x->data.variable.symbol->local) {
+			dtclose(x->data.variable.symbol->local);
+			x->data.variable.symbol->local = NULL;
 		}
 		break;
 //	case IN_OP:
 	case UNSET:
 		if (x->data.variable.index)
 			exfreenode(p, x->data.variable.index);
-		if (x->data.variable.symbol->local.pointer) {
-			dtclose((Dt_t *) x->data.variable.symbol->local.pointer);
-			x->data.variable.symbol->local.pointer = 0;
+		if (x->data.variable.symbol->local) {
+			dtclose(x->data.variable.symbol->local);
+			x->data.variable.symbol->local = NULL;
 		}
 		break;
 	case ITERATE:
-	case ITERATER:
+	case ITERATOR:
 		if (x->data.generate.statement)
 			exfreenode(p, x->data.generate.statement);
 		break;
@@ -148,9 +138,9 @@ exfreenode(Expr_t* p, Exnode_t* x)
 		if (x->data.split.seps)
 			exfreenode(p, x->data.split.seps);
 		exfreenode(p, x->data.split.string);
-		if (x->data.split.array->local.pointer) {
-			dtclose((Dt_t *) x->data.split.array->local.pointer);
-			x->data.split.array->local.pointer = 0;
+		if (x->data.split.array->local) {
+			dtclose(x->data.split.array->local);
+			x->data.split.array->local = NULL;
 		}
 		break;
 	case PRINT:
@@ -172,6 +162,12 @@ exfreenode(Expr_t* p, Exnode_t* x)
 			vmfree(p->vm, pr);
 		}
 		break;
+	case PROCEDURE:
+		if (x->data.procedure.args)
+			exfreenode(p, x->data.procedure.args);
+		if (x->data.procedure.body)
+			exfreenode(p, x->data.procedure.body);
+		break;
 	default:
 		if (x->data.operand.left)
 			exfreenode(p, x->data.operand.left);
@@ -205,10 +201,11 @@ static Exnode_t *extract(Expr_t * p, Exnode_t ** argp, int type) {
  * Generate split/tokens node.
  * Fifth argument is optional.
  */
-static Exnode_t *exnewsplit(Expr_t * p, int op, Exid_t* dyn, Exnode_t * s, Exnode_t* seps) {
+static Exnode_t *exnewsplit(Expr_t *p, long op, Exid_t *dyn, Exnode_t *s,
+                            Exnode_t *seps) {
 	Exnode_t *ss = 0;
 
-	if (dyn->local.pointer == 0)
+	if (dyn->local == NULL)
               	exerror("cannot use non-array %s in %s", dyn->name, exopname(op));
 	if ((dyn->index_type > 0) && (dyn->index_type != INTEGER))
             exerror("in %s, array %s must have integer index type, not %s", 
@@ -222,7 +219,7 @@ static Exnode_t *exnewsplit(Expr_t * p, int op, Exid_t* dyn, Exnode_t * s, Exnod
 	if (seps && (seps->type != STRING))
             exerror("third argument to %s must have string type, not %s", 
 		exopname(op), extypename(p, seps->type));
-	ss = exnewnode(p, op, 0, INTEGER, NULL, NULL);
+	ss = exnewnode(p, op, false, INTEGER, NULL, NULL);
 	ss->data.split.array = dyn;
 	ss->data.split.string = s;
 	ss->data.split.seps = seps;
@@ -253,7 +250,7 @@ static Exnode_t *exnewsub(Expr_t * p, Exnode_t * args, int op) {
 	    repl = 0;
 	if (args)
 	    exerror("too many arguments to sub operator");
-	ss = exnewnode(p, op, 0, STRING, NULL, NULL);
+	ss = exnewnode(p, op, false, STRING, NULL, NULL);
 	ss->data.string.base = base;
 	ss->data.string.pat = pat;
 	ss->data.string.repl = repl;
@@ -283,127 +280,13 @@ static Exnode_t *exnewsubstr(Expr_t * p, Exnode_t * args) {
 	    repl = 0;
 	if (args)
 	    exerror("too many arguments to substr operator");
-	ss = exnewnode(p, SUBSTR, 0, STRING, NULL, NULL);
+	ss = exnewnode(p, SUBSTR, false, STRING, NULL, NULL);
 	ss->data.string.base = base;
 	ss->data.string.pat = pat;
 	ss->data.string.repl = repl;
 	return ss;
 }
 
-/* exstringOf:
- * Cast x to type STRING
- * Assume x->type != STRING
- */
-static Exnode_t *exstringOf(Expr_t * p, Exnode_t * x) {
-	int type = x->type;
-	int cvt = 0;
-
-	if (!type) {
-	    x->type = STRING;
-	    return x;
-	}
-	if (!BUILTIN(type) && !p->disc->stringof)
-	    exerror("cannot convert %s to STRING", extypename(p, type));
-	if (x->op != CONSTANT) {
-	    if (!BUILTIN(type)) {
-		if (p->disc->stringof(p, x, 1) < 0) {
-		    exerror("cannot convert %s to STRING",
-			    extypename(p, type));
-		}
-		cvt = XPRINT;
-	    } else
-		switch (type) {
-		case FLOATING:
-		    cvt = F2S;
-		    break;
-		case INTEGER:
-		    cvt = I2S;
-		    break;
-		}
-	    x = exnewnode(p, cvt, 0, STRING, x, 0);
-	} else if (!BUILTIN(type)) {
-	    if (p->disc->stringof(p, x, 0) < 0)
-		exerror("cannot convert constant %s to STRING",
-			extypename(p, x->type));
-	} else
-	    switch (type) {
-	    case FLOATING:
-		x->data.constant.value.string =
-		  exprintf(p->vm, "%g", x->data.constant.value.floating);
-		break;
-	    case INTEGER:
-		x->data.constant.value.string =
-		  exprintf(p->vm, "%lld", (long long)x->data.constant.value.integer);
-		break;
-	    default:
-		exerror("internal error: %d: unknown type", type);
-		break;
-	    }
-	x->type = STRING;
-	return x;
-}
-
-/* exprint:
- * Generate argument list of strings.
- */
-static Exnode_t *exprint(Expr_t * p, Exid_t * ex, Exnode_t * args) {
-	Exnode_t *arg = args;
-	Exnode_t *pr;
-
-	while (arg) {
-	    if (arg->data.operand.left->type != STRING)
-		arg->data.operand.left =
-		    exstringOf(p, arg->data.operand.left);
-	    arg = arg->data.operand.right;
-	}
-	pr = exnewnode(p, ex->index, 1, ex->type, args, NULL);
-	return pr;
-}
-
-/* makeVar:
- *
- * Create variable from s[idx].refs 
- * If s is DYNAMIC, refs is non-empty and dyna represents s[idx].
- * The rightmost element in s[idx].refs becomes the dominant symbol,
- * and the prefix gets stored in refs. (This format is used to simplify
- * the yacc parser.)
- */
-static Exnode_t *makeVar(Expr_t * prog, Exid_t * s, Exnode_t * idx,
-			     Exnode_t * dyna, Exref_t * refs) {
-	Exnode_t *nn;
-	int kind;
-	Exid_t *sym;
-
-	/* parse components */
-	if (refs) {
-	    if (refs->next) {
-		sym = refs->next->symbol;
-		refs->next->symbol = refs->symbol;
-	    } else
-		sym = refs->symbol;
-	    refs->symbol = s;
-	    refs->index = idx;
-	} else
-	    sym = s;
-
-	if (sym->type)
-	    kind = sym->type;
-	else
-	    kind = STRING;
-
-	nn = exnewnode(prog, ID, 0, kind, NULL, NULL);
-	nn->data.variable.symbol = sym;
-	nn->data.variable.reference = refs;
-	nn->data.variable.index = 0;
-	nn->data.variable.dyna = dyna;
-	if (!prog->disc->getf)
-	    exerror("%s: identifier references not supported", sym->name);
-	else if (expr.program->disc->reff)
-	    expr.program->disc->reff(prog, nn, nn->data.variable.symbol, refs);
-
-	return nn;
-}
-
 /*
  * cast x to type
  */
@@ -429,12 +312,116 @@ static int	typecast[6][6] =
 
 #define EXTERNAL(t)	((t)>=F2X)
 
-char *extypename(Expr_t * p, int type) {
+char *extypename(Expr_t *p, long type) {
 	if (BUILTIN(type))
 	    return TYPENAME(type);
 	return p->disc->typename(type);
 }
 
+/* exstringOf:
+ * Cast x to type STRING
+ * Assume x->type != STRING
+ */
+static Exnode_t *exstringOf(Expr_t * p, Exnode_t * x) {
+	const long type = x->type;
+	int cvt = 0;
+
+	if (!type) {
+	    x->type = STRING;
+	    return x;
+	}
+	if (!BUILTIN(type) && !p->disc->stringof)
+	    exerror("cannot convert %s to STRING", extypename(p, type));
+	if (x->op != CONSTANT) {
+	    if (!BUILTIN(type)) {
+		if (p->disc->stringof(p, x, 1) < 0) {
+		    exerror("cannot convert %s to STRING",
+			    extypename(p, type));
+		}
+		cvt = XPRINT;
+	    } else if (TYPEINDEX(type) != 0) {
+		cvt = TYPECAST(type, STRING);
+	    }
+	    x = exnewnode(p, cvt, false, STRING, x, 0);
+	} else if (!BUILTIN(type)) {
+	    if (p->disc->stringof(p, x, 0) < 0)
+		exerror("cannot convert constant %s to STRING",
+			extypename(p, x->type));
+	} else
+	    switch (type) {
+	    case FLOATING:
+		x->data.constant.value.string =
+		  exprintf(p->vm, "%g", x->data.constant.value.floating);
+		break;
+	    case INTEGER:
+		x->data.constant.value.string =
+		  exprintf(p->vm, "%lld", x->data.constant.value.integer);
+		break;
+	    default:
+		exerror("internal error: %ld: unknown type", type);
+		break;
+	    }
+	x->type = STRING;
+	return x;
+}
+
+/* exprint:
+ * Generate argument list of strings.
+ */
+static Exnode_t *exprint(Expr_t * p, Exid_t * ex, Exnode_t * args) {
+	Exnode_t *arg = args;
+	Exnode_t *pr;
+
+	while (arg) {
+	    if (arg->data.operand.left->type != STRING)
+		arg->data.operand.left =
+		    exstringOf(p, arg->data.operand.left);
+	    arg = arg->data.operand.right;
+	}
+	pr = exnewnode(p, ex->index, true, ex->type, args, NULL);
+	return pr;
+}
+
+/* makeVar:
+ *
+ * Create variable from s[idx].refs 
+ * If s is DYNAMIC, refs is non-empty and dyna represents s[idx].
+ * The rightmost element in s[idx].refs becomes the dominant symbol,
+ * and the prefix gets stored in refs. (This format is used to simplify
+ * the yacc parser.)
+ */
+static Exnode_t *makeVar(Expr_t * prog, Exid_t * s, Exnode_t * idx,
+			     Exnode_t * dyna, Exref_t * refs) {
+	Exnode_t *nn;
+	Exid_t *sym;
+
+	/* parse components */
+	if (refs) {
+	    if (refs->next) {
+		sym = refs->next->symbol;
+		refs->next->symbol = refs->symbol;
+	    } else
+		sym = refs->symbol;
+	    refs->symbol = s;
+	    refs->index = idx;
+	} else
+	    sym = s;
+
+	const long kind = sym->type ? sym->type : STRING;
+
+	nn = exnewnode(prog, ID, false, kind, NULL, NULL);
+	nn->data.variable.symbol = sym;
+	nn->data.variable.reference = refs;
+	nn->data.variable.index = 0;
+	nn->data.variable.dyna = dyna;
+	if (!prog->disc->getf)
+	    exerror("%s: identifier references not supported", sym->name);
+	else if (expr.program->disc->reff)
+	    expr.program->disc->reff(prog, nn, nn->data.variable.symbol, refs);
+
+	return nn;
+}
+
 /* exnoncast:
  * Return first non-cast node.
  */
@@ -444,9 +431,7 @@ Exnode_t *exnoncast(Exnode_t * x) {
 	return x;
 }
 
-Exnode_t*
-excast(Expr_t* p, Exnode_t* x, int type, Exnode_t* xref, int arg)
-{
+Exnode_t *excast(Expr_t *p, Exnode_t *x, long type, Exnode_t *xref, int arg) {
 	int	t2t;
 	char*		s;
 	char*		e;
@@ -482,7 +467,7 @@ excast(Expr_t* p, Exnode_t* x, int type, Exnode_t* xref, int arg)
 					}
 				}
 			}
-			x = exnewnode(p, t2t, 0, type, x, xref);
+			x = exnewnode(p, t2t, false, type, x, xref);
 		}
 		else switch (t2t)
 		{
@@ -513,11 +498,11 @@ excast(Expr_t* p, Exnode_t* x, int type, Exnode_t* xref, int arg)
 			break;
 		case I2S:
 			x->data.constant.value.string =
-			  exprintf(p->vm, "%lld", (long long)x->data.constant.value.integer);
+			  exprintf(p->vm, "%lld", x->data.constant.value.integer);
 			break;
 		case S2F:
 			s =  x->data.constant.value.string;
-			x->data.constant.value.integer = strtod(s, &e);
+			x->data.constant.value.floating = strtod(s, &e);
 			if (*e)
 				x->data.constant.value.floating = (*s != 0);
 			break;
@@ -536,42 +521,6 @@ excast(Expr_t* p, Exnode_t* x, int type, Exnode_t* xref, int arg)
 	return x;
 }
 
-/*
- * force ref . sym qualification
- */
-
-static Exid_t*
-qualify(Exref_t* ref, Exid_t* sym)
-{
-	Exid_t*	x;
-
-	while (ref->next)
-		ref = ref->next;
-	size_t len = strlen(ref->symbol->name) + strlen(sym->name) + 2;
-	char *s = malloc(sizeof(char) * len);
-	if (s == NULL) {
-		exnospace();
-		return NULL;
-	}
-	snprintf(s, len, "%s.%s", ref->symbol->name, sym->name);
-	if (!(x = dtmatch(expr.program->symbols, s)))
-	{
-		if ((x = calloc(1, sizeof(Exid_t) + strlen(s) - EX_NAMELEN + 1)))
-		{
-			memcpy(x, sym, sizeof(Exid_t) - EX_NAMELEN);
-			strcpy(x->name, s);
-			dtinsert(expr.program->symbols, x);
-		}
-		else
-		{
-			exnospace();
-			x = sym;
-		}
-	}
-	free(s);
-	return x;
-}
-
 /*
  * check function call arg types and count
  * return function identifier node
@@ -580,14 +529,13 @@ qualify(Exref_t* ref, Exid_t* sym)
 static Exnode_t*
 call(Exref_t* ref, Exid_t* fun, Exnode_t* args)
 {
-	int	t;
 	int	type;
 	Exnode_t*	x;
 	int		num;
 
-	x = exnewnode(expr.program, ID, 0, 0, NULL, NULL);
-	t = fun->type;
-	x->data.variable.symbol = fun = QUALIFY(ref, fun);
+	x = exnewnode(expr.program, ID, false, 0, NULL, NULL);
+	long t = fun->type;
+	x->data.variable.symbol = fun;
 	x->data.variable.reference = ref;
 	num = 0;
 	N(t);
@@ -632,7 +580,7 @@ preprint(Exnode_t* args)
 	if (args->data.operand.left->op != CONSTANT)
 	{
 		x = ALLOCATE(expr.program, Print_t);
-		memzero(x, sizeof(*x));
+		*x = (Print_t){0};
 		x->arg = args;
 		return x;
 	}
@@ -660,7 +608,7 @@ preprint(Exnode_t* args)
 		else
 			p = q;
 		x = q;
-		memzero(x, sizeof(*x));
+		*x = (Print_t){0};
 		if (*s)
 		{
 			i = 0;
@@ -673,7 +621,7 @@ preprint(Exnode_t* args)
 					exerror("unterminated %%... in format");
 					goto done;
 				case '*':
-					if (i >= elementsof(x->param))
+					if (i >= (int)elementsof(x->param))
 					{
 						*s = 0;
 						exerror("format %s has too many * arguments", f);
@@ -736,7 +684,7 @@ preprint(Exnode_t* args)
 					t = STRING;
 					goto specified;
 				default:
-					if (isalpha(c))
+					if (gv_isalpha(c))
 						goto specified;
 					break;
 				}
@@ -773,12 +721,18 @@ preprint(Exnode_t* args)
 			{
 			case FLOATING:
 				if (x->arg->type != FLOATING)
-					x->arg = exnewnode(expr.program, x->arg->type == STRING ? S2F : INTEGRAL(x->arg->type) ? I2F : X2F, 0, FLOATING, x->arg, x->arg->op == ID ? x->arg : (Exnode_t*)0);
+					x->arg = exnewnode(expr.program,
+					                   x->arg->type == STRING ? S2F : INTEGRAL(x->arg->type) ? I2F : X2F,
+					                   false, FLOATING, x->arg,
+					                   x->arg->op == ID ? x->arg : NULL);
 				break;
 			case INTEGER:
 			case UNSIGNED:
 				if (!INTEGRAL(x->arg->type))
-					x->arg = exnewnode(expr.program, x->arg->type == STRING ? S2I : x->arg->type == FLOATING ? F2I : X2I, 0, INTEGER, x->arg, x->arg->op == ID ? x->arg : (Exnode_t*)0);
+					x->arg = exnewnode(expr.program,
+					                   x->arg->type == STRING ? S2I : x->arg->type == FLOATING ? F2I : X2I,
+					                   false, INTEGER, x->arg,
+					                   x->arg->op == ID ? x->arg : NULL);
 				x->arg->type = t;
 				break;
 			case STRING:
@@ -793,7 +747,10 @@ preprint(Exnode_t* args)
 					else if (!expr.program->disc->convertf || (x->arg->op != ID && x->arg->op != DYNAMIC && x->arg->op != F2X && x->arg->op != I2X && x->arg->op != S2X))
 						exerror("string format argument expected");
 					else
-						x->arg = exnewnode(expr.program, x->arg->type == FLOATING ? F2S : INTEGRAL(x->arg->type) ? I2S : X2S, 0, STRING, x->arg, x->arg->op == ID ? x->arg : (Exnode_t*)0);
+						x->arg = exnewnode(expr.program,
+						                   x->arg->type == FLOATING ? F2S : INTEGRAL(x->arg->type) ? I2S : X2S,
+						                   false, STRING, x->arg,
+						                   x->arg->op == ID ? x->arg : NULL);
 				}
 				break;
 			}
@@ -820,8 +777,6 @@ preprint(Exnode_t* args)
 
 int expush(Expr_t *p, const char *name, int line, FILE *fp) {
 	Exinput_t*	in;
-	char*		s;
-	char			buf[PATH_MAX];
 
 	if (!(in = calloc(1, sizeof(Exinput_t))))
 	{
@@ -830,32 +785,25 @@ int expush(Expr_t *p, const char *name, int line, FILE *fp) {
 	}
 	if (!p->input)
 		p->input = &expr.null;
-	in->bp = in->sp = NULL;
 	if ((in->fp = fp))
 		in->close = 0;
 	else if (name)
 	{
-		if (!(s = pathfind(name, p->disc->lib, p->disc->type, buf, sizeof(buf))) || !(in->fp = fopen(s, "r")))
+		if (!(in->fp = fopen(name, "r")))
 		{
 			exerror("%s: file not found", name);
-			in->bp = in->sp = "";
 		}
 		else
 		{
-			name = vmstrdup(p->vm, s);
+			name = vmstrdup(p->vm, name);
 			in->close = 1;
 		}
 	}
 	if (!(in->next = p->input)->next)
 	{
 		p->errors = 0;
-		if (!(p->disc->flags & EX_INTERACTIVE))
-		{
-			if (line >= 0)
-				error_info.line = line;
-		}
-		else if (!error_info.line)
-			error_info.line = 1;
+		if (line >= 0)
+			error_info.line = line;
 	}
 	else if (line >= 0)
 		error_info.line = line;
@@ -899,8 +847,7 @@ expop(Expr_t* p)
 					error_info.line++;
 					break;
 				}
-		if (!(p->disc->flags & EX_INTERACTIVE))
-			error_info.line = in->line;
+		error_info.line = in->line;
 	}
 	if (in->fp && in->close)
 		fclose(in->fp);
@@ -917,12 +864,9 @@ expop(Expr_t* p)
  * clear global state of stale pointers
  */
 
-void exinit(void) {
-	memset (&expr, 0, sizeof(Exstate_t));
-}
+void exinit(void) { expr = (Exstate_t){0}; }
 
 int excomp(Expr_t *p, const char *name, int line, FILE *fp, char *prefix) {
-	Exid_t*	v;
 	int	eof;
 
 	eof = p->eof;
@@ -935,17 +879,6 @@ int excomp(Expr_t *p, const char *name, int line, FILE *fp, char *prefix) {
 	p->input->unit = 0;
 	expop(p);
 	p->eof = eof;
-	if (expr.statics)
-	{
-		for (v = dtfirst(p->symbols); v; v = dtnext(p->symbols, v))
-			if (v->isstatic)
-			{
-				dtdelete(p->symbols, v);
-				if (!--expr.statics)
-					break;
-			}
-		expr.statics = 0;
-	}
 	return 0;
 }
 
@@ -953,41 +886,31 @@ int excomp(Expr_t *p, const char *name, int line, FILE *fp, char *prefix) {
  * free the program p
  */
 
-void
-exclose(Expr_t* p, int all)
-{
+void exclose(Expr_t *p) {
 	Exinput_t*	in;
 
 	if (p)
 	{
-		if (all)
+		size_t i;
+		for (i = 3; i < elementsof(p->file); i++)
+			if (p->file[i])
+				fclose(p->file[i]);
+		if (p->symbols)
+			dtclose(p->symbols);
+		if (p->vm)
+			vmclose(p->vm);
+		if (p->ve)
+			vmclose(p->ve);
+		agxbfree(&p->tmp);
+		while ((in = p->input))
 		{
-			size_t i;
-			for (i = 3; i < elementsof(p->file); i++)
-				if (p->file[i])
-					fclose(p->file[i]);
-			if (p->vm)
-				vmclose(p->vm);
-			if (p->ve)
-				vmclose(p->ve);
-			if (p->symbols)
-				dtclose(p->symbols);
-			agxbfree(&p->tmp);
-			while ((in = p->input))
-			{
-				free(in->pushback);
-				if (in->fp && in->close)
-					fclose(in->fp);
-				if ((p->input = in->next))
-					free(in);
-			}
-			free(p);
-		}
-		else
-		{
-			vmclear(p->ve);
-			p->main.value = 0;
+			free(in->pushback);
+			if (in->fp && in->close)
+				fclose(in->fp);
+			if ((p->input = in->next))
+				free(in);
 		}
+		free(p);
 	}
 }
 
@@ -1017,8 +940,7 @@ checkBinary(Expr_t * p, Exnode_t * l, Exnode_t * ex, Exnode_t * r)
  * order to check that the name is undeclared and give a better
  * error message if it isn't.
  */
-static void checkName(Exid_t * id) 
-{
+static void checkName(const Exid_t *id) {
 	switch (id->lex) {
 	case DYNAMIC:
 	    exerror("Variable \"%s\" already declared", id->name);
@@ -1038,12 +960,9 @@ static void checkName(Exid_t * id)
 	}
 }
 
-static int
-cmpKey(Dt_t * d, Extype_t * key1, Extype_t * key2, Dtdisc_t * disc) 
-{
-	(void)d;
-	(void)disc;
-
+static int cmpKey(void *k1, void *k2) {
+	const Extype_t *key1 = k1;
+	const Extype_t *key2 = k2;
 	if (key1->integer < key2->integer)
 	    return -1;
 	else if (key1->integer > key2->integer)
