@@ -50,7 +50,7 @@ extern "C" {
 
 #define EX_NAMELEN	32		/* default Exid_t.name length	*/
 
-#define EX_ID(n, l, i, t) {{0}, (l), (i), (t), 0, 0, 0, 0, n}
+#define EX_ID(n, l, i, t) {{0}, (l), (i), (t), 0, 0, 0, n}
 
 #define DELETE_T		MINTOKEN		/* exexpr() delete `type'	*/
 
@@ -96,7 +96,6 @@ typedef struct Exid_s			/* id symbol table info		*/
 	long		index;		/* user defined index		*/
 	long		type;		/* symbol and arg types		*/
 	long		index_type;	/* index type for arrays        */
-	long		flags;		/* user defined flags		*/
 	Exnode_t*	value;		/* value			*/
 	void *local; ///< user defined local stuff
 	char		name[EX_NAMELEN];/* symbol name			*/
@@ -122,7 +121,6 @@ union Exdata_u
 	{
 	Exnode_t*	left;		/* left operand			*/
 	Exnode_t*	right;		/* right operand		*/
-	Exnode_t*	last;		/* for cons			*/
 	}		operand;	/* operands			*/
 
 	struct
@@ -190,6 +188,28 @@ struct Exdisc_s				/* discipline			*/
 					/* reference value function	*/
 	int		(*setf)(Expr_t*, Exnode_t*, Exid_t*, Exref_t*, void*, Extype_t);
 					/* set value function		*/
+	/// length function
+	///
+	/// This callback allows a user-defined '#' handler. That is, extending the
+	/// array length operator’s function.
+	///
+	/// @param rhs The right-hand side operand. That is 'foo' in the expression
+	///   '# foo'.
+	/// @param disc Pointer to the containing discipline.
+	/// @return The “length”, according to the user’s interpretation.
+	Extype_t	(*lengthf)(Exid_t *rhs, Exdisc_t *disc);
+	/// array membership function
+	///
+	/// This callback allows a user-defined `in` handler. That is, extending the
+	/// array membership test’s function.
+	///
+	/// @param lhs The left-hand side operand. That is 'foo' in the expression
+	///   'foo in bar'.
+	/// @param rhs The right-hand side operand. That is 'bar' in the expression
+	///   'foo in bar'.
+	/// @param disc Pointer to the containing discipline.
+	/// @return 0 if `lhs` is not in the array `rhs`, non-zero otherwise
+	int		(*inf)(Extype_t lhs, Exid_t *rhs, Exdisc_t *disc);
 	/* exit function           */
 	Exexit_f	exitf;
 	int*		types;
@@ -211,7 +231,7 @@ struct Expr_s				/* ex program state		*/
 
 extern Exnode_t *excast(Expr_t *, Exnode_t *, long, Exnode_t *, int);
 extern Exnode_t*	exnoncast(Exnode_t *);
-extern void		exclose(Expr_t*, int);
+extern void exclose(Expr_t *);
 
 /** Compile an expression
  *
@@ -229,7 +249,7 @@ extern int excomp(Expr_t *p, const char *name, int line, FILE *fp,
                   char *prefix);
 
 extern char*		excontext(Expr_t*, char*, int);
-extern int		exdump(Expr_t*, Exnode_t*, agxbuf*);
+extern void exdump(Expr_t *, Exnode_t *, agxbuf *);
 #ifdef __GNUC__
 __attribute__((format(printf, 1, 2)))
 #endif
