@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -30,11 +30,12 @@
 
 #include "config.h"
 #include <assert.h>
+#include <float.h>
 #include <limits.h>
 #include <inttypes.h>
 #include <assert.h>
-#include <cgraph/alloc.h>
-#include <cgraph/list.h>
+#include <common/render.h>
+#include <common/utils.h>
 #include <fdpgen/tlayout.h>
 #include <math.h>
 #include <neatogen/neatoprocs.h>
@@ -45,6 +46,8 @@
 #include <fdpgen/dbg.h>
 #include <stddef.h>
 #include <stdbool.h>
+#include <util/alloc.h>
+#include <util/list.h>
 
 typedef struct {
     graph_t*  rootg;  /* logical root; graph passed in to fdp_layout */
@@ -72,26 +75,24 @@ typedef struct {
  * node's connected component.
  * Also, entire layout is translated to origin.
  */
-static void
-finalCC(graph_t * g, int c_cnt, graph_t ** cc, point * pts, graph_t * rg,
-	layout_info* infop)
-{
+static void finalCC(graph_t *g, size_t c_cnt, graph_t **cc, pointf *pts,
+                    graph_t *rg, layout_info* infop) {
     attrsym_t * G_width = infop->G_width;
     attrsym_t * G_height = infop->G_height;
     graph_t *cg;
-    box b, bb;
+    boxf bb;
     boxf bbf;
-    point pt;
+    pointf pt;
     int margin;
     graph_t **cp = cc;
-    point *pp = pts;
+    pointf *pp = pts;
     int isRoot = rg == infop->rootg;
     int isEmpty = 0;
 
     /* compute graph bounding box in points */
     if (c_cnt) {
 	cg = *cp++;
-	BF2B(GD_bb(cg), bb);
+	bb = GD_bb(cg);
 	if (c_cnt > 1) {
 	    pt = *pp++;
 	    bb.LL.x += pt.x;
@@ -99,16 +100,16 @@ finalCC(graph_t * g, int c_cnt, graph_t ** cc, point * pts, graph_t * rg,
 	    bb.UR.x += pt.x;
 	    bb.UR.y += pt.y;
 	    while ((cg = *cp++)) {
-		BF2B(GD_bb(cg), b);
+		boxf b = GD_bb(cg);
 		pt = *pp++;
 		b.LL.x += pt.x;
 		b.LL.y += pt.y;
 		b.UR.x += pt.x;
 		b.UR.y += pt.y;
-		bb.LL.x = MIN(bb.LL.x, b.LL.x);
-		bb.LL.y = MIN(bb.LL.y, b.LL.y);
-		bb.UR.x = MAX(bb.UR.x, b.UR.x);
-		bb.UR.y = MAX(bb.UR.y, b.UR.y);
+		bb.LL.x = fmin(bb.LL.x, b.LL.x);
+		bb.LL.y = fmin(bb.LL.y, b.LL.y);
+		bb.UR.x = fmax(bb.UR.x, b.UR.x);
+		bb.UR.y = fmax(bb.UR.y, b.UR.y);
 	    }
 	}
     } else {			/* empty graph */
@@ -120,12 +121,8 @@ finalCC(graph_t * g, int c_cnt, graph_t ** cc, point * pts, graph_t * rg,
     }
 
     if (GD_label(rg)) {
-	point p;
-	int d;
-
 	isEmpty = 0;
-	PF2P(GD_label(rg)->dimen, p);
-	d = p.x - (bb.UR.x - bb.LL.x);
+	double d = round(GD_label(rg)->dimen.x) - (bb.UR.x - bb.LL.x);
 	if (d > 0) {		/* height of label added below */
 	    d /= 2;
 	    bb.LL.x -= d;
@@ -149,7 +146,7 @@ finalCC(graph_t * g, int c_cnt, graph_t ** cc, point * pts, graph_t * rg,
 	cp = cc;
 	pp = pts;
 	while ((cg = *cp++)) {
-	    point p;
+	    pointf p;
 	    node_t *n;
 	    pointf del;
 
@@ -340,7 +337,7 @@ static void chkPos(graph_t* g, node_t* n, layout_info* infop, boxf* bbp)
 		ND_pinned(n) = P_SET;
 	    *bbp = bb;
 	} else
-	    agerr(AGWARN, "graph %s, coord %s, expected four doubles\n",
+	    agwarningf("graph %s, coord %s, expected four doubles\n",
 		  agnameof(g), p);
     }
 }
@@ -354,7 +351,7 @@ static void addEdge(edge_t * de, edge_t * e)
     short cnt = ED_count(de);
     edge_t **el;
 
-    el = (edge_t **) (ED_to_virt(de));
+    el = (edge_t**)ED_to_virt(de);
     el = gv_recalloc(el, cnt, cnt + 1, sizeof(edge_t*));
     el[cnt] = e;
     ED_to_virt(de) = (edge_t *) el;
@@ -370,13 +367,16 @@ copyAttr (graph_t* g, graph_t* dg, char* attr)
     char*     ov_val;
     Agsym_t*  ov;
 
-    if ((ov = agattr(g,AGRAPH, attr, NULL))) {
+    if ((ov = agattr_text(g,AGRAPH, attr, NULL))) {
 	ov_val = agxget(g,ov);
-	ov = agattr(dg,AGRAPH, attr, NULL);
+	ov = agattr_text(dg,AGRAPH, attr, NULL);
 	if (ov)
 	    agxset (dg, ov, ov_val);
-	else
-	    agattr(dg, AGRAPH, attr, ov_val);
+	else {
+	    const bool is_html = aghtmlstr(ov_val);
+	    is_html ? agattr_html(dg, AGRAPH, attr, ov_val)
+	            : agattr_text(dg, AGRAPH, attr, ov_val);
+	}
     }
 }
 
@@ -403,7 +403,7 @@ static graph_t *deriveGraph(graph_t * g, layout_info * infop)
 
     dg = agopen("derived", Agstrictdirected,NULL);
     agbindrec(dg, "Agraphinfo_t", sizeof(Agraphinfo_t), true);
-    GD_alg(dg) = NEW(gdata);	/* freed in freeDeriveGraph */
+    GD_alg(dg) = gv_alloc(sizeof(gdata)); // freed in freeDeriveGraph
 #ifdef DEBUG
     GORIG(dg) = g;
 #endif
@@ -417,7 +417,7 @@ static graph_t *deriveGraph(graph_t * g, layout_info * infop)
 
     /* create derived nodes from clusters */
     for (i = 1; i <= GD_n_cluster(g); i++) {
-	boxf fix_bb = {{ MAXDOUBLE, MAXDOUBLE },{ -MAXDOUBLE, -MAXDOUBLE }};
+	boxf fix_bb = {{DBL_MAX, DBL_MAX}, {-DBL_MAX, -DBL_MAX}};
 	subg = GD_clust(g)[i];
 
 	do_graph_label(subg);
@@ -439,7 +439,7 @@ static graph_t *deriveGraph(graph_t * g, layout_info * infop)
     for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
 	if (!DNODE(n)) {
 	    if (PARENT(n) && PARENT(n) != GPARENT(g)) {
-		agerr (AGERR, "node \"%s\" is contained in two non-comparable clusters \"%s\" and \"%s\"\n", agnameof(n), agnameof(g), agnameof(PARENT(n)));
+		agerrorf("node \"%s\" is contained in two non-comparable clusters \"%s\" and \"%s\"\n", agnameof(n), agnameof(g), agnameof(PARENT(n)));
 		return NULL;
 	    }
 	    PARENT(n) = g;
@@ -754,6 +754,10 @@ setClustNodes(graph_t* root)
 	ND_pos(n)[1] = ctr.y;
 	ND_width(n) = w;
 	ND_height(n) = h;
+	const double penwidth = late_double(n, N_penwidth, DEFAULT_NODEPENWIDTH,
+	                                    MIN_NODEPENWIDTH);
+	ND_outline_width(n) = w + penwidth;
+	ND_outline_height(n) = h + penwidth;
 	/* ND_xsize(n) = POINTS(w); */
 	ND_lw(n) = ND_rw(n) = w2;
 	ND_ht(n) = h_pts;
@@ -767,6 +771,16 @@ setClustNodes(graph_t* root)
 	vertices[2].y = -h2;
 	vertices[3].x = ND_rw(n);
 	vertices[3].y = -h2;
+	// allocate extra vertices representing the outline, i.e., the outermost
+	// periphery with penwidth taken into account
+	vertices[4].x = ND_rw(n) + penwidth / 2;
+	vertices[4].y = h2 + penwidth / 2;
+	vertices[5].x = -ND_lw(n) - penwidth / 2;
+	vertices[5].y = h2 + penwidth / 2;
+	vertices[6].x = -ND_lw(n) - penwidth / 2;
+	vertices[6].y = -h2 - penwidth / 2;
+	vertices[7].x = ND_rw(n) + penwidth / 2;
+	vertices[7].y = -h2 - penwidth / 2;
     }
 }
 
@@ -799,7 +813,7 @@ setClustNodes(graph_t* root)
  */
 static int layout(graph_t * g, layout_info * infop)
 {
-    point *pts = NULL;
+    pointf *pts = NULL;
     graph_t *dg;
     node_t *dn;
     node_t *n;
@@ -807,7 +821,6 @@ static int layout(graph_t * g, layout_info * infop)
     graph_t *sg;
     graph_t **cc;
     graph_t **pg;
-    int c_cnt;
     int pinned;
     xparams xpms;
 
@@ -828,6 +841,7 @@ static int layout(graph_t * g, layout_info * infop)
     if (dg == NULL) {
 	return -1;
     }
+    size_t c_cnt;
     cc = pg = findCComp(dg, &c_cnt, &pinned);
 
     while ((cg = *pg++)) {
@@ -907,7 +921,7 @@ static int layout(graph_t * g, layout_info * infop)
     BB(g) = BB(dg);
 #ifdef DEBUG
     if (g == infop->rootg)
-	dump(g, 1, 0);
+	dump(g, 1);
 #endif
 
     /* clean up temp graphs */
@@ -950,9 +964,9 @@ static void setBB(graph_t * g)
  */
 static void init_info(graph_t * g, layout_info * infop)
 {
-    infop->G_coord = agattr(g, AGRAPH, "coords", NULL);
-    infop->G_width = agattr(g, AGRAPH, "width", NULL);
-    infop->G_height = agattr(g, AGRAPH, "height", NULL);
+    infop->G_coord = agattr_text(g, AGRAPH, "coords", NULL);
+    infop->G_width = agattr_text(g, AGRAPH, "width", NULL);
+    infop->G_height = agattr_text(g, AGRAPH, "height", NULL);
     infop->rootg = g;
     infop->gid = 0;
     infop->pack.mode = getPackInfo(g, l_node, CL_OFFSET / 2, &infop->pack);
@@ -982,9 +996,9 @@ mkClusters (graph_t * g, clist_t* pclist, graph_t* parent)
 
     for (subg = agfstsubg(g); subg; subg = agnxtsubg(subg))
 	{
-	if (!strncmp(agnameof(subg), "cluster", 7)) {
+	if (is_a_cluster(subg)) {
 	    agbindrec(subg, "Agraphinfo_t", sizeof(Agraphinfo_t), true);
-	    GD_alg(subg) = NEW(gdata);	/* freed in cleanup_subgs */
+	    GD_alg(subg) = gv_alloc(sizeof(gdata)); // freed in cleanup_subgs
 	    GD_ndim(subg) = GD_ndim(agroot(parent));
 	    LEVEL(subg) = LEVEL(parent) + 1;
 	    GPARENT(subg) = parent;
@@ -1010,8 +1024,8 @@ mkClusters (graph_t * g, clist_t* pclist, graph_t* parent)
 static void fdp_init_graph(Agraph_t * g)
 {
     setEdgeType (g, EDGETYPE_LINE);
-    GD_alg(g) = NEW(gdata);	/* freed in cleanup_graph */
-    GD_ndim(agroot(g)) = late_int(g, agattr(g,AGRAPH, "dim", NULL), 2, 2);
+    GD_alg(g) = gv_alloc(sizeof(gdata)); // freed in cleanup_graph
+    GD_ndim(agroot(g)) = late_int(g, agattr_text(g,AGRAPH, "dim", NULL), 2, 2);
     Ndim = GD_ndim(agroot(g)) = MIN(GD_ndim(agroot(g)), MAXDIM);
 
     mkClusters (g, NULL, g);
@@ -1055,7 +1069,7 @@ fdpSplines (graph_t * g)
 	}
 	if (trySplines || et != EDGETYPE_COMPOUND) {
 	    if (HAS_CLUST_EDGE(g)) {
-		agerr(AGWARN,
+		agwarningf(
 		      "splines and cluster edges not supported - using line segments\n");
 		et = EDGETYPE_LINE;
 	    } else {
