@@ -3,19 +3,16 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
 #pragma once
 
+#include <stdbool.h>
 #include <stdlib.h>
 #include <string.h>
-#ifdef _WIN32
-#include <glcomp/glpangofont.h>
-typedef float GLfloat;
-#endif
 #include <glcomp/opengl.h>
 #include <glcomp/glcompdefs.h>
 #include <GL/glut.h>
@@ -24,15 +21,16 @@ typedef float GLfloat;
 extern "C" {
 #endif
 
-    void glprintfglut(void *font, GLfloat xpos, GLfloat ypos, GLfloat zpos, char *bf);
+    void glprintfglut(void *font, float xpos, float ypos, float zpos, char *bf);
 
-    glCompFont *glNewFont(glCompSet * s, char *text, glCompColor * c,
-          glCompFontType type, char *fontdesc, int fs,int is2D);
-    glCompFont *glNewFontFromParent(glCompObj * o, char *text);
+glCompFont glNewFont(glCompSet *s, const char *text, glCompColor *c,
+                     char *fontdesc, int fs, bool is2D);
+glCompFont glNewFontFromParent(glCompObj *o, const char *text);
     void glDeleteFont(glCompFont * f);
-    void glCompDrawText(glCompFont * f,GLfloat x,GLfloat y);
-    void glCompRenderText(glCompFont * f, glCompObj * parentObj);
-    void glCompDrawText3D(glCompFont * f,GLfloat x,GLfloat y,GLfloat z,GLfloat w,GLfloat h);
+void glCompDrawText(glCompFont f, float x, float y);
+void glCompRenderText(glCompFont f, glCompObj *parentObj);
+void glCompDrawText3D(glCompFont f, float x, float y, double z, float w,
+                      float h);
 
 #ifdef __cplusplus
 }
