@@ -3,120 +3,107 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
 #include <glcomp/glutils.h>
+#include <stdbool.h>
+#include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
 #include <glcomp/glcompdefs.h>
 
 /*transforms 2d windows location to 3d gl coords but depth is calculated unlike the previous function*/
-int GetOGLPosRef(int x, int y, float *X, float *Y, float *Z)
-{
-    GLdouble wwinX;
-    GLdouble wwinY;
-    GLdouble wwinZ;
-    GLdouble posX, posY, posZ;
+void GetOGLPosRef(int x, int y, float *X, float *Y) {
+    double wwinZ;
+    double posX, posY;
 
-    GLint viewport[4];
-    GLdouble modelview[16];
-    GLdouble projection[16];
-    GLfloat winX, winY;
+    int32_t viewport[4];
+    double modelview[16];
+    double projection[16];
     glGetDoublev(GL_MODELVIEW_MATRIX, modelview);
     glGetDoublev(GL_PROJECTION_MATRIX, projection);
     glGetIntegerv(GL_VIEWPORT, viewport);
 
-    //draw a point  to a not important location to get window coordinates
+    // draw a point to an unimportant location to get window coordinates
     glColor4f(0.0f, 0.0f, 0.0f, 0.001f);
 
     glBegin(GL_POINTS);
     glVertex3f(-100.0f, -100.0f, 0.0f);
     glEnd();
     gluProject(-100.0, -100.0, 0.00, modelview, projection, viewport,
-	       &wwinX, &wwinY, &wwinZ);
-    winX = (float) x;
-    winY = (float) viewport[3] - (float) y;
+               &(double){0}, &(double){0}, &wwinZ);
+    const double winX = x;
+    const double winY = (double)viewport[3] - y;
     gluUnProject(winX, winY, wwinZ, modelview, projection, viewport, &posX,
-		 &posY, &posZ);
+                 &posY, &(double){0}); // ignore Z that the caller does not need
 
     *X = (float) posX;
     *Y = (float) posY;
-    *Z = (float) posZ;
-
-    return 1;
 }
 
-float GetOGLDistance(int l)
-{
-    int x, y;
-    GLdouble wwinX;
-    GLdouble wwinY;
-    GLdouble wwinZ;
-    GLdouble posX, posY, posZ;
-    GLdouble posXX, posYY, posZZ;
+double GetOGLDistance(double l) {
+    double wwinZ;
+    double posX;
+    double posXX;
 
-    GLint viewport[4];
-    GLdouble modelview[16];
-    GLdouble projection[16];
-    GLfloat winX, winY;
+    int32_t viewport[4];
+    double modelview[16];
+    double projection[16];
 
     glGetDoublev(GL_MODELVIEW_MATRIX, modelview);
     glGetDoublev(GL_PROJECTION_MATRIX, projection);
     glGetIntegerv(GL_VIEWPORT, viewport);
 
-    //draw a point  to a not important location to get window coordinates
+    // draw a point to an unimportant location to get window coordinates
     glColor4f(0.0f, 0.0f, 0.0f, 0.001f);
 
     glBegin(GL_POINTS);
     glVertex3f(10.0f, 10.0f, 1.0f);
     glEnd();
-    gluProject(10.0, 10.0, 1.00, modelview, projection, viewport, &wwinX,
-	       &wwinY, &wwinZ);
-    x = 50;
-    y = 50;
-    winX = (float) x;
-    winY = (float) viewport[3] - (float) y;
+    gluProject(10.0, 10.0, 1.0, modelview, projection, viewport, &(double){0},
+               &(double){0}, &wwinZ);
+    double x = 50.0;
+    const double y = 50.0;
+    double winX = x;
+    double winY = viewport[3] - y;
     gluUnProject(winX, winY, wwinZ, modelview, projection, viewport, &posX,
-		 &posY, &posZ);
-    x = x + l;
-    y = 50;
-    winX = (float) x;
-    winY = (float) viewport[3] - (float) y;
+                 &(double){0}, &(double){0});
+    x += l;
+    winX = x;
+    winY = viewport[3] - y;
     gluUnProject(winX, winY, wwinZ, modelview, projection, viewport,
-		 &posXX, &posYY, &posZZ);
-    return ((float) (posXX - posX));
+                 &posXX, &(double){0}, &(double){0});
+    return posXX - posX;
 }
 
 /*
-	functions def: returns opengl coordinates of firt hit object by using screen coordinates
-	x,y; 2D screen coordiantes (usually received from mouse events
+	functions def: returns opengl coordinates of first hit object by using screen coordinates
+	x,y; 2D screen coordinates (usually received from mouse events
 	X,Y,Z; pointers to coordinates values to be calculated
 	return value: no return value
 
 
 */
 
-void to3D(int x, int y, GLfloat * X, GLfloat * Y, GLfloat * Z)
-{
+void to3D(int x, int y, float *X, float *Y, float *Z) {
     int const WIDTH = 20;
 
-    GLint viewport[4];
-    GLdouble modelview[16];
-    GLdouble projection[16];
-    GLfloat winX, winY;
-    GLfloat winZ[400];
-    GLdouble posX, posY, posZ;
+    int32_t viewport[4];
+    double modelview[16];
+    double projection[16];
+    float winZ[400];
+    double posX, posY, posZ;
     int idx;
 
     glGetDoublev(GL_MODELVIEW_MATRIX, modelview);
     glGetDoublev(GL_PROJECTION_MATRIX, projection);
     glGetIntegerv(GL_VIEWPORT, viewport);
 
-    winX = (float) x;
-    winY = (float) viewport[3] - (float) y;
+    const double winX = x;
+    const double winY = (double)viewport[3] - y;
 
     glReadPixels(x - WIDTH / 2, (int) winY - WIDTH / 2, WIDTH, WIDTH,
 		 GL_DEPTH_COMPONENT, GL_FLOAT, &winZ);
@@ -129,20 +116,16 @@ void to3D(int x, int y, GLfloat * X, GLfloat * Y, GLfloat * Z)
     gluUnProject(winX, winY, comp, modelview, projection, viewport, &posX,
 		 &posY, &posZ);
 
-    *X = (GLfloat) posX;
-    *Y = (GLfloat) posY;
-    *Z = (GLfloat) posZ;
-    return;
+    *X = (float)posX;
+    *Y = (float)posY;
+    *Z = (float)posZ;
 }
 
 #include <math.h>
 
 static glCompPoint sub(glCompPoint p, glCompPoint q)
 {
-    p.x -= q.x;
-    p.y -= q.y;
-    p.z -= q.z;
-    return p;
+    return (glCompPoint){.x = p.x - q.x, .y = p.y - q.y, .z = p.z - q.z};
 }
 
 static double dot(glCompPoint p, glCompPoint q)
@@ -195,18 +178,11 @@ double point_to_lineseg_dist(glCompPoint p, glCompPoint a, glCompPoint b)
     return dist(p, q);
 }
 
-void replacestr(char *source, char **target)
-{
-
-    free(*target);
-    *target = strdup(source);
-}
-
 void glCompCalcWidget(glCompCommon * parent, glCompCommon * child,
 		      glCompCommon * ref)
 {
     /*check alignments first , alignments overrides anchors */
-    GLfloat borderWidth;
+    float borderWidth;
     ref->height = child->height;
     ref->width = child->width;
     if(!parent)
@@ -215,10 +191,7 @@ void glCompCalcWidget(glCompCommon * parent, glCompCommon * child,
 	child->refPos.y = child->pos.y;
 	return;
     }
-    if (parent->borderType == glBorderNone)
-	borderWidth = 0.0f;
-    else
-	borderWidth = parent->borderWidth;
+    borderWidth = parent->borderWidth;
     if (child->align != glAlignNone)	//if alignment, make sure width and height is no greater than parent
     {
 	if (child->width > parent->width)
@@ -268,7 +241,7 @@ void glCompCalcWidget(glCompCommon * parent, glCompCommon * child,
     case glAlignNone:
 	break;
     }
-    if (child->align == glAlignNone)	// No alignment , chekc anchors
+    if (child->align == glAlignNone)	// No alignment, check anchors
     {
 	ref->pos.x = parent->refPos.x + child->pos.x + borderWidth;
 	ref->pos.y = parent->refPos.y + child->pos.y + borderWidth;
@@ -304,10 +277,7 @@ static void glCompQuadVertex(glCompPoint * p0, glCompPoint * p1,
     glVertex3f(p3->x, p3->y, p3->z);
 }
 
-void glCompSetColor(glCompColor * c)
-{
-    glColor4f(c->R, c->G, c->B, c->A);
-}
+void glCompSetColor(glCompColor c) { glColor4d(c.R, c.G, c.B, c.A); }
 
 void glCompDrawRectangle(glCompRect * r)
 {
@@ -318,12 +288,12 @@ void glCompDrawRectangle(glCompRect * r)
     glVertex3f(r->pos.x, r->pos.y + r->h, r->pos.z);
     glEnd();
 }
-void glCompDrawRectPrism(glCompPoint * p, GLfloat w, GLfloat h, GLfloat b,
-			 GLfloat d, glCompColor * c, int bumped)
-{
-    GLfloat color_fac;
+
+void glCompDrawRectPrism(glCompPoint *p, float w, float h, float b, float d,
+                         glCompColor *c, bool bumped) {
+    float color_fac;
     glCompPoint A, B, C, D, E, F, G, H;
-    GLfloat dim = 1.00;
+    float dim = 1;
     if (!bumped) {
 	color_fac = 1.3f;
 	b = b - 2;
@@ -357,15 +327,15 @@ void glCompDrawRectPrism(glCompPoint * p, GLfloat w, GLfloat h, GLfloat b,
     F.y = p->y + h - b;
     F.z = p->z + d;
     glBegin(GL_QUADS);
-    glColor4f(c->R * dim, c->G * dim, c->B * dim, c->A);
+    glColor4d(c->R * dim, c->G * dim, c->B * dim, c->A);
     glCompQuadVertex(&G, &H, &F, &E);
 
-    glColor4f(c->R * color_fac * dim, c->G * color_fac * dim,
+    glColor4d(c->R * color_fac * dim, c->G * color_fac * dim,
 	      c->B * color_fac * dim, c->A);
     glCompQuadVertex(&A, &B, &H, &G);
     glCompQuadVertex(&B, &H, &F, &C);
 
-    glColor4f(c->R / color_fac * dim, c->G / color_fac * dim,
+    glColor4d(c->R / color_fac * dim, c->G / color_fac * dim,
 	      c->B / color_fac * dim, c->A);
     glCompQuadVertex(&A, &G, &E, &D);
     glCompQuadVertex(&E, &F, &C, &D);
@@ -373,11 +343,9 @@ void glCompDrawRectPrism(glCompPoint * p, GLfloat w, GLfloat h, GLfloat b,
 
 }
 
-GLfloat distBetweenPts(glCompPoint A,glCompPoint B,float R)
-{
-    GLfloat rv=0;	
-    rv=(A.x-B.x)*(A.x-B.x) + (A.y-B.y)*(A.y-B.y) +(A.z-B.z)*(A.z-B.z);
-    rv=sqrtf(rv);
+double distBetweenPts(glCompPoint A, glCompPoint B, double R) {
+    double rv = (A.x - B.x) * (A.x - B.x) + (A.y - B.y) * (A.y - B.y) + (A.z - B.z) * (A.z - B.z);
+    rv = sqrt(rv);
     if (rv <=R)
 	return 0;
     return rv;
@@ -390,3 +358,8 @@ int is_point_in_rectangle(float X, float Y, float RX, float RY, float RW,float R
     else
 	return 0;
 }
+
+/**
+ * @dir lib/glcomp
+ * @brief OpenGL GUI for cmd/smyrna
+ */
