@@ -15,19 +15,20 @@
 #endif
 
 #include <assert.h>
-#include <cgraph/agxbuf.h>
-#include <cgraph/gv_ctype.h>
-#include <cgraph/list.h>
-#include <cgraph/startswith.h>
 #include <gvc/gvconfig.h>
 #include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include	<string.h>
 #include <unistd.h>
+#include <util/agxbuf.h>
 #include <util/alloc.h>
 #include <util/exit.h>
+#include <util/gv_ctype.h>
 #include <util/gv_fopen.h>
+#include <util/list.h>
+#include <util/path.h>
+#include <util/startswith.h>
 
 #ifdef ENABLE_LTDL
 #ifdef HAVE_DL_ITERATE_PHDR
@@ -507,7 +508,7 @@ static void config_rescan(GVC_t *gvc, char *config_path)
     libdir = gvconfig_libdir(gvc);
 
     agxbuf config_glob = {0};
-    agxbprint(&config_glob, "%s%s%s", libdir, DIRSEP, plugin_glob);
+    agxbprint(&config_glob, "%s%c%s", libdir, PATH_SEPARATOR, plugin_glob);
 
     /* load all libraries even if can't save config */
 
@@ -530,7 +531,7 @@ static void config_rescan(GVC_t *gvc, char *config_path)
 	    if (is_plugin(globbuf.gl_pathv[i])) {
 		library = gvplugin_library_load(gvc, globbuf.gl_pathv[i]);
 		if (library) {
-		    char *p = strrchr(globbuf.gl_pathv[i], DIRSEP[0]);
+		    char *p = strrchr(globbuf.gl_pathv[i], PATH_SEPARATOR);
 		    if (p)
 			p++;
 		    if (f && p)
@@ -576,7 +577,7 @@ void gvconfig(GVC_t * gvc, bool rescan)
     
         if (! gvc->config_path) {
             agxbuf xb = {0};
-            agxbprint(&xb, "%s%s%s", libdir, DIRSEP, config_file_name);
+            agxbprint(&xb, "%s%c%s", libdir, PATH_SEPARATOR, config_file_name);
             gvc->config_path = agxbdisown(&xb);
         }
     	
@@ -624,6 +625,8 @@ void gvconfig(GVC_t * gvc, bool rescan)
         }
     }
 done:
+#else
+    (void)rescan;
 #endif
     gvtextlayout_select(gvc);   /* choose best available textlayout plugin immediately */
     textfont_dict_open(gvc);    /* initialize font dict */
@@ -644,6 +647,9 @@ DEFINE_LIST_WITH_DTOR(strs, char *, free)
 static int
 glob (GVC_t* gvc, char* pattern, int flags, int (*errfunc)(const char *, int), glob_t *pglob)
 {
+    (void)flags;
+    (void)errfunc;
+
     char* libdir;
     WIN32_FIND_DATA wfd;
     HANDLE h;
@@ -651,18 +657,28 @@ glob (GVC_t* gvc, char* pattern, int flags, int (*errfunc)(const char *, int), g
     
     pglob->gl_pathc = 0;
     pglob->gl_pathv = NULL;
+
+    // the Windows APIs use '\'-separated directory components, but MinGW uses
+    // '/' as the canonical directory separator, so convert these
+#ifdef __MINGW32__
+    for (size_t i = 0; pattern[i] != '\0'; ++i) {
+      if (pattern[i] == '/') {
+        pattern[i] = '\\';
+      }
+    }
+#endif
     
     h = FindFirstFile (pattern, &wfd);
     if (h == INVALID_HANDLE_VALUE) return GLOB_NOMATCH;
     libdir = gvconfig_libdir(gvc);
     do {
       const size_t size =
-        strlen(libdir) + strlen(DIRSEP) + strlen(wfd.cFileName) + 1;
+        strlen(libdir) + 1 /* path separator */ + strlen(wfd.cFileName) + 1;
       char *const entry = malloc(size);
       if (!entry) {
         goto oom;
       }
-      snprintf(entry, size, "%s%s%s", libdir, DIRSEP, wfd.cFileName);
+      snprintf(entry, size, "%s%c%s", libdir, PATH_SEPARATOR, wfd.cFileName);
       if (strs_try_append(&strs, entry) != 0) {
         free(entry);
         goto oom;
