@@ -19,6 +19,7 @@
 #include <gvpr/actions.h>
 #include <gvpr/compile.h>
 #include <inttypes.h>
+#include <limits.h>
 #include <math.h>
 #include <stdbool.h>
 #include <stddef.h>
@@ -29,7 +30,6 @@
 #include <unistd.h>
 #include <util/agxbuf.h>
 #include <util/alloc.h>
-#include <util/exit.h>
 #include <util/prisize_t.h>
 #include <util/startswith.h>
 #include <util/unreachable.h>
@@ -46,32 +46,11 @@ static int isedge(Agobj_t *obj) {
 #include <ctype.h>
 #include <gvpr/trie.c>
 
-#define BITS_PER_BYTE 8
-
 static void *int2ptr(long long i) { return (void *)(intptr_t)i; }
 
 static long long ptr2int(const void *p) { return (long long)(intptr_t)p; }
 
-static int iofread(void *chan, char *buf, int bufsize) {
-  FILE *fp = chan;
-
-  return (int)read(fileno(fp), buf, bufsize);
-}
-
-static int ioputstr(void *chan, const char *str) { return fputs(str, chan); }
-
-static int ioflush(void *chan) { return fflush(chan); }
-
-static Agiodisc_t gprIoDisc = {iofread, ioputstr, ioflush};
-
-#ifdef GVDLL
-static Agdisc_t gprDisc = {0, &gprIoDisc};
-#else
-static Agdisc_t gprDisc = {&AgIdDisc, &gprIoDisc};
-#endif
-
-/* nameOf:
- * Return name of object.
+/* Return name of object.
  * Assumes obj !=  NULL
  */
 static char *nameOf(Expr_t *ex, Agobj_t *obj, agxbuf *tmps) {
@@ -104,55 +83,48 @@ static char *nameOf(Expr_t *ex, Agobj_t *obj, agxbuf *tmps) {
   return s;
 }
 
-/* bbOf:
- * If string as form "x,y,u,v" where is all are numeric,
+/* If string as form "x,y,u,v" where is all are numeric,
  * return "x,y" or "u,v", depending on getll, else return ""
  */
 static char *bbOf(Expr_t *pgm, char *pt, bool getll) {
   double x, y, u, v;
-  char *s;
-  char *p;
 
   if (sscanf(pt, "%lf,%lf,%lf,%lf", &x, &y, &u, &v) == 4) {
-    p = strchr(pt, ',');
+    char *p = strchr(pt, ',');
     p = strchr(p + 1, ',');
     if (getll) {
       size_t len = (size_t)(p - pt);
-      s = exstralloc(pgm, len + 1);
+      char *const s = exstralloc(pgm, len + 1);
       strncpy(s, pt, len);
       s[len] = '\0';
-    } else
-      s = exstring(pgm, p + 1);
-  } else
-    s = "";
-  return s;
+      return s;
+    }
+    return exstring(pgm, p + 1);
+  }
+  return "";
 }
 
-/* xyOf:
- * If string as form "x,y" where is x and y are numeric,
+/* If string as form "x,y" where is x and y are numeric,
  * return "x" or "y", depending on getx, else return ""
  */
 static char *xyOf(Expr_t *pgm, char *pt, bool getx) {
   double x, y;
-  char *v;
-  char *p;
 
   if (sscanf(pt, "%lf,%lf", &x, &y) == 2) {
-    p = strchr(pt, ',');
+    char *const p = strchr(pt, ',');
     if (getx) {
       size_t len = (size_t)(p - pt);
-      v = exstralloc(pgm, len + 1);
+      char *const v = exstralloc(pgm, len + 1);
       strncpy(v, pt, len);
       v[len] = '\0';
-    } else
-      v = exstring(pgm, p + 1);
-  } else
-    v = "";
-  return v;
+      return v;
+    }
+    return exstring(pgm, p + 1);
+  }
+  return "";
 }
 
-/* posOf:
- * Get pos data from node; store x or y into v if successful and return  0;
+/* Get pos data from node; store x or y into v if successful and return  0;
  * else return -1
  */
 static int posOf(Agnode_t *np, int idx, double *v) {
@@ -164,7 +136,7 @@ static int posOf(Agnode_t *np, int idx, double *v) {
 
   if (root != nroot) {
     root = nroot;
-    pos = agattr(root, AGNODE, "pos", 0);
+    pos = agattr_text(root, AGNODE, "pos", 0);
   }
   if (!pos)
     return -1;
@@ -176,33 +148,7 @@ static int posOf(Agnode_t *np, int idx, double *v) {
     return -1;
 }
 
-#if defined(DEBUG) && DEBUG > 1
-static char *symName(Expr_t *ex, int op) {
-  if (op >= MINNAME && op <= MAXNAME)
-    return gprnames[op];
-  else {
-    // calculate how much space we need to construct a name
-    int bytes = vsnprintf(NULL, 0, "<unknown (%d)>", op);
-    if (bytes < 0) {
-      fprintf(stderr, "%s: vsnprintf failure\n", __func__);
-      graphviz_exit(EXIT_FAILURE);
-    }
-
-    // construct a managed buffer to store this name
-    char *s = vmalloc(ex->ve, (size_t)bytes + 1);
-
-    // make the name
-    if (s != NULL) {
-      snprintf(s, (size_t)bytes + 1, "<unknown (%d)>", op);
-    }
-
-    return s;
-  }
-}
-#endif
-
-/* xargs:
- * Convert string argument to graph to type of graph desired.
+/* Convert string argument to graph to type of graph desired.
  *   u => undirected
  *   d => directed
  *   s => strict
@@ -240,8 +186,7 @@ static Agdesc_t xargs(char *args) {
   return desc;
 }
 
-/* deparse:
- * Recreate string representation of expression involving
+/* Recreate string representation of expression involving
  * a reference and a symbol.
  */
 static char *deparse(Expr_t *ex, Exnode_t *n, agxbuf *xb) {
@@ -249,8 +194,7 @@ static char *deparse(Expr_t *ex, Exnode_t *n, agxbuf *xb) {
   return agxbuse(xb);
 }
 
-/* deref:
- * Evaluate reference to derive desired graph object.
+/* Evaluate reference to derive desired graph object.
  * A reference is either DI* or II*
  * The parameter objp is the current object.
  * Assume ref is type-correct.
@@ -319,8 +263,7 @@ static Agobj_t *deref(Expr_t *pgm, Exnode_t *x, Exref_t *ref, Agobj_t *objp,
   return 0;
 }
 
-/* assignable:
- * Check that attribute is not a read-only, pseudo-attribute.
+/* Check that attribute is not a read-only, pseudo-attribute.
  * fatal if not OK.
  */
 static void assignable(Agobj_t *objp, unsigned char *name) {
@@ -353,20 +296,17 @@ static void assignable(Agobj_t *objp, unsigned char *name) {
   }
 }
 
-/* setattr:
- * Set object's attribute name to val.
+/* Set object's attribute name to val.
  * Initialize attribute if necessary.
  */
 static int setattr(Agobj_t *objp, char *name, char *val) {
   Agsym_t *gsym = agattrsym(objp, name);
   if (!gsym) {
-    gsym = agattr(agroot(agraphof(objp)), AGTYPE(objp), name, "");
+    gsym = agattr_text(agroot(agraphof(objp)), AGTYPE(objp), name, "");
   }
   return agxset(objp, gsym, val);
 }
 
-/* kindToStr:
- */
 static char *kindToStr(int kind) {
   char *s;
 
@@ -384,13 +324,10 @@ static char *kindToStr(int kind) {
   return s;
 }
 
-/* kindOf:
- * Return string rep of object's kind
- */
+// return string rep of object’s kind
 static char *kindOf(Agobj_t *objp) { return kindToStr(agobjkind(objp)); }
 
-/* lookup:
- * Apply symbol to get field value of objp
+/* Apply symbol to get field value of objp
  * Assume objp != NULL
  */
 static int lookup(Expr_t *pgm, Agobj_t *objp, Exid_t *sym, Extype_t *v) {
@@ -511,7 +448,7 @@ static int lookup(Expr_t *pgm, Agobj_t *objp, Exid_t *sym, Extype_t *v) {
   } else {
     Agsym_t *gsym = agattrsym(objp, sym->name);
     if (!gsym) {
-      gsym = agattr(agroot(agraphof(objp)), AGTYPE(objp), sym->name, "");
+      gsym = agattr_text(agroot(agraphof(objp)), AGTYPE(objp), sym->name, "");
       agxbuf tmp = {0};
       error(ERROR_WARNING,
             "Using value of uninitialized %s attribute \"%s\" of \"%s\"",
@@ -524,19 +461,15 @@ static int lookup(Expr_t *pgm, Agobj_t *objp, Exid_t *sym, Extype_t *v) {
   return 0;
 }
 
-/* getArg:
- * Return value associated with $n.
- */
+// return value associated with $n
 static char *getArg(int n, Gpr_t *state) {
   if (n >= state->argc) {
     exerror("program references ARGV[%d] - undefined", n);
     return 0;
   }
-  return (state->argv[n]);
+  return state->argv[n];
 }
 
-/* setDfltAttr:
- */
 static int setDfltAttr(Agraph_t *gp, char *k, char *name, char *value) {
   int kind;
 
@@ -553,45 +486,44 @@ static int setDfltAttr(Agraph_t *gp, char *k, char *name, char *value) {
   default:
     error(ERROR_WARNING, "Unknown kind \"%s\" passed to setDflt()", k);
     return 1;
-    break;
   }
-  agattr(gp, kind, name, value);
+
+  // make the implicit default on the root graph explicit in order to avoid the
+  // next `agattr_text` thinking its assignment should be hoisted to the root
+  {
+    Agraph_t *const root = agroot(gp);
+    if (agattr_text(root, kind, name, NULL) == NULL) {
+      agattr_text(root, kind, name, "");
+    }
+  }
+
+  agattr_text(gp, kind, name, value);
   return 0;
 }
 
-/* toKind:
- * Map string to object kind
- */
+// map string to object kind
 static int toKind(char *k, char *fn) {
-  int kind;
-
   switch (*k) {
   case 'G':
-    kind = AGRAPH;
-    break;
+    return AGRAPH;
   case 'E':
-    kind = AGEDGE;
-    break;
+    return AGEDGE;
   case 'N':
-    kind = AGNODE;
-    break;
+    return AGNODE;
   default:
     exerror("Unknown kind \"%s\" passed to %s()", k, fn);
-    kind = 0;
     break;
   }
-  return kind;
+  return 0;
 }
 
-/* nxtAttr:
- */
 static char *nxtAttr(Agraph_t *gp, char *k, char *name) {
-  char *fn = (name ? "nxtAttr" : "fstAttr");
+  char *fn = name ? "nxtAttr" : "fstAttr";
   int kind = toKind(k, fn);
   Agsym_t *sym;
 
   if (name) {
-    sym = agattr(gp, kind, name, 0);
+    sym = agattr_text(gp, kind, name, 0);
     if (!sym) {
       exerror("Third argument \"%s\" in nxtAttr() must be the name of an "
               "existing attribute",
@@ -609,22 +541,18 @@ static char *nxtAttr(Agraph_t *gp, char *k, char *name) {
     return "";
 }
 
-/* getDfltAttr:
- */
 static char *getDfltAttr(Agraph_t *gp, char *k, char *name) {
   int kind = toKind(k, "getDflt");
-  Agsym_t *sym = agattr(gp, kind, name, 0);
+  Agsym_t *sym = agattr_text(gp, kind, name, 0);
   if (!sym) {
-    sym = agattr(gp, kind, name, "");
+    sym = agattr_text(gp, kind, name, "");
     error(ERROR_WARNING, "Uninitialized %s attribute \"%s\" in %s",
           kindToStr(kind), name, "getDflt");
   }
   return sym->defval;
 }
 
-/* getval:
- * Return value associated with gpr identifier.
- */
+// return value associated with gpr identifier
 static Extype_t getval(Expr_t *pgm, Exnode_t *node, Exid_t *sym, Exref_t *ref,
                        void *env, int elt, Exdisc_t *disc) {
   Extype_t v;
@@ -1136,6 +1064,14 @@ static Extype_t getval(Expr_t *pgm, Exnode_t *node, Exid_t *sym, Exref_t *ref,
       } else
         v.integer = copyAttr(objp, objp1);
       break;
+    case F_rename:
+      objp = int2ptr(args[0].integer);
+      if (!objp) {
+        error(ERROR_WARNING, "NULL object passed to rename()");
+        v.integer = -1;
+      } else
+        v.integer = agrelabel_node((Agnode_t *)objp, args[1].string);
+      break;
     case F_induce:
       gp = int2ptr(args[0].integer);
       if (!gp) {
@@ -1228,8 +1164,10 @@ static Extype_t getval(Expr_t *pgm, Exnode_t *node, Exid_t *sym, Exref_t *ref,
       if (!gp) {
         error(ERROR_WARNING, "NULL graph passed to lock()");
         v.integer = -1;
-      } else
-        v.integer = lockGraph(gp, args[1].integer);
+      } else {
+        const int op = args[1].integer > 0 ? 1 : args[1].integer < 0 ? -1 : 0;
+        v.integer = lockGraph(gp, op);
+      }
       break;
     case F_nnodes:
       gp = int2ptr(args[0].integer);
@@ -1301,7 +1239,7 @@ static Extype_t getval(Expr_t *pgm, Exnode_t *node, Exid_t *sym, Exref_t *ref,
           v.integer = (gsym != NULL);
         else {
           if (!gsym) {
-            gsym = agattr(agroot(agraphof(objp)), AGTYPE(objp), name, "");
+            gsym = agattr_text(agroot(agraphof(objp)), AGTYPE(objp), name, "");
             agxbuf tmp = {0};
             error(ERROR_WARNING,
                   "Using value of %s uninitialized attribute \"%s\" of \"%s\" "
@@ -1385,7 +1323,7 @@ static Extype_t getval(Expr_t *pgm, Exnode_t *node, Exid_t *sym, Exref_t *ref,
           exerror("NULL kind passed to %s", sym->name);
           v.string = 0;
         } else if (sym->index == F_isattr) {
-          v.integer = (agattr(gp, toKind(kind, sym->name), name, 0) != NULL);
+          v.integer = agattr_text(gp, toKind(kind, sym->name), name, 0) != NULL;
         } else if (sym->index == F_nxtattr) {
           v.string = nxtAttr(gp, kind, name);
         } else {
@@ -1561,8 +1499,7 @@ static Extype_t getval(Expr_t *pgm, Exnode_t *node, Exid_t *sym, Exref_t *ref,
 
 static char *typeName(long op) { return typenames[op - MINTYPE]; }
 
-/* setval:
- * Set sym to value v.
+/* Set sym to value v.
  * Return -1 if not allowed.
  * Assume already type correct.
  */
@@ -1640,6 +1577,51 @@ static int setval(Expr_t *pgm, Exnode_t *x, Exid_t *sym, Exref_t *ref,
   return setattr(objp, sym->name, v.string);
 }
 
+/// gvpr’s custom `#` implementation
+///
+/// The main purpose of this is to extend `#` to work on the command line
+/// options array, `ARGV`.
+///
+/// @param rhs The right-hand side operand to `#`.
+/// @param disc The libexpr discipline.
+/// @return The “length” as defined by us.
+static Extype_t length(Exid_t *rhs, Exdisc_t *disc) {
+  Extype_t v = {0};
+  switch (rhs->index) {
+  case A_ARGV: {
+    Gpr_t *const state = disc->user;
+    v.integer = state->argc;
+    break;
+  }
+  default:
+    exerror("unknown array name: %s", rhs->name);
+    break;
+  }
+  return v;
+}
+
+/// gvpr’s custom `in` implementation
+///
+/// The main purpose of this is to extend `in` to work on the command line
+/// options array, `ARGV`.
+///
+/// @param lhs The left-hand side operand to `in`.
+/// @param rhs The right-hand side operand to `in`.
+/// @param disc The libexpr discipline.
+/// @return Non-zero if `lhs` was found to be within `rhs`.
+static int in(Extype_t lhs, Exid_t *rhs, Exdisc_t *disc) {
+  switch (rhs->index) {
+  case A_ARGV: {
+    Gpr_t *const state = disc->user;
+    return lhs.integer >= 0 && lhs.integer < state->argc;
+  }
+  default:
+    exerror("unknown array name: %s", rhs->name);
+    break;
+  }
+  return 0;
+}
+
 static int codePhase;
 
 #define haveGraph (1 <= codePhase && codePhase <= 4)
@@ -1662,7 +1644,6 @@ static tctype typeChk(tctype intype, Exid_t *sym) {
     switch (sym->type) {
     case T_obj:
       rng = YALL;
-      ;
       break;
     case T_node:
       rng = Y(V);
@@ -1736,9 +1717,7 @@ static tctype typeChk(tctype intype, Exid_t *sym) {
   return rng;
 }
 
-/* typeChkExp:
- * Type check variable expression.
- */
+// type check variable expression
 static tctype typeChkExp(Exref_t *ref, Exid_t *sym) {
   tctype ty;
 
@@ -1753,8 +1732,7 @@ static tctype typeChkExp(Exref_t *ref, Exid_t *sym) {
   return typeChk(ty, sym);
 }
 
-/* refval:
- * Called during compilation for uses of references:   abc.x
+/* Called during compilation for uses of references:   abc.x
  * Also for abc.f(..),  type abc.v, "abc".x and CONSTANTS.
  * The grammar has been  altered to disallow the first 3.
  * Type check expressions; return value unused.
@@ -1821,8 +1799,7 @@ static Extype_t refval(Expr_t *pgm, Exnode_t *node, Exid_t *sym, Exref_t *ref) {
   return v;
 }
 
-/* binary:
- * Evaluate (l ex->op r) producing a value of type ex->type,
+/* Evaluate (l ex->op r) producing a value of type ex->type,
  * stored in l.
  * May be unary, with r = NULL
  * Return -1 if operation cannot be done, 0 otherwise.
@@ -1940,8 +1917,6 @@ static int binary(Exnode_t *l, Exnode_t *ex, Exnode_t *r, int arg) {
   return ret;
 }
 
-/* strToTvtype:
- */
 static int strToTvtype(char *s) {
   int rt = 0;
   char *sfx;
@@ -1981,8 +1956,6 @@ static int strToTvtype(char *s) {
   return rt;
 }
 
-/* tvtypeToStr:
- */
 static char *tvtypeToStr(long long v) {
   char *s = 0;
 
@@ -2033,8 +2006,7 @@ static char *tvtypeToStr(long long v) {
   return s;
 }
 
-/* stringOf:
- * Convert value x to type string.
+/* Convert value x to type string.
  * Assume x does not have a built-in type
  * Return -1 if conversion cannot be done, 0 otherwise.
  * If arg is != 0, conversion unnecessary; just report possibility.
@@ -2065,8 +2037,7 @@ static int stringOf(Expr_t *prog, Exnode_t *x, int arg) {
   return rv;
 }
 
-/* convert:
- * Convert value x of type x->type to type type.
+/* Convert value x of type x->type to type type.
  * Return -1 if conversion cannot be done, 0 otherwise.
  * If arg is != 0, conversion unnecessary; just report possibility.
  * In particular, assume x != 0 if arg == 0.
@@ -2144,8 +2115,7 @@ static int convert(Exnode_t *x, long type, int arg) {
   return ret;
 }
 
-/* keyval;
- * Calculate unique key for object.
+/* Calculate unique key for object.
  * We use this to unify local copies of nodes and edges.
  */
 static Extype_t keyval(Extype_t v, long type) {
@@ -2155,15 +2125,11 @@ static Extype_t keyval(Extype_t v, long type) {
   return v;
 }
 
-/* a2t:
- * Convert type indices to symbolic name.
- */
+// convert type indices to symbolic name
 static int a2t[] = {0,      FLOATING, INTEGER, STRING,
                     T_node, T_edge,   T_graph, T_obj};
 
-/* initDisc:
- * Create and initialize expr discipline.
- */
+// create and initialize expr discipline
 static Exdisc_t *initDisc(Gpr_t *state) {
   Exdisc_t *dp = calloc(1, sizeof(Exdisc_t));
   if (!dp) {
@@ -2186,6 +2152,8 @@ static Exdisc_t *initDisc(Gpr_t *state) {
   dp->getf = getval;
   dp->reff = refval;
   dp->setf = setval;
+  dp->lengthf = length;
+  dp->inf = in;
   dp->exitf = state->exitf;
   dp->types = a2t;
   dp->user = state;
@@ -2195,8 +2163,7 @@ static Exdisc_t *initDisc(Gpr_t *state) {
   return dp;
 }
 
-/* compile:
- * Compile given string, then extract and return
+/* Compile given string, then extract and return
  * typed expression.
  */
 static Exnode_t *compile(Expr_t *prog, char *src, char *input, int line,
@@ -2233,9 +2200,7 @@ static Exnode_t *compile(Expr_t *prog, char *src, char *input, int line,
   return e;
 }
 
-/* checkGuard:
- * Check if guard is an assignment and warn.
- */
+// check if guard is an assignment and warn
 static void checkGuard(Exnode_t *gp, char *src, int line) {
   gp = exnoncast(gp);
   if (gp && exisAssign(gp)) {
@@ -2246,8 +2211,6 @@ static void checkGuard(Exnode_t *gp, char *src, int line) {
   }
 }
 
-/* mkStmts:
- */
 static case_stmt *mkStmts(Expr_t *prog, char *src, case_infos_t cases,
                           const char *lbl) {
   agxbuf tmp = {0};
@@ -2347,9 +2310,7 @@ finishBlk:
   return has_begin_g || bp->does_walk_graph;
 }
 
-/* doFlags:
- * Convert command line flags to actions in END_G.
- */
+// convert command line flags to actions in END_G
 static const char *doFlags(compflags_t flags) {
   if (flags.srcout) {
     if (flags.induce) {
@@ -2363,15 +2324,13 @@ static const char *doFlags(compflags_t flags) {
   return "\n";
 }
 
-/* compileProg:
- * Convert gpr sections in libexpr program.
- */
+// convert gpr sections in libexpr program
 comp_prog *compileProg(parse_prog *inp, Gpr_t *state, compflags_t flags) {
   const char *endg_sfx = NULL;
   bool uses_graph = false;
 
   /* Make sure we have enough bits for types */
-  assert(BITS_PER_BYTE * sizeof(tctype) >= (1 << TBITS));
+  assert(CHAR_BIT * sizeof(tctype) >= (1 << TBITS));
 
   comp_prog *p = calloc(1, sizeof(comp_prog));
   if (!p) {
@@ -2452,7 +2411,7 @@ void freeCompileProg(comp_prog *p) {
   if (!p)
     return;
 
-  exclose(p->prog, 1);
+  exclose(p->prog);
   for (size_t i = 0; i < p->n_blocks; i++) {
     bp = p->blocks + i;
     free(bp->node_stmts);
@@ -2462,17 +2421,11 @@ void freeCompileProg(comp_prog *p) {
   free(p);
 }
 
-/* readG:
- * Read graph from file and initialize
+/* Read graph from file and initialize
  * dynamic data.
  */
 Agraph_t *readG(FILE *fp) {
-  Agraph_t *g;
-
-#ifdef _WIN32
-  gprDisc.id = &AgIdDisc;
-#endif
-  g = agread(fp, &gprDisc);
+  Agraph_t *g = agread(fp, NULL);
   if (g) {
     aginit(g, AGRAPH, UDATA, sizeof(gdata), false);
     aginit(g, AGNODE, UDATA, sizeof(ndata), false);
@@ -2481,24 +2434,15 @@ Agraph_t *readG(FILE *fp) {
   return g;
 }
 
-/* openG:
- * Open graph and initialize dynamic data.
- */
+// open graph and initialize dynamic data
 Agraph_t *openG(char *name, Agdesc_t desc) {
-  Agraph_t *g;
-
-#ifdef _WIN32
-  gprDisc.id = &AgIdDisc;
-#endif
-  g = agopen(name, desc, &gprDisc);
+  Agraph_t *g = agopen(name, desc, NULL);
   if (g)
     agbindrec(g, UDATA, sizeof(gdata), false);
   return g;
 }
 
-/* openSubg:
- * Open subgraph and initialize dynamic data.
- */
+// open subgraph and initialize dynamic data
 Agraph_t *openSubg(Agraph_t *g, char *name) {
   Agraph_t *sg;
 
@@ -2508,9 +2452,7 @@ Agraph_t *openSubg(Agraph_t *g, char *name) {
   return sg;
 }
 
-/* openNode:
- * Create node and initialize dynamic data.
- */
+// create node and initialize dynamic data
 Agnode_t *openNode(Agraph_t *g, char *name) {
   Agnode_t *np;
 
@@ -2520,9 +2462,7 @@ Agnode_t *openNode(Agraph_t *g, char *name) {
   return np;
 }
 
-/* openEdge:
- * Create edge and initialize dynamic data.
- */
+// create edge and initialize dynamic data
 Agedge_t *openEdge(Agraph_t *g, Agnode_t *t, Agnode_t *h, char *key) {
   Agedge_t *ep;
   Agraph_t *root;
