@@ -3,22 +3,24 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
+#pragma once
+
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-#ifndef GPRSTATE_H
-#define GPRSTATE_H
-
 #include <sfio/sfio.h>
 #include "cgraph.h"
 #include <ast/ast.h>
-#include <vmalloc/vmalloc.h>
+#include <setjmp.h>
+#include <stdbool.h>
+#include <stddef.h>
+#include <stdio.h>
 #include <expr/expr.h>
 #include "gvpr.h"
 
@@ -29,28 +31,18 @@ extern "C" {
                    TV_prepostdfs, TV_prepostfwd, TV_prepostrev,
     } trav_type;
 
-/* Bits for flags variable. 
- */
-  /* If set, gvpr calls exit() on errors */
-#define GV_USE_EXIT 1    
-  /* If set, gvpr stores output graphs in gvpropts */
-#define GV_USE_OUTGRAPH 2
-#define GV_USE_JUMP 4
-
     typedef struct {
 	Agraph_t *curgraph;
 	Agraph_t *nextgraph;
 	Agraph_t *target;
 	Agraph_t *outgraph;
 	Agobj_t *curobj;
-	Sfio_t *tmp;
 	Exdisc_t *dp;
 	Exerror_f errf;
 	Exexit_f exitf;
 	char *tgtname;
 	char *infname;
-	Sfio_t *outFile;
-	Agiodisc_t* dfltIO;
+	FILE *outFile;
 	trav_type tvt;
 	Agnode_t *tvroot;
 	Agnode_t *tvnext;
@@ -60,11 +52,12 @@ extern "C" {
 	char **argv;
 	int flags;
 	gvprbinding* bindings;
-	int n_bindings;
+	size_t n_bindings;
+	jmp_buf jbuf;
     } Gpr_t;
 
     typedef struct {
-	Sfio_t *outFile;
+	FILE *outFile;
 	int argc;
 	char **argv;
 	Exerror_f errf;
@@ -76,10 +69,8 @@ extern "C" {
     extern void addBindings(Gpr_t* state, gvprbinding*);
     extern gvprbinding* findBinding(Gpr_t* state, char*);
     extern void closeGPRState(Gpr_t* state);
-    extern void initGPRState(Gpr_t *, Vmalloc_t *);
-    extern int validTVT(int);
-
-#endif
+    extern void initGPRState(Gpr_t *);
+    extern bool validTVT(long long);
 
 #ifdef __cplusplus
 }
