@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -15,24 +15,33 @@ extern "C" {
 #endif
 
 #include <neatogen/defs.h>
+#include <stdbool.h>
+
+#if !defined(__CYGWIN__) && defined(__GNUC__) && !defined(__MINGW32__)
+#define INTERNAL __attribute__((visibility("hidden")))
+#else
+#define INTERNAL /* nothing */
+#endif
 
 /*************************
  * C.G. method - SPARSE  *
  ************************/
 
-    extern int conjugate_gradient(vtx_data *, double *, double *, int,
+INTERNAL int conjugate_gradient(vtx_data *, double *, double *, int,
 				   double, int);
 
 /*************************
  * C.G. method - DENSE   *
  ************************/
 
-    extern int conjugate_gradient_f(float **, double *, double *, int,
-				     double, int, boolean);
+INTERNAL int conjugate_gradient_f(float **, double *, double *, int,
+				     double, int, bool);
 
-    extern int conjugate_gradient_mkernel(float *, float *, float *, int,
+INTERNAL int conjugate_gradient_mkernel(float *, float *, float *, int,
 					   double, int);
 
+#undef INTERNAL
+
 #ifdef __cplusplus
 }
 #endif
