@@ -3,21 +3,22 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
 
 #include "config.h"
-#include <cgraph/alloc.h>
-#include <cgraph/list.h>
 #include <common/geom.h>
 #include <math.h>
 #include <neatogen/neato.h>
 #include <neatogen/adjust.h>
 #include <stddef.h>
 #include <stdbool.h>
+#include <util/alloc.h>
+#include <util/itos.h>
+#include <util/list.h>
 
 /* For precision, scale up before algorithms, then scale down */
 #define SCALE 10   
@@ -36,11 +37,9 @@ typedef struct nitem {
 typedef int (*distfn) (box *, box *);
 typedef int (*intersectfn) (nitem *, nitem *);
 
-static int cmpitem(Dt_t * d, int *p1, int *p2, Dtdisc_t * disc)
-{
-    (void)d;
-    (void)disc;
-
+static int cmpitem(void *item1, void *item2) {
+    const int *p1 = item1;
+    const int *p2 = item2;
     if (*p1 < *p2) {
         return -1;
     }
@@ -56,8 +55,7 @@ static Dtdisc_t constr = {
     offsetof(nitem, link),
     NULL,
     NULL,
-    (Dtcompar_f) cmpitem,
-    NULL,
+    cmpitem,
 };
 
 static int distY(box * b1, box * b2)
@@ -374,8 +372,7 @@ static graph_t *mkConstraintG(Dt_t * list, intersectfn intersect, distfn dist) {
 
 static void closeGraph(graph_t * cg)
 {
-    node_t *n;
-    for (n = agfstnode(cg); n; n = agnxtnode(cg, n)) {
+    for (node_t *n = agfstnode(cg); n; n = agnxtnode(cg, n)) {
 	free_list(ND_in(n));
 	free_list(ND_out(n));
     }
@@ -445,17 +442,14 @@ static void constrainY(graph_t* g, nitem* nlist, int nnodes, intersectfn ifn,
     rank(cg, 2, INT_MAX);
 #ifdef DEBUG
     {
-	Agsym_t *mlsym = agattr(cg, AGEDGE, "minlen", "");
-	Agsym_t *rksym = agattr(cg, AGNODE, "rank", "");
-	char buf[100];
+	Agsym_t *mlsym = agattr_text(cg, AGEDGE, "minlen", "");
+	Agsym_t *rksym = agattr_text(cg, AGNODE, "rank", "");
 	node_t *n;
 	edge_t *e;
 	for (n = agfstnode(cg); n; n = agnxtnode(cg, n)) {
-	    snprintf(buf, sizeof(buf), "%d", ND_rank(n));
-	    agxset(n, rksym, buf);
+	    agxset(n, rksym, ITOS(ND_rank(n)));
 	    for (e = agfstedge(cg, n); e; e = agnxtedge(cg, e, n)) {
-		snprintf(buf, sizeof(buf), "%d", ED_minlen(e));
-		agxset(e, mlsym, buf);
+		agxset(e, mlsym, ITOS(ED_minlen(e)));
 	    }
 	}
     }
@@ -541,7 +535,8 @@ static void initItem(node_t * n, nitem * p, expand_t margin)
  * we require relative positions to remain the same. That is, if two nodes
  * have the same x originally, they have the same x at the end, and if one
  * node is to the left of another, it remains to the left. In addition, if
- * two nodes could overlap by moving their X coordinates, we insert a constraint * to keep the two nodes sufficiently apart. Similarly, for Y.
+ * two nodes could overlap by moving their X coordinates, we insert a constraint
+ * to keep the two nodes sufficiently apart. Similarly, for Y.
  * 
  * mode = AM_ORTHOXY => first X, then Y
  * mode = AM_ORTHOYX => first Y, then X
@@ -553,11 +548,6 @@ static void initItem(node_t * n, nitem * p, expand_t margin)
  * but leave it for the Y pass to remove any remaining overlaps. Without this,
  * the X pass will remove all overlaps, and the Y pass only compresses in the
  * Y direction, causing a skewing of the aspect ratio.
- * 
- * mode = AM_ORTHOXY => first X, then Y
- * mode = AM_ORTHOYX => first Y, then X
- * mode = AM_ORTHO   => first X, then Y
- * mode = AM_ORTHO_YX   => first Y, then X
  */
 int cAdjust(graph_t * g, int mode)
 {
@@ -589,9 +579,11 @@ int cAdjust(graph_t * g, int mode)
 	case AM_ORTHO :
 	    constrainX(g, nlist, nnodes, intersectY0, 1);
 	    constrainY(g, nlist, nnodes, intersectX, 1);
+	    break;
 	case AM_ORTHO_YX :
 	    constrainY(g, nlist, nnodes, intersectX0, 1);
 	    constrainX(g, nlist, nnodes, intersectY, 1);
+	    break;
 	case AM_PORTHOXY:
 	    constrainX(g, nlist, nnodes, intersectY, 0);
 	    constrainY(g, nlist, nnodes, intersectX, 0);
@@ -633,10 +625,9 @@ typedef struct {
     node_t *np;
 } info;
 
-typedef int (*sortfn_t) (const void *, const void *);
-
-static int sortf(pointf * p, pointf * q)
-{
+static int sortf(const void *x, const void *y) {
+    const pointf *p = x;
+    const pointf *q = y;
     if (p->x < q->x)
 	return -1;
     else if (p->x > q->x)
@@ -731,7 +722,7 @@ static pointf computeScaleXY(pointf *aarr, size_t m) {
 
     aarr[0].x = 1;
     aarr[0].y = HUGE_VAL;
-    qsort(aarr + 1, m - 1, sizeof(pointf), (sortfn_t)sortf);
+    qsort(aarr + 1, m - 1, sizeof(pointf), sortf);
 
     pointf *barr = gv_calloc(m, sizeof(pointf));
     barr[m - 1].x = aarr[m - 1].x;
@@ -842,7 +833,7 @@ int scAdjust(graph_t * g, int equal)
 	assert(nnodes >= 0);
 	aarr = mkOverlapSet(nlist, (size_t)nnodes, &m);
 
-	if (m == 0) {		/* no overlaps */
+	if (m == 1) { // no overlaps
 	    free(aarr);
 	    free(nlist);
 	    return 0;
