@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -15,16 +15,15 @@
 #include <stdlib.h>
 #include <string.h>
 #include <math.h>
-#include <cgraph/alloc.h>
 #include <cgraph/cgraph.h>     /* for agerr() and friends */
 #include <neatogen/delaunay.h>
-#include <common/memory.h>
+#include <util/alloc.h>
+#include <util/sort.h>
 
 #ifdef HAVE_GTS
 #include <gts.h>
 
-static gint triangle_is_hole(void *triangle, void *ignored)
-{
+static int triangle_is_hole(void *triangle, void *ignored) {
     GtsTriangle *t = triangle;
     (void)ignored;
 
@@ -41,8 +40,7 @@ static gint triangle_is_hole(void *triangle, void *ignored)
     return FALSE;
 }
 
-static guint delaunay_remove_holes(GtsSurface * surface)
-{
+static unsigned delaunay_remove_holes(GtsSurface *surface) {
     return gts_surface_foreach_face_remove(surface, triangle_is_hole, NULL);
 }
 
@@ -60,25 +58,20 @@ typedef struct {
     GtsVertexClass parent_class;
 } GVertexClass;
 
-static GVertexClass *g_vertex_class(void)
-{
+static GtsVertexClass *g_vertex_class(void) {
     static GVertexClass *klass = NULL;
 
     if (klass == NULL) {
 	GtsObjectClassInfo vertex_info = {
-	    "GVertex",
-	    sizeof(GVertex),
-	    sizeof(GVertexClass),
-	    (GtsObjectClassInitFunc) NULL,
-	    (GtsObjectInitFunc) NULL,
-	    (GtsArgSetFunc) NULL,
-	    (GtsArgGetFunc) NULL
+	    .name = "GVertex",
+	    .object_size = sizeof(GVertex),
+	    .class_size = sizeof(GVertexClass),
 	};
 	klass = gts_object_class_new(GTS_OBJECT_CLASS(gts_vertex_class()),
 				     &vertex_info);
     }
 
-    return klass;
+    return &klass->parent_class;
 }
 
 typedef struct {
@@ -90,25 +83,20 @@ typedef struct {
     GtsFaceClass parent_class;
 } GFaceClass;
 
-static GFaceClass *g_face_class(void)
-{
+static GtsFaceClass *g_face_class(void) {
     static GFaceClass *klass = NULL;
 
     if (klass == NULL) {
 	GtsObjectClassInfo face_info = {
-	    "GFace",
-	    sizeof(GFace),
-	    sizeof(GFaceClass),
-	    (GtsObjectClassInitFunc) NULL,
-	    (GtsObjectInitFunc) NULL,
-	    (GtsArgSetFunc) NULL,
-	    (GtsArgGetFunc) NULL
+	    .name = "GFace",
+	    .object_size = sizeof(GFace),
+	    .class_size = sizeof(GFaceClass),
 	};
 	klass = gts_object_class_new(GTS_OBJECT_CLASS(gts_face_class()),
 				     &face_info);
     }
 
-    return klass;
+    return &klass->parent_class;
 }
 
 /* destroy:
@@ -150,7 +138,7 @@ tri(double *x, double *y, int npt, int *segs, int nsegs, int sepArr)
     GSList *list = NULL;
     GtsVertex *v1, *v2, *v3;
     GtsTriangle *t;
-    GtsVertexClass *vcl = (GtsVertexClass *) g_vertex_class();
+    GtsVertexClass *vcl = g_vertex_class();
     GtsEdgeClass *ecl = GTS_EDGE_CLASS (gts_constraint_class ());
 
     if (sepArr) {
@@ -185,10 +173,8 @@ tri(double *x, double *y, int npt, int *segs, int nsegs, int sepArr)
 
     gts_triangle_vertices(t, &v1, &v2, &v3);
 
-    surface = gts_surface_new(gts_surface_class(),
-				  (GtsFaceClass *) g_face_class(),
-				  gts_edge_class(),
-				  gts_vertex_class());
+    surface = gts_surface_new(gts_surface_class(), g_face_class(),
+                              gts_edge_class(), gts_vertex_class());
     gts_surface_add_face(surface, gts_face_new(gts_face_class(),
 					       t->e1, t->e2, t->e3));
 
@@ -199,7 +185,7 @@ tri(double *x, double *y, int npt, int *segs, int nsegs, int sepArr)
 	/* if v != NULL, it is a previously added pt with the same
 	 * coordinates as v4, in which case we replace v4 with v
 	 */
-	if (v) {
+	if (v && v4 != v) {
 	    gts_vertex_replace(v4, v);
 	}
     }
@@ -230,7 +216,7 @@ typedef struct {
     v_data *delaunay;
 } estats;
     
-static gint cnt_edge(void *edge, void *stats) {
+static int cnt_edge(void *edge, void *stats) {
     GtsSegment *e = edge;
     estats *sp = stats;
 
@@ -249,7 +235,7 @@ edgeStats (GtsSurface* s, estats* sp)
     gts_surface_foreach_edge(s, cnt_edge, sp);
 }
 
-static gint add_edge(void *edge, void *data) {
+static int add_edge(void *edge, void *data) {
     GtsSegment *e = edge;
     v_data *delaunay = data;
 
@@ -300,7 +286,7 @@ typedef struct {
     int* edges;
 } estate;
 
-static gint addEdge(void *edge, void *state) {
+static int addEdge(void *edge, void *state) {
     GtsSegment *e = edge;
     estate *es = state;
 
@@ -314,20 +300,16 @@ static gint addEdge(void *edge, void *state) {
     return 0;
 }
 
-// when moving to C11, qsort_s should be used instead of having a global
-// variable
-static double* _vals;
-typedef int (*qsort_cmpf) (const void *, const void *);
-
-static int 
-vcmp (int* a, int* b)
-{
+static int vcmp(const void *x, const void *y, void *values) {
+    const int *a = x;
+    const int *b = y;
+    const double *_vals = values;
     double va = _vals[*a];
     double vb = _vals[*b];
 
-    if (va < vb) return -1; 
-    else if (va > vb) return 1; 
-    else return 0;
+    if (va < vb) return -1;
+    if (va > vb) return 1;
+    return 0;
 }
 
 /* delaunay_tri:
@@ -374,11 +356,7 @@ int *delaunay_tri(double *x, double *y, int n, int* pnedges)
 	for (i = 0; i < n; i++)
 	    vs[i] = i;
 
-	if (x[0] == x[1])  /* vertical line */ 
-	    _vals = y;
-	else              
-	    _vals = x;
-	qsort (vs, n, sizeof(int), (qsort_cmpf)vcmp);
+	gv_sort(vs, n, sizeof(int), vcmp, x[0] == x[1] /* vertical line? */ ? y : x);
 
 	tl = vs[0];
 	for (i = 1; i < n; i++) {
@@ -396,7 +374,7 @@ int *delaunay_tri(double *x, double *y, int n, int* pnedges)
     return edges;
 }
 
-static gint cntFace(void *face, void *data) {
+static int cntFace(void *face, void *data) {
     GFace *fp = face;
     int *ip = data;
 
@@ -417,7 +395,7 @@ typedef struct {
     int* neigh;
 } ninfo;
 
-static gint addNeighbor(void *face, void *ni) {
+static int addNeighbor(void *face, void *ni) {
     GFace *f = face;
     ninfo *es = ni;
 
@@ -427,7 +405,7 @@ static gint addNeighbor(void *face, void *ni) {
     return 0;
 }
 
-static gint addFace(void *face, void *state) {
+static int addFace(void *face, void *state) {
     GFace *f = face;
     fstate *es = state;
 
@@ -451,7 +429,7 @@ static gint addFace(void *face, void *state) {
     return 0;
 }
 
-static gint addTri(void *face, void *state) {
+static int addTri(void *face, void *state) {
     GFace *f = face;
     fstate *es = state;
 
@@ -551,231 +529,55 @@ freeSurface (surface_t* s)
     free (s->edges);
     free (s->faces);
     free (s->neigh);
-}
-#elif defined(HAVE_TRIANGLE)
-#define TRILIBRARY
-#include <triangle.c>
-#include <assert.h>
-#include <sparse/general.h>
-
-int*
-get_triangles (double *x, int n, int* tris)
-{
-    struct triangulateio in, mid, vorout;
-    int i;
-
-    if (n <= 2) return NULL;
-
-    in.numberofpoints = n;
-    in.numberofpointattributes = 0;
-    in.pointlist = gv_calloc(in.numberofpoints * 2, sizeof(REAL));
-
-    for (i = 0; i < n; i++){
-	in.pointlist[i*2] = x[i*2];
-	in.pointlist[i*2 + 1] = x[i*2 + 1];
-    }
-    in.pointattributelist = NULL;
-    in.pointmarkerlist = NULL;
-    in.numberofsegments = 0;
-    in.numberofholes = 0;
-    in.numberofregions = 0;
-    in.regionlist = NULL;
-    mid.pointlist = NULL; // Not needed if -N switch used.
-    mid.pointattributelist = NULL;
-    mid.pointmarkerlist = NULL; /* Not needed if -N or -B switch used. */
-    mid.trianglelist = NULL; // Not needed if -E switch used.
-    mid.triangleattributelist = NULL;
-    mid.neighborlist = NULL; // Needed only if -n switch used.
-    mid.segmentlist = NULL;
-    mid.segmentmarkerlist = NULL;
-    mid.edgelist = NULL; // Needed only if -e switch used.
-    mid.edgemarkerlist = NULL; // Needed if -e used and -B not used.
-    vorout.pointlist = NULL; // Needed only if -v switch used.
-    vorout.pointattributelist = NULL;
-    vorout.edgelist = NULL; // Needed only if -v switch used.
-    vorout.normlist = NULL; // Needed only if -v switch used.
-
-    /* Triangulate the points.  Switches are chosen to read and write a  */
-    /*   PSLG (p), preserve the convex hull (c), number everything from  */
-    /*   zero (z), assign a regional attribute to each element (A), and  */
-    /*   produce an edge list (e), a Voronoi diagram (v), and a triangle */
-    /*   neighbor list (n).                                              */
-
-    triangulate("Qenv", &in, &mid, &vorout);
-    assert (mid.numberofcorners == 3);
-
-    *tris = mid.numberoftriangles;
-    
-    free(in.pointlist);
-    free(in.pointattributelist);
-    free(in.pointmarkerlist);
-    free(in.regionlist);
-    free(mid.pointlist);
-    free(mid.pointattributelist);
-    free(mid.pointmarkerlist);
-    free(mid.triangleattributelist);
-    free(mid.neighborlist);
-    free(mid.segmentlist);
-    free(mid.segmentmarkerlist);
-    free(mid.edgelist);
-    free(mid.edgemarkerlist);
-    free(vorout.pointlist);
-    free(vorout.pointattributelist);
-    free(vorout.edgelist);
-    free(vorout.normlist);
-
-    return mid.trianglelist;
-}
-
-// maybe it should be replaced by RNG - relative neighborhood graph, or by GG - gabriel graph
-int* 
-delaunay_tri (double *x, double *y, int n, int* nedges)
-{
-    struct triangulateio in, out;
-    int i;
-
-    in.pointlist = gv_calloc(2 * n, sizeof(REAL));
-    for (i = 0; i < n; i++) {
-	in.pointlist[2 * i] = x[i];
-	in.pointlist[2 * i + 1] = y[i];
-    }
-
-    in.pointattributelist = NULL;
-    in.pointmarkerlist = NULL;
-    in.numberofpoints = n;
-    in.numberofpointattributes = 0;
-    in.trianglearealist = NULL;
-    in.triangleattributelist = NULL;
-    in.numberoftriangleattributes = 0;
-    in.neighborlist = NULL;
-    in.segmentlist = NULL;
-    in.segmentmarkerlist = NULL;
-    in.holelist = NULL;
-    in.numberofholes = 0;
-    in.regionlist = NULL;
-    in.edgelist = NULL;
-    in.edgemarkerlist = NULL;
-    in.normlist = NULL;
-
-    out.pointattributelist = NULL;
-    out.pointmarkerlist = NULL;
-    out.numberofpoints = n;
-    out.numberofpointattributes = 0;
-    out.trianglearealist = NULL;
-    out.triangleattributelist = NULL;
-    out.numberoftriangleattributes = 0;
-    out.neighborlist = NULL;
-    out.segmentlist = NULL;
-    out.segmentmarkerlist = NULL;
-    out.holelist = NULL;
-    out.numberofholes = 0;
-    out.regionlist = NULL;
-    out.edgelist = NULL;
-    out.edgemarkerlist = NULL;
-    out.normlist = NULL;
-
-    triangulate("zQNEeB", &in, &out, NULL);
-
-    *nedges = out.numberofedges;
-    free (in.pointlist);
-    free (in.pointattributelist);
-    free (in.pointmarkerlist);
-    free (in.trianglearealist);
-    free (in.triangleattributelist);
-    free (in.neighborlist);
-    free (in.segmentlist);
-    free (in.segmentmarkerlist);
-    free (in.holelist);
-    free (in.regionlist);
-    free (in.edgemarkerlist);
-    free (in.normlist);
-    free (out.pointattributelist);
-    free (out.pointmarkerlist);
-    free (out.trianglearealist);
-    free (out.triangleattributelist);
-    free (out.neighborlist);
-    free (out.segmentlist);
-    free (out.segmentmarkerlist);
-    free (out.holelist);
-    free (out.regionlist);
-    free (out.edgemarkerlist);
-    free (out.normlist);
-    return out.edgelist;
-}
-
-static v_data *delaunay_triangulation(double *x, double *y, int n) {
-    int nedges;
-    int source, dest;
-    int* edgelist = delaunay_tri (x, y, n, &nedges);
-    int i;
-
-    v_data *delaunay = gv_calloc(n, sizeof(v_data));
-    int *edges = gv_calloc(2 * nedges + n, sizeof(int));
-
-    for (i = 0; i < n; i++) {
-	delaunay[i].ewgts = NULL;
-	delaunay[i].nedges = 1;
-    }
-
-    for (i = 0; i < 2 * nedges; i++)
-	delaunay[edgelist[i]].nedges++;
-
-    for (i = 0; i < n; i++) {
-	delaunay[i].edges = edges;
-	edges += delaunay[i].nedges;
-	delaunay[i].edges[0] = i;
-	delaunay[i].nedges = 1;
-    }
-    for (i = 0; i < nedges; i++) {
-	source = edgelist[2 * i];
-	dest = edgelist[2 * i + 1];
-	delaunay[source].edges[delaunay[source].nedges++] = dest;
-	delaunay[dest].edges[delaunay[dest].nedges++] = source;
-    }
-
-    free(edgelist);
-    return delaunay;
-}
-
-surface_t* 
-mkSurface (double *x, double *y, int n, int* segs, int nsegs)
-{
-    agerr (AGERR, "mkSurface not yet implemented using Triangle library\n");
-    assert (0);
-    return 0;
-}
-void 
-freeSurface (surface_t* s)
-{
-    agerr (AGERR, "freeSurface not yet implemented using Triangle library\n");
-    assert (0);
+    free(s);
 }
 #else
 static char* err = "Graphviz built without any triangulation library\n";
 int* get_triangles (double *x, int n, int* tris)
 {
-    agerr(AGERR, "get_triangles: %s\n", err);
+    (void)x;
+    (void)n;
+    (void)tris;
+
+    agerrorf("get_triangles: %s\n", err);
     return 0;
 }
 static v_data *delaunay_triangulation(double *x, double *y, int n) {
-    agerr(AGERR, "delaunay_triangulation: %s\n", err);
+    (void)x;
+    (void)y;
+    (void)n;
+
+    agerrorf("delaunay_triangulation: %s\n", err);
     return 0;
 }
 int *delaunay_tri(double *x, double *y, int n, int* nedges)
 {
-    agerr(AGERR, "delaunay_tri: %s\n", err);
+    (void)x;
+    (void)y;
+    (void)n;
+    (void)nedges;
+
+    agerrorf("delaunay_tri: %s\n", err);
     return 0;
 }
 surface_t* 
 mkSurface (double *x, double *y, int n, int* segs, int nsegs)
 {
-    agerr(AGERR, "mkSurface: %s\n", err);
+    (void)x;
+    (void)y;
+    (void)n;
+    (void)segs;
+    (void)nsegs;
+
+    agerrorf("mkSurface: %s\n", err);
     return 0;
 }
 void 
 freeSurface (surface_t* s)
 {
-    agerr (AGERR, "freeSurface: %s\n", err);
+    (void)s;
+
+    agerrorf("freeSurface: %s\n", err);
 }
 #endif
 
