@@ -3,18 +3,22 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
-#include <cgraph/alloc.h>
+#include <assert.h>
 #include <float.h>
+#include <limits.h>
 #include <neatogen/multispline.h>
 #include <neatogen/delaunay.h>
 #include <neatogen/neatoprocs.h>
 #include <math.h>
 #include <stdbool.h>
+#include <stddef.h>
+#include <util/alloc.h>
+#include <util/gv_math.h>
 
 static bool spline_merge(node_t * n)
 {
@@ -56,11 +60,9 @@ typedef struct {
     int t;
 } item;
 
-static int cmpItem(Dt_t * d, int p1[], int p2[], Dtdisc_t * disc)
-{
-    (void)d;
-    (void)disc;
-
+static int cmpItem(void *item1, void *item2) {
+    const int *p1 = item1;
+    const int *p2 = item2;
     if (p1[0] < p2[0]) return -1;
     if (p1[0] > p2[0]) return 1;
     if (p1[1] < p2[1]) return -1;
@@ -70,7 +72,8 @@ static int cmpItem(Dt_t * d, int p1[], int p2[], Dtdisc_t * disc)
 
 /* newItem:
  */
-static void *newItem(item *objp, Dtdisc_t *disc) {
+static void *newItem(void *p, Dtdisc_t *disc) {
+    item *objp = p;
     item *newp = gv_alloc(sizeof(item));
 
     (void)disc;
@@ -81,28 +84,20 @@ static void *newItem(item *objp, Dtdisc_t *disc) {
     return newp;
 }
 
-static void freeItem(item *obj, Dtdisc_t *disc) {
-    (void)disc;
-    free(obj);
-}
-
 static Dtdisc_t itemdisc = {
     .key = offsetof(item, a),
     .size = 2 * sizeof(int),
     .link = offsetof(item, link),
-    .makef = (Dtmake_f)newItem,
-    .freef = (Dtfree_f)freeItem,
-    .comparf = (Dtcompar_f)cmpItem,
+    .makef = newItem,
+    .freef = free,
+    .comparf = cmpItem,
 };
 
 static void addMap(Dt_t * map, int a, int b, int t)
 {
     item it;
-    int tmp;
     if (a > b) {
-	tmp = a;
-	a = b;
-	b = tmp;
+	SWAP(&a, &b);
     }
     it.a[0] = a;
     it.a[1] = b;
@@ -136,9 +131,7 @@ static int findMap(Dt_t * map, int a, int b)
     item it;
     item *ip;
     if (a > b) {
-	int tmp = a;
-	a = b;
-	b = tmp;
+	SWAP(&a, &b);
     }
     it.a[0] = a;
     it.a[1] = b;
@@ -157,11 +150,9 @@ typedef struct {
     int j;
 } Ipair;
 
-static int cmpIpair(Dt_t * d, int *p1, int *p2, Dtdisc_t * disc)
-{
-    (void)d;
-    (void)disc;
-
+static int cmpIpair(void *pair1, void *pair2) {
+    const int *p1 = pair1;
+    const int *p2 = pair2;
     if (*p1 < *p2) {
         return -1;
     }
@@ -171,7 +162,8 @@ static int cmpIpair(Dt_t * d, int *p1, int *p2, Dtdisc_t * disc)
     return 0;
 }
 
-static void *newIpair(Ipair *objp, Dtdisc_t *disc) {
+static void *newIpair(void *p, Dtdisc_t *disc) {
+    Ipair *objp = p;
     Ipair *newp = gv_alloc(sizeof(Ipair));
 
     (void)disc;
@@ -181,18 +173,13 @@ static void *newIpair(Ipair *objp, Dtdisc_t *disc) {
     return newp;
 }
 
-static void freeIpair(Ipair *obj, Dtdisc_t *disc) {
-    (void)disc;
-    free(obj);
-}
-
 static Dtdisc_t ipairdisc = {
     .key = offsetof(Ipair, i),
     .size = sizeof(int),
     .link = offsetof(Ipair, link),
-    .makef = (Dtmake_f)newIpair,
-    .freef = (Dtfree_f)freeIpair,
-    .comparf = (Dtcompar_f)cmpIpair,
+    .makef = newIpair,
+    .freef = free,
+    .comparf = cmpIpair,
 };
 
 static void vmapAdd(Dt_t * map, int i, int j)
@@ -301,11 +288,12 @@ triPoint(tripoly_t * trip, int vx, pointf v, pointf w, pointf * ip)
 static int ctrlPtIdx(pointf v, Ppoly_t * polys)
 {
     pointf w;
-    int i;
-    for (i = 1; i < polys->pn; i++) {
+    for (size_t i = 1; i < polys->pn; i++) {
 	w = polys->ps[i];
-	if (w.x == v.x && w.y == v.y)
-	    return i;
+	if (w.x == v.x && w.y == v.y) {
+	    assert(i <= INT_MAX);
+	    return (int)i;
+	}
     }
     return -1;
 }
@@ -434,16 +422,16 @@ static pointf triCenter(pointf * pts, int *idxs)
 static boxf bbox(Ppoly_t** obsp, int npoly, int *np)
 {
     boxf bb;
-    int i, j, cnt = 0;
+    int i, cnt = 0;
     pointf p;
     Ppoly_t* obs;
 
-    bb.LL.x = bb.LL.y = MAXDOUBLE;
-    bb.UR.x = bb.UR.y = -MAXDOUBLE;
+    bb.LL.x = bb.LL.y = DBL_MAX;
+    bb.UR.x = bb.UR.y = -DBL_MAX;
 
     for (i = 0; i < npoly; i++) {
 	obs = *obsp++;
-	for (j = 0; j < obs->pn; j++) {
+	for (size_t j = 0; j < obs->pn; j++) {
 	    p = obs->ps[j];
 	    bb.LL.x = fmin(bb.LL.x, p.x);
 	    bb.UR.x = fmax(bb.UR.x, p.x);
@@ -477,7 +465,7 @@ static int *mkTriIndices(surface_t * sf)
 static ipair sharedEdge(int *p, int *q)
 {
     ipair pt;
-    int tmp, p1, p2;
+    int p1, p2;
     p1 = *p;
     p2 = *(p + 1);
     if (p1 == *q) {
@@ -497,9 +485,7 @@ static ipair sharedEdge(int *p, int *q)
     }
 
     if (p1 > p2) {
-	tmp = p1;
-	p1 = p2;
-	p2 = tmp;
+	SWAP(&p1, &p2);
     }
     pt.i = p1;
     pt.j = p2;
@@ -546,8 +532,7 @@ static void freeTriGraph(tgraph * tg)
  * Generate graph with triangles as nodes and an edge iff two triangles
  * share an edge.
  */
-static tgraph *mkTriGraph(surface_t * sf, int maxv, pointf * pts)
-{
+static tgraph *mkTriGraph(surface_t *sf, pointf *pts) {
     tnode *np;
     int j, i, ne = 0;
     int *jp;
@@ -602,11 +587,10 @@ router_t *mkRouter(Ppoly_t** obsp, int npoly)
     boxf bb;
     int npts;
     surface_t *sf;
-    int maxv = 4; /* default max. no. of vertices in an obstacle; set below */
     /* points in obstacle i have indices obsi[i] through obsi[i+1]-1 in pts
      */
     int *obsi = gv_calloc(npoly + 1, sizeof(int));
-    int i, j, ix = 4, six = 0;
+    int i, ix = 4, six = 0;
 
     bb = bbox(obsp, npoly, &npts);
     npts += 4;			/* 4 points of bounding box */
@@ -632,7 +616,7 @@ router_t *mkRouter(Ppoly_t** obsp, int npoly)
     for (i = 0; i < npoly; i++) {
 	obsi[i] = ix;
         obs = *obsp++;
-	for (j = 1; j <= obs->pn; j++) {
+	for (size_t j = 1; j <= obs->pn; j++) {
 	    segs[six++] = ix;
 	    if (j < obs->pn)
 		segs[six++] = ix + 1;
@@ -640,8 +624,6 @@ router_t *mkRouter(Ppoly_t** obsp, int npoly)
 		segs[six++] = obsi[i];
 	    pts[ix++] = obs->ps[j - 1];
 	}
-	if (obs->pn > maxv)
-	    maxv = obs->pn;
     }
     obsi[i] = ix;
 
@@ -663,7 +645,7 @@ router_t *mkRouter(Ppoly_t** obsp, int npoly)
     rtr->tris = mkTriIndices(sf);
     rtr->trimap = mapSegToTri(sf);
     rtr->tn = sf->nfaces;
-    rtr->tg = mkTriGraph(sf, maxv, pts);
+    rtr->tg = mkTriGraph(sf, pts);
 
     freeSurface(sf);
     return rtr;
@@ -675,10 +657,8 @@ router_t *mkRouter(Ppoly_t** obsp, int npoly)
  */
 static void finishEdge(edge_t* e, Ppoly_t spl, int flip) {
     if (flip) {
-	for (int j = 0; j < spl.pn / 2; j++) {
-	    pointf tmp = spl.ps[spl.pn - 1 - j];
-	    spl.ps[spl.pn - 1 - j] = spl.ps[j];
-	    spl.ps[j] = tmp;
+	for (size_t j = 0; j < spl.pn / 2; j++) {
+	    SWAP(&spl.ps[spl.pn - 1 - j], &spl.ps[j]);
 	}
     }
     if (Verbose > 1)
@@ -704,7 +684,7 @@ static void finishEdge(edge_t* e, Ppoly_t spl, int flip) {
  *
  * Otherwise, return p unchanged.
  */
-static Ppoint_t tweakEnd (Ppoly_t poly, int s, Ppoint_t q) {
+static Ppoint_t tweakEnd(Ppoly_t poly, size_t s, Ppoint_t q) {
     Ppoint_t prv, nxt, p;
 
     p = poly.ps[s];
@@ -715,50 +695,43 @@ static Ppoint_t tweakEnd (Ppoly_t poly, int s, Ppoint_t q) {
 	prv = poly.ps[s - 1];
     if (EQPT(q, nxt) || EQPT(q, prv) ){
 	Ppoint_t m;
-	double d;
 	m.x = (nxt.x + prv.x)/2.0 - p.x;
 	m.y = (nxt.y + prv.y)/2.0 - p.y;
-	d = LEN(m.x,m.y);
+	const double d = hypot(m.x, m.y);
 	p.x += 0.1*m.x/d;
 	p.y += 0.1*m.y/d;
     }
     return p;
 }
 
-static void
-tweakPath (Ppoly_t poly, int s, int t, Ppolyline_t pl)
-{
-    pl.ps[0] = tweakEnd (poly, s, pl.ps[1]);
+static void tweakPath(Ppoly_t poly, size_t t, Ppolyline_t pl) {
+    pl.ps[0] = tweakEnd(poly, 0, pl.ps[1]);
     pl.ps[pl.pn-1] = tweakEnd (poly, t, pl.ps[pl.pn-2]);
 }
 
 
 /* genroute:
  * Generate splines for e and cohorts.
- * Edges go from s to t.
+ * Edges go from 0 to t.
  * Return 0 on success.
  */
-static int genroute(tripoly_t * trip, int s, int t, edge_t * e, int doPolyline)
-{
+static int genroute(tripoly_t *trip, int t, edge_t *e, int doPolyline) {
     pointf eps[2];
     Pvector_t evs[2];
     pointf **cpts = NULL;		/* lists of control points */
     Ppoly_t poly;
     Ppolyline_t pl, spl;
-    int i, j;
     Ppolyline_t mmpl;
-    Pedge_t *medges = NULL;
-    int pn;
     int mult = ED_count(e);
     node_t* head = aghead(e);
     int rv = 0;
 
     poly.ps = NULL;
     pl.pn = 0;
-    eps[0].x = trip->poly.ps[s].x, eps[0].y = trip->poly.ps[s].y;
+    eps[0].x = trip->poly.ps[0].x, eps[0].y = trip->poly.ps[0].y;
     eps[1].x = trip->poly.ps[t].x, eps[1].y = trip->poly.ps[t].y;
     if (Pshortestpath(&(trip->poly), eps, &pl) < 0) {
-	agerr(AGWARN, "Could not create control points for multiple spline for edge (%s,%s)\n", agnameof(agtail(e)), agnameof(aghead(e)));
+	agwarningf("Could not create control points for multiple spline for edge (%s,%s)\n", agnameof(agtail(e)), agnameof(aghead(e)));
 	rv = 1;
 	goto finish;
     }
@@ -773,14 +746,15 @@ static int genroute(tripoly_t * trip, int s, int t, edge_t * e, int doPolyline)
 
     if (mult == 1 || Concentrate) {
 	poly = trip->poly;
-	medges = gv_calloc(poly.pn, sizeof(Pedge_t));
-	for (j = 0; j < poly.pn; j++) {
+	Pedge_t *medges = gv_calloc(poly.pn, sizeof(Pedge_t));
+	for (size_t j = 0; j < poly.pn; j++) {
 	    medges[j].a = poly.ps[j];
 	    medges[j].b = poly.ps[(j + 1) % poly.pn];
 	}
-	tweakPath (poly, s, t, pl);
+	assert(t >= 0);
+	tweakPath(poly, (size_t)t, pl);
 	if (Proutespline(medges, poly.pn, pl, evs, &spl) < 0) {
-	    agerr(AGWARN, "Could not create control points for multiple spline for edge (%s,%s)\n", agnameof(agtail(e)), agnameof(aghead(e)));
+	    agwarningf("Could not create control points for multiple spline for edge (%s,%s)\n", agnameof(agtail(e)), agnameof(aghead(e)));
 	    rv = 1;
 	    goto finish;
 	}
@@ -790,14 +764,14 @@ static int genroute(tripoly_t * trip, int s, int t, edge_t * e, int doPolyline)
 	return 0;
     }
     
-    pn = 2 * (pl.pn - 1);
+    const size_t pn = 2 * (pl.pn - 1);
 
     cpts = gv_calloc(pl.pn - 2, sizeof(pointf *));
-    for (i = 0; i < pl.pn - 2; i++) {
+    for (size_t i = 0; i + 2 < pl.pn; i++) {
 	cpts[i] =
 	    mkCtrlPts(t, mult+1, pl.ps[i], pl.ps[i + 1], pl.ps[i + 2], trip);
 	if (!cpts[i]) {
-	    agerr(AGWARN, "Could not create control points for multiple spline for edge (%s,%s)\n", agnameof(agtail(e)), agnameof(aghead(e)));
+	    agwarningf("Could not create control points for multiple spline for edge (%s,%s)\n", agnameof(agtail(e)), agnameof(aghead(e)));
 	    rv = 1;
 	    goto finish;
 	}
@@ -806,17 +780,17 @@ static int genroute(tripoly_t * trip, int s, int t, edge_t * e, int doPolyline)
     poly.ps = gv_calloc(pn, sizeof(pointf));
     poly.pn = pn;
 
-    for (i = 0; i < mult; i++) {
+    for (int i = 0; i < mult; i++) {
 	poly.ps[0] = eps[0];
-	for (j = 1; j < pl.pn - 1; j++) {
+	for (size_t j = 1; j + 1 < pl.pn; j++) {
 	    poly.ps[j] = cpts[j - 1][i];
 	}
 	poly.ps[pl.pn - 1] = eps[1];
-	for (j = 1; j < pl.pn - 1; j++) {
+	for (size_t j = 1; j + 1 < pl.pn; j++) {
 	    poly.ps[pn - j] = cpts[j - 1][i + 1];
 	}
 	if (Pshortestpath(&poly, eps, &mmpl) < 0) {
-	    agerr(AGWARN, "Could not create control points for multiple spline for edge (%s,%s)\n", agnameof(agtail(e)), agnameof(aghead(e)));
+	    agwarningf("Could not create control points for multiple spline for edge (%s,%s)\n", agnameof(agtail(e)), agnameof(aghead(e)));
 	    rv = 1;
 	    goto finish;
 	}
@@ -825,14 +799,16 @@ static int genroute(tripoly_t * trip, int s, int t, edge_t * e, int doPolyline)
 	    make_polyline (mmpl, &spl);
 	}
 	else {
-	    medges = gv_calloc(poly.pn, sizeof(Pedge_t));
-	    for (j = 0; j < poly.pn; j++) {
+	    Pedge_t *medges = gv_calloc(poly.pn, sizeof(Pedge_t));
+	    for (size_t j = 0; j < poly.pn; j++) {
 		medges[j].a = poly.ps[j];
 		medges[j].b = poly.ps[(j + 1) % poly.pn];
 	    }
-	    tweakPath (poly, 0, pl.pn-1, mmpl);
-	    if (Proutespline(medges, poly.pn, mmpl, evs, &spl) < 0) {
-		agerr(AGWARN, "Could not create control points for multiple spline for edge (%s,%s)\n", 
+	    tweakPath(poly, pl.pn - 1, mmpl);
+	    const bool failed_routing = Proutespline(medges, poly.pn, mmpl, evs, &spl) < 0;
+	    free(medges);
+	    if (failed_routing) {
+		agwarningf("Could not create control points for multiple spline for edge (%s,%s)\n", 
 		    agnameof(agtail(e)), agnameof(aghead(e)));
 		rv = 1;
 		goto finish;
@@ -845,11 +821,10 @@ static int genroute(tripoly_t * trip, int s, int t, edge_t * e, int doPolyline)
 
 finish :
     if (cpts) {
-	for (i = 0; i < pl.pn - 2; i++)
+	for (size_t i = 0; i + 2 < pl.pn; i++)
 	    free(cpts[i]);
 	free(cpts);
     }
-    free(medges);
     free(poly.ps);
     return rv;
 }
@@ -952,6 +927,7 @@ static void addEndpoint(router_t * rtr, pointf p, node_t* v, int v_id, int sides
 	    continue;
 	addTriEdge(rtr->tg, v_id, t, seg);
     }
+    assert(rtr->tg->nodes[v_id].ne > 0 && "no edges were added");
 }
 
 /* edgeToSeg:
@@ -981,12 +957,11 @@ static ipair edgeToSeg(tgraph * tg, int i, int j)
 static void
 freeTripoly (tripoly_t* trip)
 {
-    int i;
     tri* tp;
     tri* nxt;
 
     free (trip->poly.ps);
-    for (i = 0; i < trip->poly.pn; i++) {
+    for (size_t i = 0; i < trip->poly.pn; i++) {
 	for (tp = trip->triMap[i]; tp; tp = nxt) {
 	    nxt = tp->nxttri;
 	    free (tp);
@@ -1020,8 +995,8 @@ static tripoly_t *mkPoly(router_t * rtr, int *dad, int s, int t,
     tripoly_t *ps;
     int nxt;
     ipair p;
-    int nt = 0;
-    int i, idx;
+    size_t nt = 0;
+    int idx;
     int cnt1 = 0;
     int cnt2 = 0;
     pointf *pts;
@@ -1029,8 +1004,10 @@ static tripoly_t *mkPoly(router_t * rtr, int *dad, int s, int t,
     Dt_t *vmap;
 
     /* count number of triangles in path */
-    for (nxt = dad[t]; nxt != s; nxt = dad[nxt])
+    for (nxt = dad[t]; nxt != s; nxt = dad[nxt]) {
 	nt++;
+	assert (nxt != dad[nxt] && "infinite loop due to 'nxt' not changing");
+    }
 
     side_t *side1 = gv_calloc(nt + 4, sizeof(side_t));
     side_t *side2 = gv_calloc(nt + 4, sizeof(side_t));
@@ -1093,20 +1070,20 @@ static tripoly_t *mkPoly(router_t * rtr, int *dad, int s, int t,
     tri **trim = gv_calloc(nt + 4, sizeof(tri*));
     *pps++ = p_t;
     idx = 1;
-    for (i = 0; i < cnt1; i++) {
+    for (int i = 0; i < cnt1; i++) {
 	vmapAdd(vmap, side1[i].v, idx);
 	*pps++ = rtr->ps[side1[i].v];
 	trim[idx++] = side1[i].ts;
     }
     *pps++ = p_s;
     idx++;
-    for (i = cnt2 - 1; i >= 0; i--) {
+    for (int i = cnt2 - 1; i >= 0; i--) {
 	vmapAdd(vmap, side2[i].v, idx);
 	*pps++ = rtr->ps[side2[i].v];
 	trim[idx++] = side2[i].ts;
     }
 
-    for (i = 0; i < nt + 4; i++) {
+    for (size_t i = 0; i < nt + 4; i++) {
 	mapTri(vmap, trim[i]);
     }
 
@@ -1200,7 +1177,10 @@ triPath(tgraph * g, int n, int v0, int v1, PQ * pq)
 		if (N_VAL(pq, adjn) == UNSEEN) {
 		    N_VAL(pq, adjn) = d;
 		    N_DAD(adjn) = i;
-		    if (PQinsert(pq, adjn)) return NULL;
+		    if (PQinsert(pq, adjn)) {
+		       free(dad);
+		       return NULL;
+		    }
 		} else if (N_VAL(pq, adjn) < d) {
 		    PQupdate(pq, adjn, d);
 		    N_DAD(adjn) = i;
@@ -1263,7 +1243,7 @@ int makeMultiSpline(edge_t* e, router_t * rtr, int doPolyline) {
 	free(sp);
 
 	/* Generate multiple splines using polygon */
-	ret = genroute(poly, 0, idx, e, doPolyline);
+	ret = genroute(poly, idx, e, doPolyline);
 	freeTripoly (poly);
     }
     else ret = -1;
