@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -32,3 +32,14 @@
 #include	"pathplan.h"
 #include	<neatogen/neatoprocs.h>
 #include	<neatogen/adjust.h>
+
+#if !defined(__CYGWIN__) && defined(__GNUC__) && !defined(__MINGW32__)
+#define INTERNAL __attribute__((visibility("hidden")))
+#else
+#define INTERNAL /* nothing */
+#endif
+
+INTERNAL int lu_decompose(double **a, int n);
+INTERNAL void lu_solve(double *x, double *b, int n);
+
+#undef INTERNAL
