@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -17,21 +17,21 @@ int siteidx;
 Site *bottomsite;
 
 static Freelist sfl;
-static int nvertices;
+static size_t nvertices;
 
-void siteinit()
+void siteinit(void)
 {
     freeinit(&sfl, sizeof(Site));
     nvertices = 0;
 }
 
 
-Site *getsite()
+Site *getsite(void)
 {
     return getfree(&sfl);
 }
 
-double dist(Site * s, Site * t)
+double ngdist(Site * s, Site * t)
 {
     double ans;
     double dx, dy;
@@ -47,12 +47,8 @@ void makevertex(Site * v)
 {
     v->sitenbr = nvertices;
     ++nvertices;
-#ifdef STANDALONE
-    out_vertex(v);
-#endif
 }
 
-
 void deref(Site * v)
 {
     --v->refcnt;
