@@ -9,15 +9,19 @@
  *************************************************************************/
 
 #include "config.h"
+#include <assert.h>
 #include <common/boxes.h>
+#include <limits.h>
 #include <ortho/partition.h>
 #include <ortho/trap.h>
 #include <math.h>
 #include <stdbool.h>
+#include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <util/alloc.h>
 #include <util/bitarray.h>
+#include <util/gv_math.h>
 #include <util/prisize_t.h>
 
 #ifndef DEBUG
@@ -63,7 +67,8 @@ typedef struct {
   int nextfree;
 } vertexchain_t;
 
-static int chain_idx, mon_idx;
+static int chain_idx;
+static size_t mon_idx;
 	/* Table to hold all the monotone */
 	/* polygons . Each monotone polygon */
 	/* is a circularly linked list */
@@ -136,33 +141,30 @@ store (segment_t* seg, int first, pointf* pts)
     return (last+1);
 }
 
-static void
-genSegments (cell* cells, int ncells, boxf bb, segment_t* seg, int flip)
-{
-    int j = 0, i = 1;
+static void genSegments(cell *cells, size_t ncells, boxf bb, segment_t *seg,
+                        int flip) {
+    int i = 1;
     pointf pts[4];
 
     convert (bb, flip, 1, pts);
     i = store (seg, i, pts);
-    for (j = 0; j < ncells; j++) {
+    for (size_t j = 0; j < ncells; j++) {
 	convert (cells[j].bb, flip, 0, pts);
 	i = store (seg, i, pts);
     }
 }
 
 /* Generate a random permutation of the segments 1..n */
-static void 
-generateRandomOrdering(int n, int* permute)
-{
-    int i, j, tmp;
-    for (i = 0; i <= n; i++) permute[i] = i;
+static void generateRandomOrdering(size_t n, int *permute) {
+    for (size_t i = 0; i < n; i++) {
+	assert(i < INT_MAX);
+	permute[i] = (int)i + 1;
+    }
 
-    for (i = 1; i <= n; i++) {
-	j = i + drand48() * (n + 1 - i);
+    for (size_t i = 0; i < n; i++) {
+	const size_t j = (size_t)((double)i + drand48() * (double)(n - i));
 	if (j != i) {
-	    tmp = permute[i];
-	    permute [i] = permute[j];
-            permute [j] = tmp;
+	    SWAP(&permute[i], &permute[j]);
 	}
     }
 }
@@ -173,14 +175,14 @@ inside_polygon (trap_t *t, segment_t* seg)
 {
   int rseg = t->rseg;
 
-  if (t->state == ST_INVALID)
+  if (!t->is_valid)
     return false;
 
   if (t->lseg <= 0 || t->rseg <= 0)
     return false;
   
-  if ((t->u0 <= 0 && t->u1 <= 0) || (t->d0 <= 0 && t->d1 <= 0)) /* triangle */
-    return _greater_than(&seg[rseg].v1, &seg[rseg].v0);
+  if ((!is_valid_trap(t->u0) && !is_valid_trap(t->u1)) || (!is_valid_trap(t->d0) && !is_valid_trap(t->d1))) // triangle
+    return greater_than(seg[rseg].v1, seg[rseg].v0);
   
   return false;
 }
@@ -256,11 +258,9 @@ get_vertex_positions (int v0, int v1, int *ip, int *iq)
  * the current monotone polygon mcur. Split the current polygon into 
  * two polygons using the diagonal (v0, v1) 
  */
-static int 
-make_new_monotone_poly (int mcur, int v0, int v1)
-{
+static size_t make_new_monotone_poly(size_t mcur, int v0, int v1) {
   int p, q, ip, iq;
-  int mnew = newmon();
+  const size_t mnew = newmon();
   int i, j, nf0, nf1;
   vertexchain_t *vp0, *vp1;
   
@@ -305,7 +305,8 @@ make_new_monotone_poly (int mcur, int v0, int v1)
   vp1->nextfree++;
 
 #if DEBUG > 0
-  fprintf(stderr, "make_poly: mcur = %d, (v0, v1) = (%d, %d)\n", mcur, v0, v1);
+  fprintf(stderr, "make_poly: mcur = %" PRISIZE_T ", (v0, v1) = (%d, %d)\n",
+          mcur, v0, v1);
   fprintf(stderr, "next posns = (p, q) = (%d, %d)\n", p, q);
 #endif
 
@@ -316,21 +317,20 @@ make_new_monotone_poly (int mcur, int v0, int v1)
 
 /* recursively visit all the trapezoids */
 static void traverse_polygon(bitarray_t *visited, boxes_t *decomp,
-                             segment_t *seg, traps_t *tr, int mcur, int trnum,
-                             int from, int flip, int dir) {
-  trap_t *t;
-  int mnew;
+                             segment_t *seg, traps_t *tr, size_t mcur,
+                             size_t trnum, size_t from, int flip, int dir) {
+  size_t mnew;
   int v0, v1;
 
-  if (trnum <= 0 || bitarray_get(*visited, (size_t)trnum))
+  if (!is_valid_trap(trnum) || bitarray_get(*visited, trnum))
     return;
 
-  t = &tr->data[trnum];
+  trap_t *t = traps_at(tr, trnum);
 
-  bitarray_set(visited, (size_t)trnum, true);
+  bitarray_set(visited, trnum, true);
   
-  if (t->hi.y > t->lo.y + C_EPS && FP_EQUAL(seg[t->lseg].v0.x, seg[t->lseg].v1.x) &&
-      FP_EQUAL(seg[t->rseg].v0.x, seg[t->rseg].v1.x)) {
+  if (t->hi.y > t->lo.y + C_EPS && fp_equal(seg[t->lseg].v0.x, seg[t->lseg].v1.x) &&
+      fp_equal(seg[t->rseg].v0.x, seg[t->rseg].v1.x)) {
       boxf newbox = {0};
       if (flip) {
           newbox.LL.x = t->lo.y;
@@ -356,11 +356,9 @@ static void traverse_polygon(bitarray_t *visited, boxes_t *decomp,
 
   /* special cases for triangles with cusps at the opposite ends. */
   /* take care of this first */
-  if (t->u0 <= 0 && t->u1 <= 0)
-    {
-      if (t->d0 > 0 && t->d1 > 0) /* downward opening triangle */
-	{
-	  v0 = tr->data[t->d1].lseg;
+  if (!is_valid_trap(t->u0) && !is_valid_trap(t->u1)) {
+      if (is_valid_trap(t->d0) && is_valid_trap(t->d1)) { // downward opening triangle
+	  v0 = traps_get(tr, t->d1).lseg;
 	  v1 = t->lseg;
 	  if (from == t->d1)
 	    {
@@ -385,12 +383,10 @@ static void traverse_polygon(bitarray_t *visited, boxes_t *decomp,
 	}
     }
   
-  else if (t->d0 <= 0 && t->d1 <= 0)
-    {
-      if (t->u0 > 0 && t->u1 > 0) /* upward opening triangle */
-	{
+  else if (!is_valid_trap(t->d0) && !is_valid_trap(t->d1)) {
+      if (is_valid_trap(t->u0) && is_valid_trap(t->u1)) { // upward opening triangle
 	  v0 = t->rseg;
-	  v1 = tr->data[t->u0].rseg;
+	  v1 = traps_get(tr, t->u0).rseg;
 	  if (from == t->u1)
 	    {
 	      mnew = make_new_monotone_poly(mcur, v1, v0);
@@ -414,12 +410,10 @@ static void traverse_polygon(bitarray_t *visited, boxes_t *decomp,
 	}
     }
   
-  else if (t->u0 > 0 && t->u1 > 0)
-    {
-      if (t->d0 > 0 && t->d1 > 0) /* downward + upward cusps */
-	{
-	  v0 = tr->data[t->d1].lseg;
-	  v1 = tr->data[t->u0].rseg;
+  else if (is_valid_trap(t->u0) && is_valid_trap(t->u1)) {
+      if (is_valid_trap(t->d0) && is_valid_trap(t->d1)) { // downward + upward cusps
+	  v0 = traps_get(tr, t->d1).lseg;
+	  v1 = traps_get(tr, t->u0).rseg;
 	  if ((dir == TR_FROM_DN && t->d1 == from) ||
 	      (dir == TR_FROM_UP && t->u1 == from))
 	    {
@@ -440,9 +434,8 @@ static void traverse_polygon(bitarray_t *visited, boxes_t *decomp,
 	}
       else			/* only downward cusp */
 	{
-	  if (_equal_to(&t->lo, &seg[t->lseg].v1))
-	    {
-	      v0 = tr->data[t->u0].rseg;
+	  if (equal_to(t->lo, seg[t->lseg].v1)) {
+	      v0 = traps_get(tr, t->u0).rseg;
 	      v1 = seg[t->lseg].next;
 
 	      if (dir == TR_FROM_UP && t->u0 == from)
@@ -465,7 +458,7 @@ static void traverse_polygon(bitarray_t *visited, boxes_t *decomp,
 	  else
 	    {
 	      v0 = t->rseg;
-	      v1 = tr->data[t->u0].rseg;	
+	      v1 = traps_get(tr, t->u0).rseg;	
 	      if (dir == TR_FROM_UP && t->u1 == from)
 		{
 		  mnew = make_new_monotone_poly(mcur, v1, v0);
@@ -485,13 +478,10 @@ static void traverse_polygon(bitarray_t *visited, boxes_t *decomp,
 	    }
 	}
     }
-  else if (t->u0 > 0 || t->u1 > 0) /* no downward cusp */
-    {
-      if (t->d0 > 0 && t->d1 > 0) /* only upward cusp */
-	{
-	  if (_equal_to(&t->hi, &seg[t->lseg].v0))
-	    {
-	      v0 = tr->data[t->d1].lseg;
+  else if (is_valid_trap(t->u0) || is_valid_trap(t->u1)) { // no downward cusp
+      if (is_valid_trap(t->d0) && is_valid_trap(t->d1)) { // only upward cusp
+	  if (equal_to(t->hi, seg[t->lseg].v0)) {
+	      v0 = traps_get(tr, t->d1).lseg;
 	      v1 = t->lseg;
 	      if (!(dir == TR_FROM_DN && t->d0 == from))
 		{
@@ -512,7 +502,7 @@ static void traverse_polygon(bitarray_t *visited, boxes_t *decomp,
 	    }
 	  else
 	    {
-	      v0 = tr->data[t->d1].lseg;
+	      v0 = traps_get(tr, t->d1).lseg;
 	      v1 = seg[t->rseg].next;
 
 	      if (dir == TR_FROM_DN && t->d1 == from)
@@ -535,9 +525,7 @@ static void traverse_polygon(bitarray_t *visited, boxes_t *decomp,
 	}
       else			/* no cusp */
 	{
-	  if (_equal_to(&t->hi, &seg[t->lseg].v0) &&
-	      _equal_to(&t->lo, &seg[t->rseg].v0))
-	    {
+	  if (equal_to(t->hi, seg[t->lseg].v0) && equal_to(t->lo, seg[t->rseg].v0)) {
 	      v0 = t->rseg;
 	      v1 = t->lseg;
 	      if (dir == TR_FROM_UP)
@@ -557,9 +545,8 @@ static void traverse_polygon(bitarray_t *visited, boxes_t *decomp,
 		  traverse_polygon(visited, decomp, seg, tr, mnew, t->u1, trnum, flip, TR_FROM_DN);
 		}
 	    }
-	  else if (_equal_to(&t->hi, &seg[t->rseg].v1) &&
-		   _equal_to(&t->lo, &seg[t->lseg].v1))
-	    {
+	  else if (equal_to(t->hi, seg[t->rseg].v1) &&
+	           equal_to(t->lo, seg[t->lseg].v1)) {
 	      v0 = seg[t->rseg].next;
 	      v1 = seg[t->lseg].next;
 
@@ -595,18 +582,18 @@ static void
 monotonate_trapezoids(int nsegs, segment_t *seg, traps_t *tr, 
     int flip, boxes_t *decomp) {
     int i;
-    int tr_start;
-    bitarray_t visited = bitarray_new(tr->length);
+    bitarray_t visited = bitarray_new(traps_size(tr));
 
-    mchain = gv_calloc(tr->length, sizeof(monchain_t));
+    mchain = gv_calloc(traps_size(tr), sizeof(monchain_t));
     vert = gv_calloc(nsegs + 1, sizeof(vertexchain_t));
     mon = gv_calloc(nsegs, sizeof(int));
 
   /* First locate a trapezoid which lies inside the polygon */
   /* and which is triangular */
-    for (i = 0; i < tr->length; i++)
-	if (inside_polygon(&tr->data[i], seg)) break;
-    tr_start = i;
+    size_t j;
+    for (j = 0; j < traps_size(tr); j++)
+	if (inside_polygon(traps_at(tr, j), seg)) break;
+    const size_t tr_start = j;
   
   /* Initialise the mon data-structure and start spanning all the */
   /* trapezoids within the polygon */
@@ -627,12 +614,12 @@ monotonate_trapezoids(int nsegs, segment_t *seg, traps_t *tr,
 				/* chain  */
   
   /* traverse the polygon */
-    if (tr->data[tr_start].u0 > 0)
+    if (is_valid_trap(traps_get(tr, tr_start).u0))
 	traverse_polygon(&visited, decomp, seg, tr, 0, tr_start,
-	                 tr->data[tr_start].u0, flip, TR_FROM_UP);
-    else if (tr->data[tr_start].d0 > 0)
+	                 traps_get(tr, tr_start).u0, flip, TR_FROM_UP);
+    else if (is_valid_trap(traps_get(tr, tr_start).d0))
 	traverse_polygon(&visited, decomp, seg, tr, 0, tr_start,
-	                 tr->data[tr_start].d0, flip, TR_FROM_DN);
+	                 traps_get(tr, tr_start).d0, flip, TR_FROM_DN);
   
     bitarray_reset(&visited);
     free (mchain);
@@ -659,11 +646,12 @@ dumpTrap (trap_t* tr, int n)
     int i;
     for (i = 1; i <= n; i++) {
       tr++;
-      fprintf (stderr, "%d : %d %d (%f,%f) (%f,%f) %d %d %d %d\n", i,
-         tr->lseg, tr->rseg, tr->hi.x, tr->hi.y, tr->lo.x, tr->lo.y,
-         tr->u0, tr->u1,  tr->d0, tr->d1);
-      fprintf (stderr, "    %d %d %d %d\n", tr->sink, tr->usave,
-         tr->uside, tr->state);
+      fprintf(stderr, "%d : %d %d (%f,%f) (%f,%f) %" PRISIZE_T " %" PRISIZE_T
+              " %" PRISIZE_T " %" PRISIZE_T "\n", i, tr->lseg, tr->rseg,
+              tr->hi.x, tr->hi.y, tr->lo.x, tr->lo.y, tr->u0, tr->u1, tr->d0,
+              tr->d1);
+      fprintf(stderr, "    %" PRISIZE_T " %" PRISIZE_T " %d %s\n", tr->sink,
+              tr->usave, tr->uside, tr->is_valid ? "valid" : "invalid");
     }
     fprintf (stderr, "====\n");
 }
@@ -674,50 +662,52 @@ dumpSegs (segment_t* sg, int n)
     int i;
     for (i = 1; i <= n; i++) {
       sg++;
-      fprintf (stderr, "%d : (%f,%f) (%f,%f) %d %d %d %d %d\n", i,
-         sg->v0.x, sg->v0.y, sg->v1.x, sg->v1.y,
-         (int)sg->is_inserted, sg->root0,  sg->root1, sg->next, sg->prev);
+      fprintf(stderr, "%d : (%f,%f) (%f,%f) %d %" PRISIZE_T " %" PRISIZE_T
+              " %d %d\n", i, sg->v0.x, sg->v0.y, sg->v1.x, sg->v1.y,
+              (int)sg->is_inserted, sg->root0,  sg->root1, sg->next, sg->prev);
     }
     fprintf (stderr, "====\n");
 }
 #endif
 
-boxf *partition(cell *cells, int ncells, size_t *nrects, boxf bb) {
-    int nsegs = 4*(ncells+1);
+boxf *partition(cell *cells, size_t ncells, size_t *nrects, boxf bb) {
+    const size_t nsegs = 4 * (ncells + 1);
     segment_t* segs = gv_calloc(nsegs + 1, sizeof(segment_t));
-    int* permute = gv_calloc(nsegs + 1, sizeof(int));
+    int* permute = gv_calloc(nsegs, sizeof(int));
 
     if (DEBUG) {
-	fprintf (stderr, "cells = %d segs = %d traps = dynamic\n", ncells, nsegs);
+	fprintf(stderr, "cells = %" PRISIZE_T " segs = %" PRISIZE_T
+	        " traps = dynamic\n", ncells, nsegs);
     }
-    genSegments (cells, ncells, bb, segs, 0);
+    genSegments(cells, ncells, bb, segs, 0);
     if (DEBUG) {
-	fprintf (stderr, "%d\n\n", ncells+1);
-	for (int i = 1; i <= nsegs; i++) {
+	fprintf(stderr, "%" PRISIZE_T "\n\n", ncells + 1);
+	for (size_t i = 1; i <= nsegs; i++) {
 	    if (i%4 == 1) fprintf(stderr, "4\n");
 	    fprintf (stderr, "%f %f\n", segs[i].v0.x, segs[i].v0.y);
 	    if (i%4 == 0) fprintf(stderr, "\n");
 	}
     }
     srand48(173);
-    generateRandomOrdering (nsegs, permute);
-    traps_t hor_traps = construct_trapezoids(nsegs, segs, permute);
+    generateRandomOrdering(nsegs, permute);
+    assert(nsegs <= INT_MAX);
+    traps_t hor_traps = construct_trapezoids((int)nsegs, segs, permute);
     if (DEBUG) {
-	fprintf (stderr, "hor traps = %" PRISIZE_T "\n", hor_traps.length);
+	fprintf(stderr, "hor traps = %" PRISIZE_T "\n", traps_size(&hor_traps));
     }
     boxes_t hor_decomp = {0};
-    monotonate_trapezoids(nsegs, segs, &hor_traps, 0, &hor_decomp);
-    free(hor_traps.data);
+    monotonate_trapezoids((int)nsegs, segs, &hor_traps, 0, &hor_decomp);
+    traps_free(&hor_traps);
 
-    genSegments (cells, ncells, bb, segs, 1);
-    generateRandomOrdering (nsegs, permute);
-    traps_t ver_traps = construct_trapezoids(nsegs, segs, permute);
+    genSegments(cells, ncells, bb, segs, 1);
+    generateRandomOrdering(nsegs, permute);
+    traps_t ver_traps = construct_trapezoids((int)nsegs, segs, permute);
     if (DEBUG) {
-	fprintf (stderr, "ver traps = %" PRISIZE_T "\n", ver_traps.length);
+	fprintf(stderr, "ver traps = %" PRISIZE_T "\n", traps_size(&ver_traps));
     }
     boxes_t vert_decomp = {0};
-    monotonate_trapezoids(nsegs, segs, &ver_traps, 1, &vert_decomp);
-    free(ver_traps.data);
+    monotonate_trapezoids((int)nsegs, segs, &ver_traps, 1, &vert_decomp);
+    traps_free(&ver_traps);
 
     boxes_t rs = {0};
     for (size_t i = 0; i < boxes_size(&vert_decomp); ++i)
