@@ -11,21 +11,25 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
 #pragma once
 
+#include <math.h>
 #include <stdbool.h>
+#include <stddef.h>
+#include <stdint.h>
+#include <util/list.h>
 
 /* Segment attributes */
 
 typedef struct {
   pointf v0, v1;       /* two endpoints */ 
   bool is_inserted;      /* inserted in trapezoidation yet ? */
-  int root0, root1;     /* root nodes in Q */
+  size_t root0, root1;   ///< root nodes in Q
   int next;         /* Next logical segment */
   int prev;         /* Previous segment */
 } segment_t;
@@ -36,22 +40,33 @@ typedef struct {
 typedef struct {
   int lseg, rseg;       /* two adjoining segments */
   pointf hi, lo;       /* max/min y-values */ 
-  int u0, u1;
-  int d0, d1;
-  int sink;         /* pointer to corresponding in Q */
-  int usave, uside;     /* I forgot what this means */
-  int state;
+  size_t u0, u1;
+  size_t d0, d1;
+  size_t sink;          ///< pointer to corresponding in Q
+  size_t usave;         ///< I forgot what this means
+  int uside;            ///< I forgot what this means
+  bool is_valid;
 } trap_t; 
 
-#define ST_VALID 1      /* for trapezium state */
-#define ST_INVALID 2
+/// is the given index a reference to an existing trapezoid?
+///
+/// The indices stored in `trap_t.{u0|u1|d0|d1|usave}` can contain sentinel
+/// values that represent an unset or invalid trapezoid. This function can be
+/// used to determine the validity of these fields.
+static inline bool is_valid_trap(size_t index) {
+  return index != 0 && index != SIZE_MAX;
+}
+
+/// an array of trapezoids
+DEFINE_LIST(traps, trap_t)
 
 #define C_EPS 1.0e-7        /* tolerance value: Used for making */
                 /* all decisions about collinearity or */
                 /* left/right of segment. Decrease */
                 /* this value if the input points are */
                 /* spaced very close together */
-#define FP_EQUAL(s, t) (fabs(s - t) <= C_EPS)
+
+static inline bool fp_equal(double s, double t) { return fabs(s - t) <= C_EPS; }
 
 /**
  * @brief double floating point three-way comparison
@@ -70,10 +85,13 @@ static inline int dfp_cmp(double f1, double f2) {
   return 0;
 }
 
-#define _equal_to(v0,v1) \
-  (FP_EQUAL((v0)->y, (v1)->y) && FP_EQUAL((v0)->x, (v1)->x))
+static inline bool equal_to(pointf v0, pointf v1) {
+  return fp_equal(v0.y, v1.y) && fp_equal(v0.x, v1.x);
+}
 
-#define _greater_than(v0, v1) \
-  (((v0)->y > (v1)->y + C_EPS) ? true : (((v0)->y < (v1)->y - C_EPS) ? false : ((v0)->x > (v1)->x)))
+static inline bool greater_than(pointf v0, pointf v1) {
+  return v0.y > v1.y + C_EPS ? true
+                             : (v0.y < v1.y - C_EPS ? false : v0.x > v1.x);
+}
 
-extern int construct_trapezoids(int, segment_t*, int*, int, trap_t*);
+extern traps_t construct_trapezoids(int, segment_t*, int*);
