@@ -25,14 +25,14 @@ enum {QUAD_TREE_HYBRID_SIZE = 10000};
 enum {QUAD_TREE_NONE = 0, QUAD_TREE_NORMAL, QUAD_TREE_FAST, QUAD_TREE_HYBRID};
 
 typedef struct {
-  double p;/*a negativve real number default to -1. repulsive force = dist^p */
+  double p; ///< a negative real number default to -1. repulsive force = distᵖ
   double K;/* the natural distance. If K < 0, K will be set to the average distance of an edge */
   int multilevels;/* if <=1, single level */
   int max_qtree_level;/* max level of quadtree */
   int maxiter;
   double step;/* initial step size */
   int random_seed;
-  bool random_start : 1; ///< whether to apply SE from a random layout, or from exisiting layout
+  bool random_start : 1; ///< whether to apply SE from a random layout, or from existing layout
   bool adaptive_cooling : 1;
   bool beautify_leaves : 1;
   int smoothing;
