@@ -3,20 +3,20 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
 #define STANDALONE
+#include <math.h>
 #include <sparse/general.h>
 #include <sparse/SparseMatrix.h>
 #include <sparse/clustering.h>
 #include <stdbool.h>
-
+#include <util/alloc.h>
 
 static Multilevel_Modularity_Clustering Multilevel_Modularity_Clustering_init(SparseMatrix A, int level){
-  Multilevel_Modularity_Clustering grid;
   int n = A->n, i, j;
 
   assert(A->type == MATRIX_TYPE_REAL);
@@ -24,31 +24,29 @@ static Multilevel_Modularity_Clustering Multilevel_Modularity_Clustering_init(Sp
 
   if (!A) return NULL;
   assert(A->m == n);
-  grid = MALLOC(sizeof(struct Multilevel_Modularity_Clustering_struct));
+  Multilevel_Modularity_Clustering grid = gv_alloc(sizeof(struct Multilevel_Modularity_Clustering_struct));
   grid->level = level;
   grid->n = n;
   grid->A = A;
   grid->P = NULL;
-  grid->R = NULL;
   grid->next = NULL;
   grid->prev = NULL;
-  grid->delete_top_level_A = FALSE;
-  grid->matching = MALLOC(sizeof(double)*(n));
+  grid->delete_top_level_A = false;
+  grid->matching = gv_calloc(n, sizeof(double));
   grid->deg = NULL;
-  grid->agglomerate_regardless = FALSE;
+  grid->agglomerate_regardless = false;
 
   if (level == 0){
     double modularity = 0;
-    int *ia = A->ia, *ja = A->ja, n = A->n;
+    int *ia = A->ia, *ja = A->ja;
     double deg_total = 0;
-    double *deg, *a = (double*) (A->a);
-    double *indeg;
+    double *deg, *a = A->a;
 
     grid->deg_total = 0.;
-    grid->deg = MALLOC(sizeof(double)*(n));
+    grid->deg = gv_calloc(n, sizeof(double));
     deg = grid->deg;
 
-    indeg = MALLOC(sizeof(double)*n);
+    double *indeg = gv_calloc(n, sizeof(double));
     for (i = 0; i < n; i++){
       deg[i] = 0;
       indeg[i] = 0.;
@@ -58,7 +56,7 @@ static Multilevel_Modularity_Clustering Multilevel_Modularity_Clustering_init(Sp
       }
       deg_total += deg[i];
     }
-    if (deg_total == 0) deg_total = 1;
+    deg_total = fmax(deg_total, 1);
     for (i = 0; i < n; i++){
       modularity += (indeg[i] - deg[i]*deg[i]/deg_total)/deg_total;
     }
@@ -82,7 +80,6 @@ static void Multilevel_Modularity_Clustering_delete(Multilevel_Modularity_Cluste
     }
   }
   SparseMatrix_delete(grid->P);
-  SparseMatrix_delete(grid->R);
   free(grid->matching);
   free(grid->deg);
 
@@ -96,20 +93,17 @@ static Multilevel_Modularity_Clustering Multilevel_Modularity_Clustering_establi
   int n = grid->n, level = grid->level, nc = 0;
   double modularity = 0;
   int *ia = A->ia, *ja = A->ja;
-  double *a;
   double *deg = grid->deg;
-  double *deg_new;
   int i, j, jj, jc, jmax;
-  double inv_deg_total = 1./(grid->deg_total);
-  double *deg_inter, gain;
-  int *mask;
+  double inv_deg_total = 1./ grid->deg_total;
+  double gain;
   double maxgain;
   double total_gain = 0;
   modularity = grid->modularity;
 
-  deg_new = MALLOC(sizeof(double)*n);
-  deg_inter = MALLOC(sizeof(double)*n);
-  mask = MALLOC(sizeof(int)*n);
+  double *deg_new = gv_calloc(n, sizeof(double));
+  double *deg_inter = gv_calloc(n, sizeof(double));
+  int *mask = gv_calloc(n, sizeof(int));
   for (i = 0; i < n; i++) mask[i] = -1;
 
   assert(n == A->n);
@@ -118,7 +112,7 @@ static Multilevel_Modularity_Clustering Multilevel_Modularity_Clustering_establi
   /* gain in merging node i into cluster j is
      deg(i,j)/deg_total - 2*deg(i)*deg(j)/deg_total^2
   */
-  a = (double*) A->a;
+  double *a = A->a;
   for (i = 0; i < n; i++){
     if (matching[i] != UNMATCHED) continue;
     /* accumulate connections between i and clusters */
@@ -186,7 +180,6 @@ static Multilevel_Modularity_Clustering Multilevel_Modularity_Clustering_establi
   if (Verbose) fprintf(stderr,"modularity = %f new modularity = %f level = %d, n = %d, nc = %d, gain = %g\n", modularity, modularity + total_gain, 
 		       level, n, nc, total_gain);
 
-  /* !!!!!!!!!!!!!!!!!!!!!! */
   if (ncluster_target > 0){
     if (nc <= ncluster_target && n >= ncluster_target){
       if (n - ncluster_target > ncluster_target - nc){/* ncluster = nc */
@@ -220,21 +213,20 @@ static Multilevel_Modularity_Clustering Multilevel_Modularity_Clustering_establi
     SparseMatrix_delete(R0);
     P = SparseMatrix_transpose(R);
     B = SparseMatrix_multiply(R, A);
+    SparseMatrix_delete(R);
     if (!B) {
       free(deg_new);
       goto RETURN;
     }
     cA = SparseMatrix_multiply(B, P); 
+    SparseMatrix_delete(B);
     if (!cA) {
       free(deg_new);
       goto RETURN;
     }
-    SparseMatrix_delete(B);
     grid->P = P;
-    grid->R = R;
     level++;
     cgrid = Multilevel_Modularity_Clustering_init(cA, level); 
-    deg_new = REALLOC(deg_new, nc*sizeof(double));
     cgrid->deg = deg_new;
     cgrid->modularity = grid->modularity + total_gain;
     cgrid->deg_total = grid->deg_total;
@@ -244,7 +236,7 @@ static Multilevel_Modularity_Clustering Multilevel_Modularity_Clustering_establi
   } else {
     /* if we want a small number of cluster but right now we have too many, we will force agglomeration */
     if (ncluster_target > 0 && nc > ncluster_target && !(grid->agglomerate_regardless)){
-      grid->agglomerate_regardless = TRUE;
+      grid->agglomerate_regardless = true;
       free(deg_inter);
       free(mask);
       free(deg_new);
@@ -280,13 +272,13 @@ static Multilevel_Modularity_Clustering Multilevel_Modularity_Clustering_new(Spa
 
   grid = Multilevel_Modularity_Clustering_establish(grid, ncluster_target);
 
-  if (A != A0) grid->delete_top_level_A = TRUE;/* be sure to clean up later */
+  if (A != A0) grid->delete_top_level_A = true; // be sure to clean up later
   return grid;
 }
 
 
 static void hierachical_modularity_clustering(SparseMatrix A, int ncluster_target,
-					      int *nclusters, int **assignment, double *modularity, int *flag){
+					      int *nclusters, int **assignment, double *modularity){
   /* find a clustering of vertices by maximize modularity
      A: symmetric square matrix n x n. If real value, value will be used as edges weights, otherwise edge weights are considered as 1.
 
@@ -305,13 +297,10 @@ static void hierachical_modularity_clustering(SparseMatrix A, int ncluster_targe
   Multilevel_Modularity_Clustering grid, cgrid;
   int *matching, i;
   SparseMatrix P;
-  double *u;
   assert(A->m == A->n);
 
   *modularity = 0.;
 
-  *flag = 0;
-
   grid = Multilevel_Modularity_Clustering_new(A, ncluster_target);
 
   /* find coarsest */
@@ -321,7 +310,7 @@ static void hierachical_modularity_clustering(SparseMatrix A, int ncluster_targe
   }
 
   /* project clustering up */
-  u =  MALLOC(sizeof(double)*cgrid->n);
+  double *u = gv_calloc(cgrid->n, sizeof(double));
   for (i = 0; i < cgrid->n; i++) u[i] = (double) (cgrid->matching)[i];
   *nclusters = cgrid->n;
   *modularity = cgrid->modularity;
@@ -338,7 +327,7 @@ static void hierachical_modularity_clustering(SparseMatrix A, int ncluster_targe
   if (*assignment){
     matching = *assignment; 
   } else {
-    matching = MALLOC(sizeof(int)*(grid->n));
+    matching = gv_calloc(grid->n, sizeof(int));
     *assignment = matching;
   }
   for (i = 0; i < grid->n; i++) (matching)[i] = (int) u[i];
@@ -349,8 +338,8 @@ static void hierachical_modularity_clustering(SparseMatrix A, int ncluster_targe
 
 
 
-void modularity_clustering(SparseMatrix A, int inplace, int ncluster_target, int use_value,
-			   int *nclusters, int **assignment, double *modularity, int *flag){
+void modularity_clustering(SparseMatrix A, bool inplace, int ncluster_target,
+			   int *nclusters, int **assignment, double *modularity){
   /* find a clustering of vertices by maximize modularity
      A: symmetric square matrix n x n. If real value, value will be used as edges weights, otherwise edge weights are considered as 1.
      inplace: whether A can e modified. If true, A will be modified by removing diagonal.
@@ -366,8 +355,6 @@ void modularity_clustering(SparseMatrix A, int inplace, int ncluster_target, int
    */
   SparseMatrix B;
 
-  *flag = 0;
-  
   assert(A->m == A->n);
 
   B = SparseMatrix_symmetrize(A, false);
@@ -378,9 +365,9 @@ void modularity_clustering(SparseMatrix A, int inplace, int ncluster_target, int
 
   B = SparseMatrix_remove_diagonal(B);
 
-  if (B->type != MATRIX_TYPE_REAL || !use_value) B = SparseMatrix_set_entries_to_real_one(B);
+  if (B->type != MATRIX_TYPE_REAL) B = SparseMatrix_set_entries_to_real_one(B);
 
-  hierachical_modularity_clustering(B, ncluster_target, nclusters, assignment, modularity, flag);
+  hierachical_modularity_clustering(B, ncluster_target, nclusters, assignment, modularity);
 
   if (B != A) SparseMatrix_delete(B);
 
