@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -56,14 +56,14 @@
 */
 
 #define STANDALONE
-#include <cgraph/alloc.h>
-#include <cgraph/list.h>
 #include <limits.h>
 #include <sparse/general.h>
 #include <sparse/SparseMatrix.h>
 #include <sparse/mq.h>
 #include <stdbool.h>
 #include <string.h>
+#include <util/alloc.h>
+#include <util/list.h>
 
 static double get_mq(SparseMatrix A, int *assignment, int *ncluster0, double *mq_in0, double *mq_out0, double **dout0){
   /* given a symmetric matrix representation of a graph and an assignment of nodes into clusters, calculate the modularity quality.
@@ -166,7 +166,6 @@ static Multilevel_MQ_Clustering Multilevel_MQ_Clustering_init(SparseMatrix A, in
   grid->n = n;
   grid->A = A;
   grid->P = NULL;
-  grid->R = NULL;
   grid->next = NULL;
   grid->prev = NULL;
   grid->delete_top_level_A = false;
@@ -215,7 +214,6 @@ static void Multilevel_MQ_Clustering_delete(Multilevel_MQ_Clustering grid){
     }
   }
   SparseMatrix_delete(grid->P);
-  SparseMatrix_delete(grid->R);
   free(grid->matching);
   free(grid->deg_intra);
   free(grid->dout);
@@ -334,8 +332,8 @@ static Multilevel_MQ_Clustering Multilevel_MQ_Clustering_establish(Multilevel_MQ
 #ifdef DEBUG
       {int ncluster;
 	double mq2, mq_in2, mq_out2, *dout2;
-	int *matching2, nc2 = nc;
-	matching2 = MALLOC(sizeof(int)*A->m);
+	int nc2 = nc;
+	int *matching2 = gv_calloc(A->m, sizeof(int));
 	memcpy(matching2, matching, sizeof(double)*A->m);
 	if (jc != UNMATCHED) {
 	  matching2[i] = jc;
@@ -461,6 +459,7 @@ static Multilevel_MQ_Clustering Multilevel_MQ_Clustering_establish(Multilevel_MQ
     SparseMatrix_delete(R0);
     P = SparseMatrix_transpose(R);
     B = SparseMatrix_multiply(R, A);
+    SparseMatrix_delete(R);
     if (!B) {
         free(deg_intra_new);
         free(wgt_new);
@@ -468,15 +467,14 @@ static Multilevel_MQ_Clustering Multilevel_MQ_Clustering_establish(Multilevel_MQ
         goto RETURN;
     }
     cA = SparseMatrix_multiply(B, P); 
+    SparseMatrix_delete(B);
     if (!cA) {
         free(deg_intra_new);
         free(wgt_new);
         free(dout_new);
         goto RETURN;
     }
-    SparseMatrix_delete(B);
     grid->P = P;
-    grid->R = R;
     level++;
     cgrid = Multilevel_MQ_Clustering_init(cA, level); 
     deg_intra_new = gv_recalloc(deg_intra_new, n, nc, sizeof(double));
