@@ -9,9 +9,6 @@
  *************************************************************************/
 
 #include    <assert.h>
-#include    <cgraph/gv_ctype.h>
-#include    <cgraph/gv_math.h>
-#include    <cgraph/list.h>
 #include    <twopigen/circle.h>
 #include    <inttypes.h>
 #include    <limits.h>
@@ -21,6 +18,9 @@
 #include    <stdlib.h>
 #include    <string.h>
 #include    <util/alloc.h>
+#include    <util/gv_ctype.h>
+#include    <util/gv_math.h>
+#include    <util/list.h>
 #include    <util/streq.h>
 #define DEF_RANKSEP 1.00
 #define UNSET 10.00
