@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -12,7 +12,9 @@
 #include "config.h"
 #include "gdgen_text.h"
 #include <assert.h>
+#include <float.h>
 #include <limits.h>
+#include <stdatomic.h>
 #include <stdbool.h>
 #include <stdlib.h>
 #include <stddef.h>
@@ -28,16 +30,16 @@
 /* for gvcolor_t */
 #include <common/color.h>
 
-#include <cgraph/agxbuf.h>
-#include <cgraph/alloc.h>
 #include <cgraph/cgraph.h>
-#include <cgraph/strview.h>
 #include <common/render.h>
+#include <util/agxbuf.h>
+#include <util/alloc.h>
+#include <util/strview.h>
 
 /* for wind() */
 #include <pathplan/pathutil.h>
 
-typedef enum { FORMAT_VRML, } format_type;
+enum { FORMAT_VRML, };
 
 #define BEZIERSUBDIVISION 10
 
@@ -152,23 +154,23 @@ static int color_index(gdImagePtr im, gvcolor_t color)
 static int set_penstyle(GVJ_t * job, gdImagePtr im, gdImagePtr brush)
 {
     obj_state_t *obj = job->obj;
-    int i, pen, pencolor, transparent, width, dashstyle[40];
+    int i, pen, pencolor, transparent, width, dashstyle[20];
 
     pen = pencolor = color_index(im, obj->pencolor);
     transparent = gdImageGetTransparent(im);
     if (obj->pen == PEN_DASHED) {
-        for (i = 0; i < 20; i++)
+        for (i = 0; i < 10; i++)
             dashstyle[i] = pencolor;
-        for (; i < 40; i++)
+        for (; i < 20; i++)
             dashstyle[i] = transparent;
         gdImageSetStyle(im, dashstyle, 20);
         pen = gdStyled;
     } else if (obj->pen == PEN_DOTTED) {
         for (i = 0; i < 2; i++)
             dashstyle[i] = pencolor;
-        for (; i < 24; i++)
+        for (; i < 12; i++)
             dashstyle[i] = transparent;
-        gdImageSetStyle(im, dashstyle, 24);
+        gdImageSetStyle(im, dashstyle, 12);
         pen = gdStyled;
     }
     width = obj->penwidth * job->scale.x;
@@ -176,7 +178,7 @@ static int set_penstyle(GVJ_t * job, gdImagePtr im, gdImagePtr brush)
         width = PENWIDTH_NORMAL;  /* gd can't do thin lines */
     gdImageSetThickness(im, width);
     /* use brush instead of Thickness to improve end butts */
-    if (width != PENWIDTH_NORMAL) {
+    if (width != (int)PENWIDTH_NORMAL) {
         brush = gdImageCreate(width, width);
         gdImagePaletteCopy(brush, im);
         gdImageFilledRectangle(brush, 0, 0, width - 1, width - 1, pencolor);
@@ -198,7 +200,7 @@ static void vrml_begin_page(GVJ_t *job)
     gvputs(job,   "#VRML V2.0 utf8\n");
 
     state->Saw_skycolor = false;
-    state->MinZ = MAXDOUBLE;
+    state->MinZ = DBL_MAX;
     gvputs(job,   "Group { children [\n"
                   "  Transform {\n");
     gvprintf(job, "    scale %.3f %.3f %.3f\n", .0278, .0278, .0278);
@@ -634,10 +636,9 @@ static void vrml_polygon(GVJ_t *job, pointf *A, size_t np, int filled) {
     case EDGE_OBJTYPE:
 	e = obj->u.e;
 	if (np != 3) {
-	    static int flag;
-	    if (!flag) {
-		flag++;
-		agerr(AGWARN,
+	    static atomic_flag flag;
+	    if (!atomic_flag_test_and_set(&flag)) {
+		agwarningf(
 		  "vrml_polygon: non-triangle arrowheads not supported - ignoring\n");
 	    }
 	}
