@@ -21,11 +21,11 @@
 #include <gvc/gvplugin_device.h>
 #include <gvc/gvio.h>
 #include <gvc/gvcint.h>
-#include <cgraph/agxbuf.h>
-#include <cgraph/prisize_t.h>
-#include <cgraph/unreachable.h>
 #include <common/const.h>
 #include <common/utils.h>
+#include <util/agxbuf.h>
+#include <util/prisize_t.h>
+#include <util/unreachable.h>
 #include "../core/ps.h"
 
 using namespace LASi;
@@ -36,7 +36,7 @@ using namespace std;
 //     from Adobe's Distiller.
 #define PDFMAX  14400 ///< Maximum size of PDF page
 
-typedef enum { FORMAT_PS, FORMAT_PS2, FORMAT_EPS } format_type;
+enum { FORMAT_PS, FORMAT_PS2, FORMAT_EPS };
 
 struct Context {
   using write_fn = size_t(*)(GVJ_t*, const char*, size_t);
